@@ -2,6 +2,7 @@
  This file is part of SLRN.
 
  Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
+ Copyright (c) 2001 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -23,6 +24,7 @@
 extern int slrn_select_article_mode (Slrn_Group_Type *, int, int);
 extern void slrn_init_article_mode (void);
 extern int slrn_insert_followup_format (char *, FILE *);
+extern void slrn_subject_strip_was (char *);
 extern SLKeyMap_List_Type *Slrn_Article_Keymap;
 extern char *Slrn_Quote_String;
 extern char *Slrn_Save_Directory;
@@ -30,6 +32,10 @@ extern char *Slrn_Header_Help_Line;
 extern char *Slrn_Art_Help_Line;
 extern char *Slrn_Followup_Custom_Headers;
 extern char *Slrn_Reply_Custom_Headers;
+extern char *Slrn_Supersedes_Custom_Headers;
+extern char *Slrn_Overview_Date_Format;
+extern char *Slrn_Followup_Date_Format;
+extern int Slrn_Use_Localtime;
 
 extern int Slrn_Simulate_Graphic_Chars;
 
@@ -44,13 +50,17 @@ extern int Slrn_Use_Tmpdir;
 extern int Slrn_Sorting_Mode;
 extern int Slrn_Threads_Visible;
 extern int Slrn_Wrap_Mode;
+extern int Slrn_Wrap_Method;
 extern int Slrn_Use_Header_Numbers;
 extern int Slrn_Reads_Per_Update;
 extern int Slrn_High_Score_Min;
 extern int Slrn_Low_Score_Max;
 extern int Slrn_Kill_Score_Max;
+extern FILE *Slrn_Kill_Log_FP;
 extern int Slrn_Signature_Hidden;
 extern int Slrn_Pgp_Signature_Hidden;
+extern int Slrn_Verbatim_Marks_Hidden;
+extern int Slrn_Warn_Followup_To;
 
 extern char *Slrn_X_Browser;
 
@@ -59,19 +69,27 @@ extern char *Slrn_NonX_Browser;
 extern int Slrn_Spoiler_Char;
 extern int Slrn_Spoiler_Display_Mode;
 #endif
-#if SLRN_HAS_TILDE_FEATURE
 extern int Slrn_Use_Tildes;
-#endif
+extern int Slrn_Generate_Email_From;
 extern int Slrn_Emphasized_Text_Mode;
 extern int Slrn_Emphasized_Text_Mask;
 extern int Slrn_New_Subject_Breaks_Threads;
 extern int Slrn_Sig_Is_End_Of_Article;
 extern int Slrn_Del_Article_Upon_Read;
 extern int Slrn_Followup_Strip_Sig;
-extern int Slrn_Process_Verbatum_Marks;
+extern int Slrn_Process_Verbatim_Marks;
 
 #endif				       /* NOT SLRNPULL_CODE */
 
+typedef struct Slrn_Header_Line_Type
+{
+   char *name;
+   unsigned int name_len;
+   char *value;
+   struct Slrn_Header_Line_Type *next;
+}
+Slrn_Header_Line_Type;
+
 typedef struct Slrn_Header_Type
 {
    struct Slrn_Header_Type *next, *prev;  /* threaded next/prev */
@@ -88,6 +106,7 @@ typedef struct Slrn_Header_Type
 #define FAKE_HEADER_HIGH_SCORE		0x1000
 #define HEADER_CHMAP_PROCESSED		0x2000
 
+#define HEADER_PROCESSED		0x8000
    struct Slrn_Header_Type *real_next, *real_prev;
    struct Slrn_Header_Type *parent, *child, *sister;  /* threaded relatives */
    struct Slrn_Header_Type *hash_next;  /* next in hash table */
@@ -104,23 +123,22 @@ typedef struct Slrn_Header_Type
    char *realname;		       /* NOT Null terminated */
    unsigned int realname_len;
    unsigned int tag_number;
-#define MAX_TREE_SIZE 32
-   unsigned char tree[MAX_TREE_SIZE];
+   Slrn_Header_Line_Type *add_hdrs;
+#define MAX_TREE_SIZE 256
+   char *tree_ptr;
 #if SLRN_HAS_GROUPLENS
    int gl_rating;
    int gl_pred;
 #endif
-#if SLRN_HAS_SORT_BY_SCORE
    int score;
    int thread_score;
-#endif
 } Slrn_Header_Type;
 
 extern Slrn_Header_Type *Slrn_First_Header;
 extern Slrn_Header_Type *Slrn_Current_Header;
 
 extern int slrn_goto_header (Slrn_Header_Type *, int);
-extern int slrn_locate_header_by_msgid (char *, int);
+extern int slrn_locate_header_by_msgid (char *, int, int);
 
 
 typedef struct Slrn_Article_Line_Type
@@ -132,7 +150,8 @@ typedef struct Slrn_Article_Line_Type
 #define QUOTE_LINE		0x0002
 #define SIGNATURE_LINE		0x0004
 #define PGP_SIGNATURE_LINE	0x0008
-#define VERBATUM_LINE		0x0010
+#define VERBATIM_LINE		0x0010
+#define VERBATIM_MARK_LINE	0x0020
 
 #define LINE_ATTRIBUTES_MASK	0x0700
 #define WRAPPED_LINE		0x0100
@@ -160,10 +179,11 @@ typedef struct
    int headers_hidden;
    int signature_hidden;
    int pgp_signature_hidden;
-#if SLRN_HAS_VERBATUM_MARKS
-   int verbatum_hidden;
+#if SLRN_HAS_VERBATIM_MARKS
+   int verbatim_hidden;
+   int verbatim_marks_hidden;
 #endif
-   int needs_sync;		       /* non-zero if line number needs updated */
+   int needs_sync;		       /* non-zero if line number/current line needs updated */
 }
 Slrn_Article_Type;
 
@@ -178,6 +198,9 @@ extern int slrn_is_hidden_headers_mode (void);
 #endif
 
 extern SLRegexp_Type *Slrn_Ignore_Quote_Regexp[];
+extern SLRegexp_Type *Slrn_Strip_Re_Regexp[];
+extern SLRegexp_Type *Slrn_Strip_Sig_Regexp[];
+extern SLRegexp_Type *Slrn_Strip_Was_Regexp[];
 
 extern int slrn_pipe_article_to_cmd (char *);
 extern int slrn_save_current_article (char *);
@@ -198,10 +221,14 @@ extern int slrn_goto_num_tagged_header (int *);
 extern int slrn_next_tagged_header (void);
 extern int slrn_prev_tagged_header (void);
 extern void slrn_set_article_window_size (int);
+extern int slrn_get_article_window_size (void);
 extern char *slrn_extract_header (char *, unsigned int);
 
 /* Third argument must be zero unless caller deals with Slrn_Current_Header. */
 extern Slrn_Header_Type *slrn_set_header_score (Slrn_Header_Type *, int, int);
+extern void slrn_apply_scores (int);
+
+extern void slrn_sort_by_sorting_mode (void);
 
 extern int slrn_is_article_visible (void);
 extern int slrn_is_article_win_zoomed (void);
@@ -212,9 +239,9 @@ int slrn_get_next_pagedn_action (void);
 
 extern int slrn_string_to_article (char *str);
 
-#if SLRN_HAS_SORT_BY_SCORE
 extern int Slrn_Display_Score;
-#endif
+extern int Slrn_Color_By_Score;
+extern int Slrn_Highlight_Unread;
 
 extern int slrn_set_header_format (unsigned int, char *);
 
@@ -230,7 +257,9 @@ extern int _slrn_art_unwrap_article (Slrn_Article_Type *a);
 extern int _slrn_art_wrap_article (Slrn_Article_Type *a);
 extern void _slrn_art_skip_quoted_text (Slrn_Article_Type *a);
 extern int _slrn_art_skip_digest_forward (Slrn_Article_Type *a);
-extern char *slrn_art_extract_header (Slrn_Article_Type *a, char *hdr, unsigned int len);
+extern char *slrn_art_extract_header (char *, unsigned int);
+extern char *slrn_cur_extract_header (char *, unsigned int);
+extern char *slrn_extract_add_header (Slrn_Header_Type *, char *);
 extern void _slrn_art_hide_headers (Slrn_Article_Type *a);
 extern void _slrn_art_unhide_headers (Slrn_Article_Type *a);
 extern int _slrn_art_unfold_header_lines (Slrn_Article_Type *a);
@@ -242,10 +271,10 @@ extern void _slrn_art_hide_pgp_signature (Slrn_Article_Type *a);
 extern void _slrn_art_unhide_pgp_signature (Slrn_Article_Type *a);
 extern void slrn_art_mark_pgp_signature (Slrn_Article_Type *a);
 
-#if SLRN_HAS_VERBATUM_MARKS
-extern void _slrn_art_hide_verbatum (Slrn_Article_Type *a);
-extern void _slrn_art_unhide_verbatum (Slrn_Article_Type *a);
-extern void slrn_art_mark_verbatum (Slrn_Article_Type *a);
+#if SLRN_HAS_VERBATIM_MARKS
+extern void _slrn_art_hide_verbatim (Slrn_Article_Type *a);
+extern void _slrn_art_unhide_verbatim (Slrn_Article_Type *a);
+extern void slrn_art_mark_verbatim (Slrn_Article_Type *a);
 #endif
 
 /* These are in artmisc.c */
