@@ -24,8 +24,9 @@
 #define HAVE_KOI8_CHARMAP       1
 #define HAVE_NEXT_CHARMAP	1
 #define HAVE_IBM850_CHARMAP	1
+#define HAVE_IBM852_CHARMAP	1
 
-#define MAX_CHARMAPS		4      /* add 1 for ISO Latin (default) */
+#define MAX_CHARMAPS		5      /* add 1 for ISO Latin (default) */
 
 typedef struct
 {
@@ -123,6 +124,90 @@ static CharMap_Type IBM850_Char_Map =
 };
 #endif
 
+#if HAVE_IBM852_CHARMAP
+static CharMap_Type IBM852_Char_Map =
+{
+   "ibm852", 128,
+   /* to_iso_map */
+     {
+     /* Conversion table from `cp852' charset to `iso88592' charset.
+      * Generated mechanically by transtab  0.97.
+      */
+	    0,     1,     2,     3,     4,     5,     6,     7,	/* 000-007 */
+	    8,     9,    10,    11,    12,    13,    14,    15,	/* 008-015 */
+	   16,    17,    18,    19,    20,    21,    22,    23,	/* 016-023 */
+	   24,    25,    26,    27,    28,    29,    30,    31,	/* 024-031 */
+	   32,    33,    34,    35,    36,    37,    38,    39,	/* 032-039 */
+	   40,    41,    42,    43,    44,    45,    46,    47,	/* 040-047 */
+	   48,    49,    50,    51,    52,    53,    54,    55,	/* 048-055 */
+	   56,    57,    58,    59,    60,    61,    62,    63,	/* 056-063 */
+	   64,    65,    66,    67,    68,    69,    70,    71,	/* 064-071 */
+	   72,    73,    74,    75,    76,    77,    78,    79,	/* 072-079 */
+	   80,    81,    82,    83,    84,    85,    86,    87,	/* 080-087 */
+	   88,    89,    90,    91,    92,    93,    94,    95,	/* 088-095 */
+	   96,    97,    98,    99,   100,   101,   102,   103,	/* 096-103 */
+	  104,   105,   106,   107,   108,   109,   110,   111,	/* 104-111 */
+	  112,   113,   114,   115,   116,   117,   118,   119,	/* 112-119 */
+	  120,   121,   122,   123,   124,   125,   126,   127,	/* 120-127 */
+	  199,   252,   233,   226,   228,   249,   230,   231,	/* 128-135 */
+	  179,   235,   213,   245,   238,   172,   196,   198,	/* 136-143 */
+	  201,   197,   229,   244,   246,   165,   181,   166,	/* 144-151 */
+	  182,   214,   220,   171,   187,   163,   215,   232,	/* 152-159 */
+	  225,   237,   243,   250,   161,   177,   174,   190,	/* 160-167 */
+	  202,   234,   128,   188,   200,   186,   129,   130,	/* 168-175 */
+	  131,   132,   133,   134,   135,   193,   194,   204,	/* 176-183 */
+	  170,   136,   137,   138,   139,   175,   191,   140,	/* 184-191 */
+	  141,   142,   143,   144,   145,   146,   195,   227,	/* 192-199 */
+	  147,   148,   149,   150,   151,   152,   153,   164,	/* 200-207 */
+	  240,   208,   207,   203,   239,   210,   205,   206,	/* 208-215 */
+	  236,   154,   155,   156,   157,   222,   217,   158,	/* 216-223 */
+	  211,   223,   212,   209,   241,   242,   169,   185,	/* 224-231 */
+	  192,   218,   224,   219,   253,   221,   254,   180,	/* 232-239 */
+	  173,   189,   178,   183,   162,   167,   247,   184,	/* 240-247 */
+	  176,   168,   255,   251,   216,   248,   159,   160	/* 248-255 */
+     },
+   /* from_iso_map */
+     {
+     /* Conversion table from `iso88592' charset to `cp852' charset.
+      * Generated mechanically by transtab  0.97.
+      */
+ 	    0,     1,     2,     3,     4,     5,     6,     7,	/* 000-007 */
+	    8,     9,    10,    11,    12,    13,    14,    15,	/* 008-015 */
+	   16,    17,    18,    19,    20,    21,    22,    23,	/* 016-023 */
+	   24,    25,    26,    27,    28,    29,    30,    31,	/* 024-031 */
+	   32,    33,    34,    35,    36,    37,    38,    39,	/* 032-039 */
+	   40,    41,    42,    43,    44,    45,    46,    47,	/* 040-047 */
+	   48,    49,    50,    51,    52,    53,    54,    55,	/* 048-055 */
+	   56,    57,    58,    59,    60,    61,    62,    63,	/* 056-063 */
+	   64,    65,    66,    67,    68,    69,    70,    71,	/* 064-071 */
+	   72,    73,    74,    75,    76,    77,    78,    79,	/* 072-079 */
+	   80,    81,    82,    83,    84,    85,    86,    87,	/* 080-087 */
+	   88,    89,    90,    91,    92,    93,    94,    95,	/* 088-095 */
+	   96,    97,    98,    99,   100,   101,   102,   103,	/* 096-103 */
+	  104,   105,   106,   107,   108,   109,   110,   111,	/* 104-111 */
+	  112,   113,   114,   115,   116,   117,   118,   119,	/* 112-119 */
+	  120,   121,   122,   123,   124,   125,   126,   127,	/* 120-127 */
+	  170,   174,   175,   176,   177,   178,   179,   180,	/* 128-135 */
+	  185,   186,   187,   188,   191,   192,   193,   194,	/* 136-143 */
+	  195,   196,   197,   200,   201,   202,   203,   204,	/* 144-151 */
+	  205,   206,   217,   218,   219,   220,   223,   254,	/* 152-159 */
+	  255,   164,   244,   157,   207,   149,   151,   245,	/* 160-167 */
+	  249,   230,   184,   155,   141,   240,   166,   189,	/* 168-175 */
+	  248,   165,   242,   136,   239,   150,   152,   243,	/* 176-183 */
+	  247,   231,   173,   156,   171,   241,   167,   190,	/* 184-191 */
+	  232,   181,   182,   198,   142,   145,   143,   128,	/* 192-199 */
+	  172,   144,   168,   211,   183,   214,   215,   210,	/* 200-207 */
+	  209,   227,   213,   224,   226,   138,   153,   158,	/* 208-215 */
+	  252,   222,   233,   235,   154,   237,   221,   225,	/* 216-223 */
+	  234,   160,   131,   199,   132,   146,   134,   135,	/* 224-231 */
+	  159,   130,   169,   137,   216,   161,   140,   212,	/* 232-239 */
+	  208,   228,   229,   162,   147,   139,   148,   246,	/* 240-247 */
+	  253,   133,   163,   251,   129,   236,   238,   250	/* 248-255 */
+
+     }
+};
+#endif
+
 #if HAVE_NEXT_CHARMAP
 static CharMap_Type NeXT_Char_Map =
 {
@@ -294,6 +379,9 @@ static CharMap_Type *Char_Maps [MAX_CHARMAPS] =
 #if HAVE_IBM850_CHARMAP
    &IBM850_Char_Map,
 #endif
+#if HAVE_IBM852_CHARMAP
+   &IBM852_Char_Map,
+#endif
 #if HAVE_NEXT_CHARMAP
    &NeXT_Char_Map,
 #endif
