@@ -3,6 +3,7 @@
  This file is part of SLRN.
 
  Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
+ Copyright (c) 2001 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -116,7 +117,7 @@ void slrn_chmap_fix_body (Slrn_Article_Type *a)
 #endif
 }
 
-int slrn_chmap_fix_file (char *file)
+int slrn_chmap_fix_file (char *file, int reverse)
 {
 #if SLRN_HAS_CHARACTER_MAP
    FILE *fp, *tmpfp;
@@ -151,7 +152,8 @@ int slrn_chmap_fix_file (char *file)
         return -1;
      }
 
-   if (NULL == (tmpfp = slrn_open_tmpfile_in_dir (tmp_file, tmp_file, "w")))
+   if (NULL == (tmpfp = slrn_open_tmpfile_in_dir (tmp_file, tmp_file,
+						  sizeof (tmp_file))))
      {
 	slrn_error ("%s %s %s", File_Error, tmp_file, Not_Posted);
 	fclose (fp);
@@ -161,7 +163,8 @@ int slrn_chmap_fix_file (char *file)
    ret = 0;
    while (NULL != fgets (buf, sizeof (buf), fp))
      {
-	chmap_map_string_to_iso (buf);
+	if (reverse) chmap_map_string_from_iso (buf);
+	else chmap_map_string_to_iso (buf);
 	if (EOF == fputs (buf, tmpfp))
 	  {
 	     slrn_error ("Write Error. Disk Full? %s", Not_Posted);
@@ -194,8 +197,10 @@ void slrn_chmap_show_supported (void)
    CharMap_Type *map;
    unsigned int i;
 
-   fprintf (stderr, "Supported character sets include: ");
-   fprintf (stderr, "isolatin");
+   fprintf (stderr, "  Default character set:     %s\n", DEFAULT_CHARSET_NAME);
+   
+   fprintf (stderr, " SUPPORTED CHARACTER SETS:\n");
+   fprintf (stderr, "  isolatin");
 
    for (i = 0; i < MAX_CHARMAPS; i++)
      {
@@ -204,8 +209,6 @@ void slrn_chmap_show_supported (void)
 	fprintf (stderr, " %s", map->map_name);
      }
    putc ('\n', stdout);
-
-   fprintf (stderr, "Default character set: %s\n", DEFAULT_CHARSET_NAME);
 }
 #endif
 
