@@ -2,6 +2,7 @@
  This file is part of SLRN.
 
  Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
+ Copyright (c) 2001 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -20,7 +21,7 @@
 #ifndef _SLRN_CHMAP_H
 #define _SLRN_CHMAP_H
 extern int slrn_set_charset (char *);
-extern int slrn_chmap_fix_file (char *);
+extern int slrn_chmap_fix_file (char *, int);
 extern void slrn_chmap_fix_body (Slrn_Article_Type *);
 extern void slrn_chmap_fix_headers (void);
 
