@@ -190,6 +190,7 @@ int socket( int, struct sockproto *, struct sockaddr_in *, int );
 
 #include "server.h"
 #include "misc.h"
+#include "snprintf.h"
 
 #ifndef NNTP_PORT
 # define NNTP_PORT 119
@@ -460,7 +461,7 @@ static int get_tcp_socket(char *machine, int nntpport)
    else
      {
 	/* Raw ip address, fake  */
-	(void) strcpy(namebuf, machine);
+	(void) slrn_strncpy (namebuf, machine, sizeof (namebuf));
 	def.h_name = namebuf;
 #   ifdef h_addr
 	def.h_addr_list = alist;
@@ -711,6 +712,8 @@ static int do_netlib_read(void *s, unsigned char *buf, unsigned int len)
 static unsigned char Read_Buffer [BUFSIZE];
 static unsigned char *Read_Buffer_Ptr = Read_Buffer;
 static unsigned char *Read_Buffer_Ptr_Max = Read_Buffer;
+static unsigned long bytes_out;
+static unsigned long bytes_in;
 
 static void reset_read_buffer (void)
 {
@@ -761,6 +764,7 @@ static int client_read (void)
      return -1;
 
    Read_Buffer_Ptr_Max = Read_Buffer + nb;
+   bytes_in += nb;
 
    return 0;
 }
@@ -913,6 +917,7 @@ static unsigned int client_write (char *buf, unsigned int len)
      }
 #endif
 
+   bytes_out += total;
    return total;
 }
 
@@ -929,7 +934,7 @@ static int client_fputs (char *s)
 static int client_vprintf (char *fmt, va_list ap)
 {
    char buf [8192];
-   vsprintf (buf, fmt, ap);
+   slrn_vsnprintf (buf, fmt, ap);
    return client_fputs (buf);
 }
 
@@ -997,8 +1002,14 @@ extern unsigned int sltcp_write (SLTCP_Type *s, char *buf, unsigned int len)
    return client_write (buf, len);
 }
 
-extern SLTCP_Type *sltcp_open_connection (char *host, int port)
+extern SLTCP_Type *sltcp_open_connection (char *host, int port, int use_ssl)
 {
+   if (use_ssl)
+     {
+	fprintf (stderr, "\n\n*** This program does not support SSL\n");
+	return NULL;
+     }
+   
    if (-1 == client_server_init (host, port))
      return NULL;
 
@@ -1022,4 +1033,30 @@ extern int sltcp_close_socket (SLTCP_Type *s)
    return 0;
 }
 
+extern unsigned int sltcp_get_num_input_bytes (SLTCP_Type *tcp)
+{
+   (void) tcp;
+   return bytes_in;
+}
+
+extern unsigned int sltcp_get_num_output_bytes (SLTCP_Type *tcp)
+{
+   (void) tcp;
+   return bytes_out;
+}
+
+extern int sltcp_reset_num_io_bytes (SLTCP_Type *tcp)
+{
+   (void) tcp;
+   bytes_in = bytes_out = 0;
+   return 0;
+}
+
+extern int sltcp_get_fd (SLTCP_Type *tcp)
+{
+   if (tcp == NULL)
+     return -1;
+   return tcp->tcp_fd;
+}
+
 int (*SLTCP_Interrupt_Hook) (void);
