@@ -3,6 +3,7 @@
  This file is part of SLRN.
 
  Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
+ Copyright (c) 2001 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -60,6 +61,7 @@
 #include "startup.h"
 #include "slrndir.h"
 #include "vfile.h"
+#include "snprintf.h"
 
 /*}}}*/
 
@@ -83,7 +85,6 @@ static Slrn_Group_Type *Groups;
 
 int Slrn_Group_Display_Descriptions = 1;
 int Slrn_No_Backups = 0;
-int Slrn_Prompt_Next_Group = 1;
 
 int Slrn_Unsubscribe_New_Groups = 0;
 int Slrn_Check_New_Groups = 1;
@@ -99,6 +100,7 @@ static Slrn_Group_Type *Group_Hash_Table [GROUP_HASH_TABLE_SIZE];
 
 static unsigned int Last_Cursor_Row;
 static int Groups_Hidden;	       /* if true, hide groups with no arts */
+static int Kill_After_Max = 1;
 
 typedef struct Unsubscribed_Slrn_Group_Type
 {
@@ -116,6 +118,7 @@ static Unsubscribed_Slrn_Group_Type *Unsubscribed_Groups;
 static void group_update_screen (void);
 static void group_quick_help (void);
 static void save_newsrc_cmd (void);
+static void read_and_parse_active (int);
 
 /*}}}*/
 
@@ -271,32 +274,10 @@ void slrn_mark_article_as_read (char *group, long num) /*{{{*/
 
 /*}}}*/
 
-static int group_sync_group_with_server (Slrn_Group_Type *g, int *minp, int *maxp) /*{{{*/
+static int group_update_range (Slrn_Group_Type *g, int min, int max) /*{{{*/
 {
-   int min, max, n, max_available;
-   char *group;
+   int n, max_available;
    Slrn_Range_Type *r;
-   int status;
-   
-   if (g == NULL) return -1;
-   
-   group = g->group_name;
-   
-   slrn_message_now ("Selecting %s ...", group);
-   
-   status = Slrn_Server_Obj->sv_select_group (group, minp, maxp);
-   if (status == -1)
-     return -1;
-   
-   if (status != OK_GROUP)
-     {
-	g->flags |= GROUP_UNSUBSCRIBED;
-	slrn_error ("This group appears to be bogus.");
-	return -1;
-     }
-
-   min = *minp;
-   max = *maxp;
    
    if (max == 0)
      {
@@ -317,11 +298,12 @@ static int group_sync_group_with_server (Slrn_Group_Type *g, int *minp, int *max
      }
    
    g->range.min = min;
-   if (max < g->range.max)
+
+   /* This might be due to a cancel -- or to a rescan delay in inn after we
+    * read the active file; if Kill_After_Max is 0, play it safe and do not
+    * mark any articles as read */
+   if (Kill_After_Max && (max < g->range.max))
      {
-	/* There is only one way for this to happen that I am aware of:
-	 * an article has been cancelled-- update the ranges.
-	 */
 	int nmax = g->range.max;
 	for (n = max + 1; n <= nmax; n++)
 	  group_mark_article_as_read (g, n);
@@ -360,11 +342,52 @@ static int group_sync_group_with_server (Slrn_Group_Type *g, int *minp, int *max
 	  r->next->min = 1;
 #endif
      }
+   else
+     g->unread = max_available;
+
+   if ((g->flags & GROUP_UNSUBSCRIBED) == 0) 
+     {
+	if (g->unread > 0)
+	  g->flags &= ~GROUP_HIDDEN;
+	else if (Groups_Hidden && (g != Slrn_Group_Current_Group))
+	  g->flags |= GROUP_HIDDEN;
+     }
    
    return 0;
 }
 
+/*}}}*/
 
+static int group_sync_group_with_server (Slrn_Group_Type *g, int *minp, int *maxp) /*{{{*/
+{
+   char *group;
+   int status, reselect;
+   
+   if (g == NULL) return -1;
+   
+   group = g->group_name;
+   
+   slrn_message_now ("Selecting %s ...", group);
+
+   if (((reselect = Kill_After_Max) == 0) &&
+       (strcmp (group, Slrn_Server_Obj->sv_current_group ())))
+     reselect = 1;
+   
+   status = Slrn_Server_Obj->sv_select_group (group, minp, maxp);
+   if (status == -1)
+     return -1;
+   
+   if (status != OK_GROUP)
+     {
+	g->flags |= GROUP_UNSUBSCRIBED;
+	slrn_error ("Group %s appears to be bogus.", group);
+	return -1;
+     }
+   
+   Kill_After_Max = reselect;
+   
+   return group_update_range (g, *minp, *maxp);
+}
 
 /*}}}*/
 
@@ -536,7 +559,7 @@ static int add_group (char *name, unsigned int len, /*{{{*/
 
 	     if (NULL != (tmp_name = slrn_strnmalloc (name, len, 1)))
 	       {
-		  fprintf (stderr, "Group %s is bogus,  Ignoring it.\r\n", tmp_name);
+		  slrn_error ("Group %s is bogus - ignoring it.", tmp_name);
 		  slrn_free (tmp_name);
 	       }
 	     return -1;
@@ -600,6 +623,77 @@ static int add_group (char *name, unsigned int len, /*{{{*/
 
 /*}}}*/
 
+/* Rearrange Slrn_Group_Current_Group such that it follows last_group and 
+ * return Slrn_Group_Current_Group.  If last_group is NULL, Slrn_Group_Current_Group
+ * should be put at top of list.
+ */
+static Slrn_Group_Type *place_group_in_newsrc_order (Slrn_Group_Type *last_group) /*{{{*/
+{
+   Slrn_Group_Type *next_group, *prev_group;
+   
+   next_group = Slrn_Group_Current_Group->next;
+   prev_group = Slrn_Group_Current_Group->prev;
+   if (next_group != NULL) next_group->prev = prev_group;
+   if (prev_group != NULL) prev_group->next = next_group;
+   
+   Slrn_Group_Current_Group->prev = last_group;
+   if (last_group != NULL)
+     {
+	Slrn_Group_Current_Group->next = last_group->next;
+	
+	if (Slrn_Group_Current_Group->next != NULL)
+	  Slrn_Group_Current_Group->next->prev = Slrn_Group_Current_Group;
+	
+	last_group->next = Slrn_Group_Current_Group;
+     }
+   else if (Slrn_Group_Current_Group != Groups)
+     {
+	Slrn_Group_Current_Group->next = Groups;
+	if (Groups != NULL) Groups->prev = Slrn_Group_Current_Group;
+	Groups = Slrn_Group_Current_Group;
+     }
+   else				       
+     {
+	/* correct next_group->prev since it was not set correctly above */
+	if (next_group != NULL)
+	  next_group->prev = Slrn_Group_Current_Group;
+     }
+
+   return Slrn_Group_Current_Group;
+}
+
+/*}}}*/
+static Slrn_Group_Type *insert_new_groups (void) /*{{{*/
+{
+   Slrn_Group_Type *last_group = NULL;
+   
+   if (Unsubscribed_Groups != NULL)
+     {
+	unsigned int subscribe_flag;
+	Unsubscribed_Slrn_Group_Type *ug = Unsubscribed_Groups, *ugnext;
+	     
+	if (Slrn_Unsubscribe_New_Groups)
+	  subscribe_flag = GROUP_UNSUBSCRIBED | GROUP_NEW_GROUP_FLAG;
+	else subscribe_flag = GROUP_NEW_GROUP_FLAG;
+	     
+	while (ug != NULL)
+	  {
+	     ugnext = ug->next;
+	     
+	     if (-1 != add_group (ug->group_name, strlen (ug->group_name), subscribe_flag, 0))
+	       last_group = place_group_in_newsrc_order (last_group);
+	     
+	     free_unsubscribed_group_type (ug);
+	     ug = ugnext;
+	  }
+	Unsubscribed_Groups = NULL;
+     }
+   
+   return last_group;
+}
+
+/*}}}*/
+
 static void find_line_num (void) /*{{{*/
 {
    Group_Window.lines = (SLscroll_Type *) Groups;
@@ -624,26 +718,6 @@ static void init_group_win_struct (void) /*{{{*/
    find_line_num ();
 }
 
-/*}}}*/
-static void free_all_groups (void) /*{{{*/
-{
-   Slrn_Group_Type *g = Groups;
-   Slrn_Group_Type *nextg;
-   int i;
-   
-   while (g != NULL)
-     {
-	nextg = g->next;
-	free_newsgroup_type (g);
-	g = nextg;
-     }
-   Groups = NULL;
-   Slrn_Group_Current_Group = NULL;
-   SLMEMSET((char *) &Group_Window, 0, sizeof (SLscroll_Window_Type));
-   
-   for (i = 0; i < GROUP_HASH_TABLE_SIZE; i++) Group_Hash_Table[i] = NULL;
-}
-
 /*}}}*/
 
 static int find_group (char *name) /*{{{*/
@@ -659,6 +733,7 @@ static int find_group (char *name) /*{{{*/
 
 /*}}}*/
 
+/* origpat needs enough space for 256 chars */
 static SLRegexp_Type *read_group_regexp (char *prompt, char *origpat) /*{{{*/
 {
    static char pattern[256];
@@ -666,7 +741,7 @@ static SLRegexp_Type *read_group_regexp (char *prompt, char *origpat) /*{{{*/
    if (slrn_read_input (prompt, NULL, pattern, 1, 0) <= 0) return NULL;
    
    if (origpat != NULL)
-     strcpy (origpat, pattern);
+     strcpy (origpat, pattern); /* safe */
 
    return slrn_compile_regexp_pattern (slrn_fix_regexp (pattern));
 }
@@ -747,6 +822,7 @@ int slrn_group_search (char *str) /*{{{*/
 {
    SLsearch_Type st;
    Slrn_Group_Type *g;
+   int found = 0;
    
    g = Slrn_Group_Current_Group;
    if (g == NULL) return 0;
@@ -770,17 +846,17 @@ int slrn_group_search (char *str) /*{{{*/
 		     && (NULL != SLsearch ((unsigned char *) g->descript,
 					   (unsigned char *) g->descript + strlen (g->descript),
 					   &st))))
+	       {
+		  found = 1;
 		  break;
+	       }
 	  }
      }
    while (g != Slrn_Group_Current_Group);
    
-   if (g == Slrn_Group_Current_Group)
-     return 0;
-   
    Slrn_Group_Current_Group = g;
    find_line_num ();
-   return 1;
+   return found;
 }
 
 /*}}}*/
@@ -831,15 +907,17 @@ int slrn_group_select_group (void) /*{{{*/
 
    max_available = Slrn_Group_Current_Group->range.max - Slrn_Group_Current_Group->range.min + 1;
 
-   if ((prefix & 1) || (n == 0))
+   if ((prefix != 0) || (n == 0))
      n = max_available;
 
-   if ((prefix & 1) || 
+   if ((prefix & 1) ||
        ((n > Slrn_Query_Group_Cutoff)
 	&& (Slrn_Query_Group_Cutoff > 0)))
      {
 	char int_prompt_buf[256];
-	sprintf (int_prompt_buf, "%s: Read how many?", Slrn_Group_Current_Group->group_name);
+	slrn_snprintf (int_prompt_buf, sizeof (int_prompt_buf),
+		       "%s: Read how many?",
+		       Slrn_Group_Current_Group->group_name);
 	if ((-1 == slrn_read_integer (int_prompt_buf, &n, &n))
 	    || (n <= 0))
 	  {
@@ -864,7 +942,7 @@ int slrn_group_select_group (void) /*{{{*/
 	       }
 	  }
      }
-   else if ((0 == (prefix & 1)) && (Slrn_Group_Current_Group->unread != 0))
+   else if ((0 == prefix) && (Slrn_Group_Current_Group->unread != 0))
      n = 0;
    
    ret = slrn_select_article_mode (Slrn_Group_Current_Group, n, 
@@ -969,35 +1047,76 @@ static void group_up (void) /*{{{*/
 
 /*}}}*/
 
+static void set_current_group (void) /*{{{*/
+{
+   Slrn_Group_Type *g;
+   
+   g = Slrn_Group_Current_Group;
+   if (g == NULL) g = Groups;
+   
+   while ((g != NULL) && (g->flags & GROUP_HIDDEN)) g = g->next;
+   if ((g == NULL) && (Slrn_Group_Current_Group != NULL))
+     {
+	g = Slrn_Group_Current_Group -> prev;
+	while ((g != NULL) && (g->flags & GROUP_HIDDEN)) g = g->prev;
+     }
+   Slrn_Group_Current_Group = g;
+   
+   find_line_num ();
+   
+   Slrn_Full_Screen_Update = 1;
+}
+
+/*}}}*/
+
 static void refresh_groups_cmd (void) /*{{{*/
 {
-   char *group_name;
+   Slrn_Group_Type *c = Slrn_Group_Current_Group;
+   Slrn_Group_Current_Group = NULL;
    
-   group_name = NULL;
-   if (Slrn_Group_Current_Group != NULL)
-     group_name = slrn_safe_strmalloc (Slrn_Group_Current_Group->group_name);
+   if (Slrn_List_Active_File)
+     {
+	read_and_parse_active(0);
+	if (c != NULL)
+	  /* hack: avoid a problem with inn not updating the high water mark */
+	  {
+	     Slrn_Group_Type *a = c, *b;
+	     int min, max;
+	     
+	     if (((NULL != (b = c->next)) || (NULL != (b = c->prev))) &&
+		 (0 == strcmp (c->group_name,
+			       Slrn_Server_Obj->sv_current_group ())))
+	       {
+		  a = b;
+	       }
+	     
+	     group_sync_group_with_server (a, &min, &max);
+	  }
+     }
    
-   /* slrn_set_suspension (1); */
-   save_newsrc_cmd ();
-   free_all_groups ();
-   /*
-    * Groups_Hidden gets toggled in slrn_read_newsrc, which gets called
-    * in slrn_get_new_news.  Therefore, pre-toggle Groups_Hidden ahead of
-    * time.
-    */
-   Groups_Hidden = !Groups_Hidden;
-   if (-1 == slrn_get_new_news (1, 0))
+   if (Slrn_Check_New_Groups)
      {
-	slrn_quit (0);
+	slrn_get_new_groups (0);
+	insert_new_groups ();
      }
-   if (group_name != NULL)
+   
+   if (!Slrn_List_Active_File)
      {
-	(void) find_group (group_name);
-	slrn_free (group_name);
+	Slrn_Group_Type *g = Groups;
+	int min, max;
+	
+	while (g != NULL)
+	  {
+	     if (!(g->flags & GROUP_UNSUBSCRIBED))
+	       group_sync_group_with_server (g, &min, &max);
+	     g = g->next;
+	  }
      }
-   /* slrn_set_suspension (0); */
 
-   init_group_win_struct ();
+   slrn_read_group_descriptions ();
+   
+   Slrn_Group_Current_Group = c;
+   set_current_group ();
    group_quick_help ();
 }
 
@@ -1098,11 +1217,11 @@ static void move_group_cmd (void) /*{{{*/
 {
    SLang_Key_Type *key;
    void (*f)(void);
-   Slrn_Group_Type *g;
+   Slrn_Group_Type *from, *to;
    
    if (Slrn_Batch) return;
    if (Slrn_Group_Current_Group == NULL) return;
-   g = Slrn_Group_Current_Group;
+   from = Slrn_Group_Current_Group;
    
    while (1)
      {
@@ -1114,28 +1233,49 @@ static void move_group_cmd (void) /*{{{*/
 	    || (key->type == SLKEY_F_INTERPRET))
 	  f = NULL; 
 	else f = (void (*)(void)) key->f.f;
-     
-	if (f == group_up)
+	
+	if ((f == group_up) || (f == group_down))
 	  {
-	     transpose_groups ();
-	     if (2 != slrn_group_up_n (2))
-	       break;
-	  }
-	else if (f == group_down)
-	  {
-	     if (1 != slrn_group_down_n (1))
-	       break;
+	     if (f == group_down)
+	       (void) slrn_group_down_n (1);
+	     else
+	       (void) slrn_group_up_n (1);
 	     
-	     transpose_groups ();
+	     to = Slrn_Group_Current_Group;
+	     if (from == to) break;
 	     
-	     if (1 != slrn_group_up_n (1))
-	       break;
+	     Slrn_Full_Screen_Update = 1;
+	     Slrn_Groups_Dirty = 1;
+	     
+	     if (NULL != from->next)
+	       from->next->prev = from->prev;
+	     if (NULL != from->prev)
+	       from->prev->next = from->next;
+	     
+	     if (f == group_down)
+	       {
+		  if (NULL != to->next)
+		    to->next->prev = from;
+		  from->next = to->next;
+		  from->prev = to;
+		  to->next = from;
+		  if (from == Groups) Groups = to;
+	       }
+	     else
+	       {
+		  if (NULL != to->prev)
+		    to->prev->next = from;
+		  from->prev = to->prev;
+		  from->next = to;
+		  to->prev = from;
+		  if (to == Groups) Groups = from;
+	       }
 	  }
 	else break;
 	
-	if (g != Slrn_Group_Current_Group)
+	if (from != Slrn_Group_Current_Group)
 	  {
-	     Slrn_Group_Current_Group = g;
+	     Slrn_Group_Current_Group = from;
 	     find_line_num ();
 	  }
 
@@ -1374,7 +1514,6 @@ static void toggle_group_display (void) /*{{{*/
 
 /*}}}*/
 
-
 static void toggle_hide_groups (void) /*{{{*/
 {
    Slrn_Group_Type *g;
@@ -1406,20 +1545,7 @@ static void toggle_hide_groups (void) /*{{{*/
 	  }
      }
    
-   g = Slrn_Group_Current_Group;
-   if (g == NULL) g = Groups;
-   
-   while ((g != NULL) && (g->flags & GROUP_HIDDEN)) g = g->next;
-   if ((g == NULL) && (Slrn_Group_Current_Group != NULL))
-     {
-	g = Slrn_Group_Current_Group -> prev;
-	while ((g != NULL) && (g->flags & GROUP_HIDDEN)) g = g->prev;
-     }
-   Slrn_Group_Current_Group = g;
-   
-   find_line_num ();
-   
-   Slrn_Full_Screen_Update = 1;
+   set_current_group ();
 }
 
 /*}}}*/
@@ -1459,7 +1585,7 @@ void slrn_post_cmd (void) /*{{{*/
 
    if (strlen (name) >= sizeof (group))
      name = "";
-   strcpy (group, name);
+   slrn_strncpy (group, name, sizeof (group));
    if (slrn_read_input ("Newsgroup", NULL, group, 1, 1) <= 0) return;
    *subj = 0; if (slrn_read_input ("Subject", NULL, subj, 1, 0) <= 0) return;
    
@@ -1560,38 +1686,45 @@ static Slrn_Mode_Type Group_Mode_Cap =
 #define A_KEY(s, f)  {s, (int (*)(void)) f}
 static SLKeymap_Function_Type Group_Functions [] = /*{{{*/
 {
-   A_KEY("digit_arg", slrn_digit_arg),
-   A_KEY("move_group", move_group_cmd),
-   A_KEY("uncatch_up", uncatch_up),
-   A_KEY("toggle_scoring", toggle_scoring),
-   A_KEY("toggle_group_display", toggle_group_display),
-   A_KEY("refresh_groups", refresh_groups_cmd),
-   A_KEY("save_newsrc", save_newsrc_cmd),
-   A_KEY("group_search", group_search_cmd),
-   A_KEY("group_search_forward", group_search_cmd),
-   A_KEY("toggle_list_all", toggle_list_all_groups),
    A_KEY("add_group", add_group_cmd),
-   A_KEY("group_bob", group_bob),
    A_KEY("bob", group_bob),
    A_KEY("catchup", catch_up),
-   A_KEY("down", group_down),
+   A_KEY("digit_arg", slrn_digit_arg),
    A_KEY("eob", group_eob),
-   A_KEY("group_eob", group_eob),
+   A_KEY("group_search", group_search_cmd),
+   A_KEY("group_search_forward", group_search_cmd),
    A_KEY("help", slrn_group_help),
-   A_KEY("pagedown", group_pagedown),
-   A_KEY("pageup", group_pageup),
+   A_KEY("line_down", group_down),
+   A_KEY("line_up", group_up),
+   A_KEY("move_group", move_group_cmd),
+   A_KEY("page_down", group_pagedown),
+   A_KEY("page_up", group_pageup),
    A_KEY("post", slrn_post_cmd),
+   A_KEY("post_postponed", slrn_post_postponed),
    A_KEY("quit", slrn_group_quit),
    A_KEY("redraw", slrn_redraw),
+   A_KEY("refresh_groups", refresh_groups_cmd),
+   A_KEY("repeat_last_key", slrn_repeat_last_key),
+   A_KEY("save_newsrc", save_newsrc_cmd),
    A_KEY("select_group", select_group_cmd),
    A_KEY("subscribe", subscribe),
    A_KEY("suspend", slrn_suspend_cmd),
+   A_KEY("toggle_group_display", toggle_group_display),
    A_KEY("toggle_hidden", toggle_hide_groups),
-   A_KEY("unsubscribe", unsubscribe),
-   A_KEY("up", group_up),
-   A_KEY("repeat_last_key", slrn_repeat_last_key),
+   A_KEY("toggle_list_all", toggle_list_all_groups),
+   A_KEY("toggle_scoring", toggle_scoring),
    A_KEY("transpose_groups", transpose_groups),
-   A_KEY("post_postponed", slrn_post_postponed),
+   A_KEY("uncatchup", uncatch_up),
+   A_KEY("unsubscribe", unsubscribe),
+#if 1 /* FIXME: These ones are going to be deleted before 1.0 */
+   A_KEY("down", group_down),
+   A_KEY("group_bob", group_bob),
+   A_KEY("group_eob", group_eob),
+   A_KEY("pagedown", group_pagedown),
+   A_KEY("pageup", group_pageup),
+   A_KEY("uncatch_up", uncatch_up),
+   A_KEY("up", group_up),
+#endif
    A_KEY(NULL, NULL)
 };
 
@@ -1831,17 +1964,17 @@ void slrn_get_group_descriptions (void) /*{{{*/
    int num;
    
 #ifdef VMS
-   sprintf (file, "%s-dsc", Slrn_Newsrc_File);
+   slrn_snprintf (file, sizeof (file), "%s-dsc", Slrn_Newsrc_File);
 #else
 # if defined(IBMPC_SYSTEM)
-   sprintf (file, "ds-%s", Slrn_Newsrc_File);
+   slrn_snprintf (file, sizeof (file), "ds-%s", Slrn_Newsrc_File);
 # else
-   sprintf (file, "%s.dsc", Slrn_Newsrc_File);
+   slrn_snprintf (file, sizeof (file), "%s.dsc", Slrn_Newsrc_File);
 # endif
 #endif
    
    
-   if (NULL == (fp = slrn_open_home_file (file, "w", line, 0)))
+   if (NULL == (fp = slrn_open_home_file (file, "w", line, sizeof (line), 0)))
      {
 	slrn_exit_error ("\
 Unable to create newsgroup description file:\n%s\n", line);
@@ -1918,30 +2051,34 @@ int slrn_read_group_descriptions (void) /*{{{*/
    char file[SLRN_MAX_PATH_LEN];
    
 #ifdef VMS
-   sprintf (file, "%s-dsc", Slrn_Newsrc_File);
+   slrn_snprintf (file, sizeof (file), "%s-dsc", Slrn_Newsrc_File);
 #else
 # if defined(IBMPC_SYSTEM)
-   sprintf (file, "ds-%s", Slrn_Newsrc_File);
+   slrn_snprintf (file, sizeof (file), "ds-%s", Slrn_Newsrc_File);
 # else
-   sprintf (file, "%s.dsc", Slrn_Newsrc_File);
+   slrn_snprintf (file, sizeof (file), "%s.dsc", Slrn_Newsrc_File);
 # endif
 #endif
    
    
-   if (NULL == (fp = slrn_open_home_file (file, "r", line, 0)))
+   if (NULL == (fp = slrn_open_home_file (file, "r", line, sizeof (line), 0)))
      {
 	if (Slrn_Lib_Dir != NULL)
 	  {
 #ifdef VMS
-	     sprintf (file, "%snewsgroups.dsc", Slrn_Lib_Dir);
-	     if (NULL == (fp = slrn_open_home_file (file, "r", line, 0)))
+	     slrn_snprintf (file, sizeof (file), "%snewsgroups.dsc",
+			    Slrn_Lib_Dir);
+	     if (NULL == (fp = slrn_open_home_file (file, "r", line,
+						    sizeof (line), 0)))
 	       {
-		  sprintf (file, "%snewsgroups-dsc", Slrn_Lib_Dir);
-		  fp = slrn_open_home_file (file, "r", line, 0);
+		  slrn_snprintf (file, sizeof (file), "%snewsgroups-dsc",
+				 Slrn_Lib_Dir);
+		  fp = slrn_open_home_file (file, "r", line, sizeof (line), 0);
 	       }
 #else
-	     sprintf (file, "%s/newsgroups.dsc", Slrn_Lib_Dir);
-	     fp = slrn_open_home_file (file, "r", line, 0);
+	     slrn_snprintf (file, sizeof (file), "%s/newsgroups.dsc",
+			    Slrn_Lib_Dir);
+	     fp = slrn_open_home_file (file, "r", line, sizeof (line), 0);
 #endif
 	  }
 	if (fp == NULL) return -1;
@@ -1972,7 +2109,7 @@ static int rfc977_patchup_year (int year)
 }
 
    
-int slrn_check_new_groups (int create_flag) /*{{{*/
+int slrn_get_new_groups (int create_flag) /*{{{*/
 {
    FILE *fp;
    time_t tloc;
@@ -1984,18 +2121,18 @@ int slrn_check_new_groups (int create_flag) /*{{{*/
    int parse_error = 0;
    
 #ifdef VMS
-   sprintf (file, "%s-time", Slrn_Newsrc_File);
+   slrn_snprintf (file, sizeof (file), "%s-time", Slrn_Newsrc_File);
 #else
 # ifdef SLRN_USE_OS2_FAT
-   slrn_os2_make_fat (file, Slrn_Newsrc_File, ".tim");
+   slrn_os2_make_fat (file, sizeof (file), Slrn_Newsrc_File, ".tim");
 # else
-   sprintf (file, "%s.time", Slrn_Newsrc_File);
+   slrn_snprintf (file, sizeof (file), "%s.time", Slrn_Newsrc_File);
 # endif
 #endif
    
    
    if ((create_flag == 0)
-       && (NULL != (fp = slrn_open_home_file (file, "r", NULL, 0))))
+       && (NULL != (fp = slrn_open_home_file (file, "r", NULL, 0, 0))))
      {
 	char ch;
 	int i;
@@ -2003,6 +2140,8 @@ int slrn_check_new_groups (int create_flag) /*{{{*/
 	fgets (line, sizeof (line), fp);
 	slrn_fclose (fp);
 	
+	slrn_message_now ("Checking for new groups ...");
+	
 	time (&tloc);
 	parse_error = 1;
 	
@@ -2049,24 +2188,36 @@ int slrn_check_new_groups (int create_flag) /*{{{*/
 	*p = 0;
 	
 	parse_error = 0;
-	
-	switch (Slrn_Server_Obj->sv_put_server_cmd (line, line, sizeof (line)))
+	i = 0;
+	do
 	  {
-	   case OK_NEWGROUPS:
-	     break;
-	     
-	   case ERR_FAULT:	     
-	     return -1;
-	     
-	   case ERR_COMMAND:
-	     slrn_message ("Server does not implement NEWGROUPS command.");
-	     return 0;
-
-	   default:
-	     slrn_message ("Server failed to return proper response to NEWGROUPS:\n%s\n",
-			   line);
-	     goto parse_error_label;
+	     switch (Slrn_Server_Obj->sv_put_server_cmd (line, line, sizeof (line)))
+	       {
+		case OK_NEWGROUPS:
+		  break;
+		  
+		case ERR_FAULT:	     
+		  return -1;
+		  
+		case ERR_COMMAND:
+		  slrn_message ("Server does not implement NEWGROUPS command.");
+		  return 0;
+		  
+		case ERR_NOAUTH:
+		  if ((i++ != 0) || (-1 == Slrn_Server_Obj->sv_send_authinfo ()))
+		    {
+		       slrn_exit_error ("Authorization failed.");
+		       return -1;
+		    }
+		  break;
+		  
+		default:
+		  slrn_message ("Server failed to return proper response to NEWGROUPS:\n%s\n",
+				line);
+		  goto parse_error_label;
+	       }
 	  }
+	while (i++ == 1); /* loops (at most) once */
 	
 	num = 0;
 	
@@ -2103,7 +2254,7 @@ I will patch the file up for you.\n", file);
      }
    
    
-   if (NULL == (fp = slrn_open_home_file (file, "w", NULL, 1)))
+   if (NULL == (fp = slrn_open_home_file (file, "w", NULL, 0, 1)))
      {
 	slrn_exit_error ("Unable to open %s to record date.", file);
      }
@@ -2134,47 +2285,6 @@ I will patch the file up for you.\n", file);
 
 /*}}}*/
 
-/* Rearrange Slrn_Group_Current_Group such that it follows last_group and 
- * return Slrn_Group_Current_Group.  If last_group is NULL, Slrn_Group_Current_Group
- * should be put at top of list.
- */
-static Slrn_Group_Type *place_group_in_newsrc_order (Slrn_Group_Type *last_group) /*{{{*/
-{
-   Slrn_Group_Type *next_group, *prev_group;
-   
-   next_group = Slrn_Group_Current_Group->next;
-   prev_group = Slrn_Group_Current_Group->prev;
-   if (next_group != NULL) next_group->prev = prev_group;
-   if (prev_group != NULL) prev_group->next = next_group;
-   
-   Slrn_Group_Current_Group->prev = last_group;
-   if (last_group != NULL)
-     {
-	Slrn_Group_Current_Group->next = last_group->next;
-	
-	if (Slrn_Group_Current_Group->next != NULL)
-	  Slrn_Group_Current_Group->next->prev = Slrn_Group_Current_Group;
-	
-	last_group->next = Slrn_Group_Current_Group;
-     }
-   else if (Slrn_Group_Current_Group != Groups)
-     {
-	Slrn_Group_Current_Group->next = Groups;
-	if (Groups != NULL) Groups->prev = Slrn_Group_Current_Group;
-	Groups = Slrn_Group_Current_Group;
-     }
-   else				       
-     {
-	/* correct next_group->prev since it was not set correctly above */
-	if (next_group != NULL)
-	  next_group->prev = Slrn_Group_Current_Group;
-     }
-
-   return Slrn_Group_Current_Group;
-}
-
-/*}}}*/
-
 static int parse_active_line (char *name, unsigned int *lenp, /*{{{*/
 			      int *minp, int *maxp)
 {
@@ -2194,46 +2304,98 @@ static int parse_active_line (char *name, unsigned int *lenp, /*{{{*/
 
 /*}}}*/
 
+static void read_and_parse_active (int create_flag) /*{{{*/
+{
+   char line[NNTP_BUFFER_SIZE];
+   int count = 0;
+   int initial_run = (Groups == NULL);
+
+   if (OK_GROUPS != Slrn_Server_Obj->sv_list_active ())
+     slrn_exit_error ("Server failed LIST ACTIVE.");
+   
+   Kill_After_Max = 0;
+
+   while (1)
+     {
+	unsigned int len;
+	int min, max;
+	int status;
+
+	status = Slrn_Server_Obj->sv_read_line (line, sizeof(line));
+	if (status == -1)
+	  {
+	     Slrn_Groups_Dirty = 0;
+	     slrn_exit_error ("Read from server failed");
+	  }
+	if (status == 0)
+	  break;
+
+	parse_active_line (line, &len, &min, &max);
+	
+	if (!initial_run)
+	  {
+	     Slrn_Group_Type *g = find_group_entry (line, len);
+	     if (g != NULL)
+	       {
+		  group_update_range (g, min, max);
+		  continue;
+	       }
+	  }
+	
+	if (NULL == create_group_entry (line, len, min, max, 0, 0))
+	  continue;
+	
+	if (create_flag)
+	  {
+	     count++;
+	     count = count % 50;
+	     if (count == 0)
+	       {
+		  putc ('.', stdout);
+		  fflush (stdout);
+	       }
+	     
+	     if ((*line == 'n')
+		 && (!strncmp (line,    "news.newusers.questions", 23)
+		     || !strncmp (line, "news.groups.questions", 21)
+		     || !strncmp (line, "news.answers", 12)
+		     || !strncmp (line, "news.announce.newusers", 22)
+		     || !strncmp (line, "news.software.readers", 21)))
+	       {
+		  add_group (line, len, 0, 1);
+	       }
+	     else if ((*line == 'a')
+		      && (!strncmp (line, "alt.test", 8))
+		      && (line[8] <= ' '))
+	       {
+		  add_group (line, len, 0, 1);
+	       }
+	     else add_group (line, len, GROUP_UNSUBSCRIBED, 1);
+	  }
+     }
+}
+
+/*}}}*/
+
 static int read_and_parse_newsrc_file (void)
 {
-   Slrn_Group_Type *last_group = NULL;
    VFILE *vp;
    char *vline;
    unsigned int vlen;
    char file[SLRN_MAX_PATH_LEN];
+   Slrn_Group_Type *last_group = insert_new_groups ();
 
-   if (Unsubscribed_Groups != NULL)
-     {
-	unsigned int subscribe_flag;
-	Unsubscribed_Slrn_Group_Type *ug = Unsubscribed_Groups, *ugnext;
-	     
-	if (Slrn_Unsubscribe_New_Groups)
-	  subscribe_flag = GROUP_UNSUBSCRIBED | GROUP_NEW_GROUP_FLAG;
-	else subscribe_flag = GROUP_NEW_GROUP_FLAG;
-	     
-	while (ug != NULL)
-	  {
-	     ugnext = ug->next;
-	     
-	     if ((-1 != add_group (ug->group_name, strlen (ug->group_name), subscribe_flag, 0))
-		 && Slrn_List_Active_File)
-	       last_group = place_group_in_newsrc_order (last_group);
-	     
-	     free_unsubscribed_group_type (ug);
-	     ug = ugnext;
-	  }
-	Unsubscribed_Groups = NULL;
-     }
-
-   if ((NULL == (vp = slrn_open_home_vfile (Slrn_Newsrc_File, file)))
-       && (NULL == (vp = slrn_open_home_vfile (".newsrc", file))))
+   if ((NULL == (vp = slrn_open_home_vfile (Slrn_Newsrc_File, file,
+					    sizeof (file))))
+       && (NULL == (vp = slrn_open_home_vfile (".newsrc", file,
+					       sizeof (file)))))
      slrn_exit_error ("Unable to open %s.", file);
 
    while (NULL != (vline = vgets (vp, &vlen)))
      {
 	char *p = vline;
 	char *pmax = p + vlen;
-	char ch;
+	char ch = 0;
 
 	while ((p < pmax)
 	       && ((ch = *p) != '!') && (ch != ':'))
@@ -2257,11 +2419,14 @@ static int read_and_parse_newsrc_file (void)
    vclose (vp);
    return 0;
 }
-
    
 int slrn_read_newsrc (int create_flag) /*{{{*/
 {
-   char line[NNTP_BUFFER_SIZE];
+   char *msg1 = "Checking news via active file ...";
+   char *msg2 = "Checking news ...";
+   
+   if (Slrn_List_Active_File) slrn_message_now (msg1);
+   else slrn_message_now (msg2);
    
    if (create_flag)
      {
@@ -2269,7 +2434,8 @@ int slrn_read_newsrc (int create_flag) /*{{{*/
 	char file[SLRN_MAX_PATH_LEN];
 
 	/* See if we can open the file. */
-	if (NULL == (fp = slrn_open_home_file (Slrn_Newsrc_File, "w", file, 1)))
+	if (NULL == (fp = slrn_open_home_file (Slrn_Newsrc_File, "w", file,
+					       sizeof (file), 1)))
 	  {
 	     slrn_exit_error ("Unable to create %s.", file);
 	  }
@@ -2281,60 +2447,7 @@ int slrn_read_newsrc (int create_flag) /*{{{*/
    
    if (create_flag || Slrn_List_Active_File)
      {
-	int count = 0;
-
-	if (OK_GROUPS != Slrn_Server_Obj->sv_list_active ())
-	  slrn_exit_error ("Server failed LIST ACTIVE.");
-
-	while (1)
-	  {
-	     unsigned int len;
-	     int min, max;
-	     int status;
-
-	     status = Slrn_Server_Obj->sv_read_line (line, sizeof(line));
-	     if (status == -1)
-	       {
-		  Slrn_Groups_Dirty = 0;
-		  slrn_exit_error ("Read from server failed");
-	       }
-	     if (status == 0)
-	       break;
-
-	     parse_active_line (line, &len, &min, &max);
-	     
-	     if (NULL == create_group_entry (line, len, min, max, 0, 0))
-	       continue;
-	     
-	     
-	     if (create_flag)
-	       {
-		  count++;
-		  count = count % 50;
-		  if (count == 0)
-		    {
-		       putc ('.', stdout);
-		       fflush (stdout);
-		    }
-		  
-		  if ((*line == 'n')
-		      && (!strncmp (line,    "news.newusers.questions", 23)
-			  || !strncmp (line, "news.groups.questions", 21)
-			  || !strncmp (line, "news.answers", 12)
-			  || !strncmp (line, "news.announce.newusers", 22)
-			  || !strncmp (line, "news.software.readers", 21)))
-		    {
-		       add_group (line, len, 0, 1);
-		    }
-		  else if ((*line == 'a')
-			   && (!strncmp (line, "alt.test", 8))
-			   && (line[8] <= ' '))
-		    {
-		       add_group (line, len, 0, 1);
-		    }
-		  else add_group (line, len, GROUP_UNSUBSCRIBED, 1);
-	       }
-	  }
+	read_and_parse_active (create_flag);
      }
    
    if ((create_flag == 0)
@@ -2416,7 +2529,7 @@ int slrn_write_newsrc (void) /*{{{*/
    
    slrn_message_now ("Writing %s ...", Slrn_Newsrc_File);
    
-   slrn_make_home_filename (Slrn_Newsrc_File, file);
+   slrn_make_home_filename (Slrn_Newsrc_File, file, sizeof (file));
    
    /* Try to preserve .newsrc permissions and owner/group.  This also 
     * confirms existence of file.
@@ -2424,12 +2537,12 @@ int slrn_write_newsrc (void) /*{{{*/
    stat_worked = (-1 != stat (file, &filestat));
   
 #ifdef VMS
-   sprintf (backup_file, "%s-bak", file);
+   slrn_snprintf (backup_file, sizeof (backup_file), "%s-bak", file);
 #else
 # ifdef SLRN_USE_OS2_FAT
-   slrn_os2_make_fat (backup_file, file, ".bak");
+   slrn_os2_make_fat (backup_file, sizeof (backup_file), file, ".bak");
 # else
-   sprintf (backup_file, "%s~", file);
+   slrn_snprintf (backup_file, sizeof (backup_file), "%s~", file);
 # endif
 #endif
    
@@ -2547,6 +2660,16 @@ int slrn_write_newsrc (void) /*{{{*/
 			 goto write_error;
 		    }
 	       }
+	     else if (g->range.min == 2)
+	       {
+		  if (EOF == fputs (" 1", fp))
+		    goto write_error;
+	       }
+	     else if (g->range.min > 2)
+	       {
+		  if (fprintf (fp, " %d-%d", 1, g->range.min - 1) < 0)
+		    goto write_error;
+	       }
 	     
 	     if (EOF == putc ('\n', fp))
 	       goto write_error;
