@@ -2,6 +2,7 @@
  This file is part of SLRN.
 
  Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
+ Copyright (c) 2001 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -25,7 +26,7 @@ extern int slrn_group_select_group (void);
 extern int slrn_read_newsrc (int);
 extern int slrn_write_newsrc (void);
 extern void slrn_mark_article_as_read (char *, long);
-extern int slrn_check_new_groups (int);
+extern int slrn_get_new_groups (int);
 extern void slrn_init_group_mode (void);
 extern void slrn_get_group_descriptions (void);
 extern int slrn_read_group_descriptions (void);
@@ -84,7 +85,6 @@ extern int Slrn_Unsubscribe_New_Groups;
 extern int Slrn_Check_New_Groups;
 extern int Slrn_List_Active_File;
 extern int Slrn_Query_Group_Cutoff;
-extern int Slrn_Prompt_Next_Group;
 extern int Slrn_Use_Xgtitle;
 extern int Slrn_Write_Newsrc_Flags;
 extern int Slrn_Display_Cursor_Bar;
