@@ -112,7 +112,8 @@ static char *Art_Help[] =
      "  ESC-a       Toggle header display format",
      "  ESC-s       Select sort mode",
      "  ESC-p       Find parent header reading from server if necessary",
-     "  ESC 1 ESC p Reconstruct thread.",
+     "  ESC 1 ESC p Reconstruct thread (slow when run on large threads)",
+     "  ESC 2 ESC p Reconstruct thread (fast; might not find all articles)",
      "  ESC Ctrl-P  Find all children of current header (queries server)",
      "  c           Catch-up (Mark ALL articles as read).",
      "  ESC-u       Mark ALL articles as UN-read",
@@ -145,10 +146,11 @@ static char *Group_Help [] =
      "  P                    Post an article to the current newsgroup",
      "  ESC P                Post a or edit postponed article",
      "  SPACE, RETURN        Select the current newsgroup.",
-     "   The following three variations permit viewing previously read articles:",
+     "   The following four variations permit viewing previously read articles:",
      "  ESC 1 SPACE          Select group with article number query.",
      "  ESC 2 SPACE          Select group but do not apply score.",
      "  ESC 3 SPACE          Select group with query but do not apply score.",
+     "  ESC 4 SPACE          Select the current newsgroup.",
      "  s                    Subscribe to the current newsgroup",
      "  ESC 1 s              Subscribe to pattern",
      "  u                    Un-subscribe from the current newsgroup",
