@@ -3,6 +3,7 @@
  This file is part of SLRN.
 
  Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
+ Copyright (c) 2001 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -32,6 +33,9 @@
 # include <stdlib.h>
 #endif
 #include <string.h>
+#ifdef HAVE_LOCALE_H
+# include <locale.h>
+#endif
 
 #include <slang.h>
 #include "jdmacros.h"
@@ -46,6 +50,11 @@
 #include "interp.h"
 #include "util.h"
 #include "print.h"
+#include "score.h"
+#include "snprintf.h"
+#if SLRN_HAS_MIME 
+# include "mime.h"
+#endif
 
 /*}}}*/
 
@@ -211,7 +220,7 @@ static void tt_send (char *s)
 static void make_home_filename (char *name) /*{{{*/
 {
    char file [SLRN_MAX_PATH_LEN];
-   slrn_make_home_filename (name, file);
+   slrn_make_home_filename (name, file, sizeof (file));
    SLang_push_string (file);
 }
 
@@ -241,8 +250,8 @@ int slrn_eval_slang_file (char *name) /*{{{*/
 						   file, sizeof (file)))
 	       break;
 	     
-	     slrn_make_home_dirname (file, dir);
-	     if ((-1 != slrn_dircat (dir, name, file))
+	     slrn_make_home_dirname (file, dir, sizeof (dir));
+	     if ((-1 != slrn_dircat (dir, name, file, sizeof (file)))
 		 && (1 == slrn_file_exists (file)))
 	       {
 		  slrn_message_now ("loading %s", file);
@@ -254,7 +263,7 @@ int slrn_eval_slang_file (char *name) /*{{{*/
 	  }
      }
    
-   slrn_make_home_filename (name, file);
+   slrn_make_home_filename (name, file, sizeof (file));
    slrn_message_now ("loading %s", file);
    if (0 == SLang_load_file (file))
      return -1;
@@ -314,6 +323,88 @@ static void get_variable_value (char *name) /*{{{*/
 
 /*}}}*/
 
+static char *interp_setlocale (int *category, char *locale) /*{{{*/
+{
+#ifdef HAVE_SETLOCALE
+   char *ret = setlocale (*category, locale);
+   return (ret == NULL) ? "" : ret;
+#else
+   (void) category; (void) locale;
+   slrn_error ("setlocale is not available on this system.");
+   return "";
+#endif
+}
+
+/*}}}*/
+
+#if SLANG_VERSION >= 10400
+static void set_utf8_conversion_table (void) /*{{{*/
+{
+# if ! SLRN_HAS_MIME
+   slrn_error ("This version of slrn lacks MIME support.");
+# else
+   SLang_Array_Type *at;
+   int rows;
+   
+   if (-1 == SLang_pop_array_of_type (&at, SLANG_INT_TYPE))
+     {
+	slrn_error ("Array of integer expected.");
+	return;
+     }
+   
+   if ((at->num_dims != 2) || (at->dims[0] != 2))
+     {
+	slrn_error ("Two-dimensional array with two columns expected.");
+	goto free_and_return;
+     }
+   
+   slrn_free (Slrn_Utf8_Table);
+   Slrn_Utf8_Table = NULL;
+   
+   if ((rows = at->dims[1]) == 0)
+     goto free_and_return; /* no data, so set default (iso-8859-1) */
+   
+   if (NULL == (Slrn_Utf8_Table = slrn_malloc (65536, 1, 0)))
+     {
+	slrn_error ("Could not get memory for conversion table.");
+	goto free_and_return;
+     }
+   
+   while (rows-- != 0)
+     {
+	int dims[2];
+	int unicode, local;
+	
+	dims[0] = 0;
+	dims[1] = rows;
+	(void) SLang_get_array_element (at, dims, &unicode);
+	
+	if (((unicode != 0) && (unicode < 128)) || (unicode > 65535))
+	  {
+	     slrn_error ("Unicode value in row %d out of range.", rows);
+	     goto free_and_return;
+	  }
+	
+	dims[0] = 1;
+	(void) SLang_get_array_element (at, dims, &local);
+	
+	if ((local < 1) || (local > 255))
+	  {
+	     slrn_error ("Local charset value in row %d out of range.",
+			 rows);
+	     goto free_and_return;
+	  }
+	
+	Slrn_Utf8_Table[unicode] = (char) local;
+     }
+   
+   free_and_return:
+   SLang_free_array (at);
+# endif /* SLRN_HAS_MIME */
+}
+/*}}}*/
+#endif /* SLANG_VERSION */
+
 /*}}}*/
 
 static char *get_server_name (void) /*{{{*/
@@ -490,11 +581,22 @@ static int check_article_mode (void) /*{{{*/
 
 /*}}}*/
 
+static int is_article_mode (void)
+{
+   return ((Slrn_Current_Mode != NULL)
+	   && (Slrn_Current_Mode->mode == SLRN_ARTICLE_MODE));
+}
+
 static void set_article_window_size (int *nrows)
 {
    slrn_set_article_window_size (*nrows);
 }
 
+static int get_article_window_size (void)
+{
+   return slrn_get_article_window_size ();
+}
+
 /*{{{ Article Mode Article Functions */
 
 static void pipe_article_cmd (char *cmd) /*{{{*/
@@ -560,8 +662,8 @@ static char *extract_article_header (char *h) /*{{{*/
    len = strlen (h);
    if ((len + 3) <= sizeof (buf))
      {
-	sprintf (buf, "%s: ", h);
-	h = slrn_extract_header (buf, len + 2);
+	slrn_snprintf (buf, sizeof (buf), "%s: ", h);
+	h = slrn_cur_extract_header (buf, len + 2);
      }
    else h = NULL;
    
@@ -572,7 +674,6 @@ static char *extract_article_header (char *h) /*{{{*/
 
 /*}}}*/
 
-
 static char *extract_current_article_header (char *h) /*{{{*/
 {
    unsigned int len;
@@ -589,8 +690,8 @@ static char *extract_current_article_header (char *h) /*{{{*/
    len = strlen (h);
    if ((len + 3) <= sizeof (buf))
      {
-	sprintf (buf, "%s: ", h);
-	h = slrn_art_extract_header (Slrn_Current_Article, buf, len + 2);
+	slrn_snprintf (buf, sizeof (buf), "%s: ", h);
+	h = slrn_extract_header (buf, len + 2);
      }
    else h = NULL;
 
@@ -657,6 +758,16 @@ static char *article_as_string (void) /*{{{*/
 
 /*}}}*/
 
+static void sort_articles (void) /*{{{*/
+{
+   if (-1 == check_article_mode ())
+     return;
+   
+   slrn_sort_by_sorting_mode ();
+}
+
+/*}}}*/
+
 /*}}}*/
 
 /*{{{ Article Mode Header Functions */
@@ -667,6 +778,14 @@ static int Interp_Header_Read = HEADER_READ;
 static int Interp_Header_Tagged = HEADER_TAGGED;
 static int Interp_Header_High_Score = HEADER_HIGH_SCORE;
 static int Interp_Header_Low_Score = HEADER_LOW_SCORE;
+#ifdef HAVE_SETLOCALE
+# ifdef LC_TIME
+static int Interp_Lc_Time = LC_TIME;
+# endif
+# ifdef LC_CTYPE
+static int Interp_Lc_Ctype = LC_CTYPE;
+# endif
+#endif
 
 /*}}}*/
 
@@ -764,6 +883,14 @@ static void set_header_flags (int *flagsp) /*{{{*/
 
 /*}}}*/
 
+static int get_header_number (void)
+{
+   if ((-1 == check_article_mode ())
+       || (Slrn_Current_Header == NULL))
+     return 0;
+   return Slrn_Current_Header->number;
+}
+
 static int get_header_tag_number (void)
 {
    if ((Slrn_Current_Header == NULL) 
@@ -784,13 +911,7 @@ static int get_header_score (void)
        || (Slrn_Current_Header == NULL))
      return 0;
 
-#if SLRN_HAS_SORT_BY_SCORE
    return Slrn_Current_Header->score;
-#else
-   if (Slrn_Current_Header->flags & HEADER_LOW_SCORE)
-     return -1;
-   return 0;
-#endif
 }
 
 static int get_grouplens_score (void)
@@ -1012,7 +1133,7 @@ static void print_file (char *file)
 
 static int locate_header_by_msgid (char *msgid, int *qs)
 {
-   if (-1 == slrn_locate_header_by_msgid (msgid, *qs))
+   if (-1 == slrn_locate_header_by_msgid (msgid, 1, *qs))
      return 0;
    return 1;
 }
@@ -1034,6 +1155,26 @@ static int is_article_visible (void)
    return x;
 }
 
+static void reload_scorefile (int *apply_now) /*{{{*/
+{
+   char file[SLRN_MAX_PATH_LEN];
+   
+   if (Slrn_Score_File == NULL)
+     return;
+   
+   slrn_make_home_filename (Slrn_Score_File, file, sizeof (file));
+
+   slrn_message_now ("Reloading score file ...");
+   
+   if (-1 == slrn_read_score_file (file))
+     slrn_error ("Error processing score file %s.", file);
+   
+   if (is_article_mode() && *apply_now)
+     slrn_apply_scores (*apply_now);
+}
+
+/*}}}*/
+
 static SLang_Intrin_Fun_Type Slrn_Intrinsics [] = /*{{{*/
 {
    MAKE_INTRINSIC_0("headers_hidden_mode", slrn_is_hidden_headers_mode, SLANG_INT_TYPE),
@@ -1054,12 +1195,14 @@ static SLang_Intrin_Fun_Type Slrn_Intrinsics [] = /*{{{*/
    MAKE_INTRINSIC_0("collapse_thread", collapse_thread, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("collapse_threads", collapse_threads, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("current_newsgroup", current_group_name, SLANG_STRING_TYPE),
+   MAKE_INTRINSIC_S("datestring_to_unixtime", slrn_date_to_order_parm, SLANG_INT_TYPE),
    MAKE_INTRINSIC_SSS("definekey", definekey, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_S("extract_article_header", extract_article_header, SLANG_STRING_TYPE),
    MAKE_INTRINSIC_S("extract_displayed_article_header", extract_current_article_header, SLANG_STRING_TYPE),
    MAKE_INTRINSIC_0("get_group_flags", get_group_flags, SLANG_INT_TYPE),
    MAKE_INTRINSIC_0("get_grouplens_score", get_grouplens_score, SLANG_INT_TYPE),
    MAKE_INTRINSIC_0("get_header_flags", get_header_flags, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_0("get_header_number", get_header_number, SLANG_INT_TYPE),
    MAKE_INTRINSIC_0("get_header_score", get_header_score, SLANG_INT_TYPE),
    MAKE_INTRINSIC_0("get_header_tag_number", get_header_tag_number, SLANG_INT_TYPE),
    MAKE_INTRINSIC_0("get_next_art_pgdn_action", slrn_get_next_pagedn_action, SLANG_INT_TYPE),
@@ -1094,11 +1237,13 @@ static SLang_Intrin_Fun_Type Slrn_Intrinsics [] = /*{{{*/
    MAKE_INTRINSIC_S("re_search_article", re_search_article, SLANG_INT_TYPE),
    MAKE_INTRINSIC_SSS("read_mini", read_mini, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_SSS("read_mini_no_echo", read_mini_no_echo, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_I("reload_scorefile", reload_scorefile, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_S("save_current_article", save_current_article, SLANG_INT_TYPE),
    MAKE_INTRINSIC_S("search_article", search_article, SLANG_INT_TYPE),
    MAKE_INTRINSIC_0("select_group", select_group, SLANG_INT_TYPE),
    MAKE_INTRINSIC_0("select_list_box", select_list_box, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("server_name", get_server_name, SLANG_STRING_TYPE),
+   MAKE_INTRINSIC_0("get_article_window_size", get_article_window_size, SLANG_INT_TYPE),
    MAKE_INTRINSIC_I("set_article_window_size", set_article_window_size, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_SSS("set_color", set_color, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_I("set_group_flags", set_group_flags, SLANG_VOID_TYPE),
@@ -1109,6 +1254,11 @@ static SLang_Intrin_Fun_Type Slrn_Intrinsics [] = /*{{{*/
    MAKE_INTRINSIC_SI("set_integer_variable", set_integer_variable, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_I("set_prefix_argument", set_prefix_arg, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_SS("set_string_variable", set_string_variable, SLANG_VOID_TYPE),
+#if SLANG_VERSION >= 10400
+   MAKE_INTRINSIC_0("set_utf8_conversion_table", set_utf8_conversion_table, SLANG_VOID_TYPE),
+#endif
+   MAKE_INTRINSIC_IS("setlocale", interp_setlocale, SLANG_STRING_TYPE),
+   MAKE_INTRINSIC_0("sort_by_sorting_method", sort_articles, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("thread_size", thread_size, SLANG_INT_TYPE),
    MAKE_INTRINSIC_S("tt_send", tt_send, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("uncollapse_thread", uncollapse_thread, SLANG_VOID_TYPE),
@@ -1121,7 +1271,7 @@ static SLang_Intrin_Fun_Type Slrn_Intrinsics [] = /*{{{*/
 
 /*}}}*/
 
-static SLang_Intrin_Var_Type Intrin_Vars [] =
+static SLang_Intrin_Var_Type Intrin_Vars [] = /*{{{*/
 {
    MAKE_VARIABLE("GROUPS_DIRTY", &Slrn_Groups_Dirty, SLANG_INT_TYPE, 1),
    MAKE_VARIABLE("GROUP_NEW_GROUP_FLAG", &Interp_Group_New_Group_Flag, SLANG_INT_TYPE, 1),
@@ -1132,10 +1282,19 @@ static SLang_Intrin_Var_Type Intrin_Vars [] =
    MAKE_VARIABLE("HEADER_TAGGED", &Interp_Header_Tagged, SLANG_INT_TYPE, 1),
    MAKE_VARIABLE("SCREEN_HEIGHT", &SLtt_Screen_Rows, SLANG_INT_TYPE, 1),
    MAKE_VARIABLE("SCREEN_WIDTH", &SLtt_Screen_Cols, SLANG_INT_TYPE, 1),
+#ifdef HAVE_SETLOCALE
+# ifdef LC_CTYPE
+   MAKE_VARIABLE("LC_CTYPE", &Interp_Lc_Ctype, SLANG_INT_TYPE, 1),
+# endif
+# ifdef LC_TIME
+   MAKE_VARIABLE("LC_TIME", &Interp_Lc_Time, SLANG_INT_TYPE, 1),
+# endif
+#endif
 
    MAKE_VARIABLE(NULL, NULL, 0, 0)
 };
 
+/*}}}*/
 
 static int interp_system (char *s) /*{{{*/
 {
