@@ -43,6 +43,7 @@
 #include "misc.h"
 #include "util.h"
 #include "slrndir.h"
+#include "snprintf.h"
 
 /*}}}*/
 
@@ -780,14 +781,13 @@ static char *browse_dir (char *dir)
    if (NULL == (d = slrn_open_dir (dir)))
      return NULL;
    
-   len = strlen (dir);
+   len = strlen (dir) + 1;
    if (len >= sizeof (title))
      {
 	dir = dir + 4 + (len - sizeof(title));
-	sprintf (title, "....%s", dir + (len + 5 - sizeof(title)));
+	sprintf (title, "....%s", dir); /* safe */
      }
-   else strcpy (title, dir);
-
+   else strcpy (title, dir); /* safe */
    
    argc = 0;
    while ((argc < MAX_DIR_FILES)
@@ -805,7 +805,7 @@ static char *browse_dir (char *dir)
 	     free_dir_list (argc, argv);
 	     return NULL;
 	  }
-	strcpy (argv[argc], de->name);
+	strcpy (argv[argc], de->name); /* safe */
 	if (status == 2)
 	  {
 	     argv[argc][len] = '/';
