@@ -2,6 +2,7 @@
  *
  * Author: Michael Elkins <elkins@aero.org>
  * Modified by John E. Davis <davis@space.mit.edu>
+ * Modified by Thomas Schultz <tststs@gmx.de>
  * 
  * Change Log:
  * Aug 20, 1997 patch from "Byrial Jensen" <byrial@post3.tele.dk>
@@ -11,7 +12,9 @@
  */
 
 #include "config.h"
+#ifndef SLRNPULL_CODE
 #include "slrnfeat.h"
+#endif
 
 #include <stdio.h>
 #include <string.h>
@@ -41,6 +44,7 @@
 #include "group.h"
 #include "art.h"
 #include "util.h"
+#include "snprintf.h"
 
 #if SLRN_HAS_MIME
 /* rest of file in this ifdef */
@@ -50,6 +54,7 @@
 int Slrn_Use_Mime = 1;
 int Slrn_Use_Meta_Mail = 1;
 char *Slrn_MetaMail_Cmd;
+char *Slrn_Utf8_Table = NULL;
 
 int Slrn_Mime_Was_Parsed;
 int Slrn_Mime_Was_Modified;
@@ -60,15 +65,18 @@ char *Slrn_Mime_Display_Charset;
 /* These are all supersets of US-ASCII.  Only the first N characters are 
  * matched, where N is the length of the table entry.
  */
-static char *Compatable_Charsets[] =
+static char **Custom_Compatible_Charsets;
+static char *Compatible_Charsets[] =
 {
    "US-ASCII",			       /* This MUST be zeroth element */
    "ISO-8859-",
    "iso-latin1",		       /* knews adds his one */
    "KOI8-R",
+   "utf-8",			 /* we now have a function to decode this */
    NULL
 };
 
+#ifndef SLRNPULL_CODE
 static char *Char_Set;
 static int Content_Type;
 #define CONTENT_TYPE_TEXT		0x01
@@ -89,13 +97,6 @@ static int Encoding_Method;
 #define ENCODED_BINARY			5
 #define ENCODED_UNSUPPORTED		6
 
-#ifndef isalnum
-#define isalnum(x) \
-  ((((x) <= 'Z') && ((x) >= 'A')) \
-   || (((x) <= 'z') && ((x) >= 'a')) \
-   || (((x) <= '9') && ((x) >= '0')))
-#endif
-
 static Slrn_Article_Line_Type *find_header_line (Slrn_Article_Type *a, char *header)
 {
    Slrn_Article_Line_Type *line;
@@ -119,29 +120,85 @@ static Slrn_Article_Line_Type *find_header_line (Slrn_Article_Type *a, char *hea
    return NULL;
 }
 
-
-static char *find_compatable_charset (char *cs, unsigned int len)
+int slrn_set_compatible_charsets (char *charsets)
 {
-   char **compat_charset;
+   static char* buf;
+   char *p;
+   char **pp;
+   unsigned int n;
    
-   compat_charset = Compatable_Charsets;
+   slrn_free (buf);
+   buf = NULL;
+   slrn_free ((char *) Custom_Compatible_Charsets);
+
+   n = 1;
+   p = charsets;
+   while (NULL != (p = strchr (p, ',')))
+     {
+	n++; p++;
+     }
+   
+   Custom_Compatible_Charsets = (char **) SLmalloc (sizeof (char **) * (n+1));
+   if (Custom_Compatible_Charsets == NULL)
+     return -1;
+   
+   buf = slrn_strmalloc (charsets, 0);
+   if (buf == NULL)
+     {
+	slrn_free ((char *) Custom_Compatible_Charsets);
+	Custom_Compatible_Charsets = NULL;
+	return -1;
+     }
+   
+   pp = Custom_Compatible_Charsets;
+   p = charsets;
+   while (n-- != 0)
+     {
+	*pp++ = p;
+	if (NULL != (p = strchr (p, ',')))
+	  {
+	     *p = 0;
+	     p++;
+	  }
+     }
+   
+   *pp = NULL;
+   
+   return 0;
+}
+#endif /* NOT SLRNPULL_CODE */
+
+static char *_find_compatible_charset (char **compat_charset, char *cs,
+				       unsigned int len)
+{
    while (*compat_charset != NULL)
      {
 	unsigned int len1;
 	
 	len1 = strlen (*compat_charset);
-	if (len1 <= len) 
-	  {
-	     if (0 == slrn_case_strncmp ((unsigned char *) cs,
-					 (unsigned char *) *compat_charset,
-					 len1))
-	       return *compat_charset;
-	  }
+	if ((len1 <= len) &&
+	    (0 == slrn_case_strncmp ((unsigned char *) cs,
+				     (unsigned char *) *compat_charset,
+				     len1)))
+	  return *compat_charset;
 	compat_charset++;
      }
    return NULL;
 }
 
+static char *find_compatible_charset (char *cs, unsigned int len)
+{
+   char *retval;
+   
+   if ((NULL == (retval = _find_compatible_charset (Compatible_Charsets, cs,
+						   len))) &&
+       (NULL != Custom_Compatible_Charsets))
+     retval = _find_compatible_charset (Custom_Compatible_Charsets, cs, len);
+   
+   return retval;
+}
+
+#ifndef SLRNPULL_CODE
 static int parse_content_type_line (Slrn_Article_Type *a)
 {
    char *b;
@@ -153,7 +210,7 @@ static int parse_content_type_line (Slrn_Article_Type *a)
    /* Use default: text/plain; charset=us-ascii */
    Content_Type = CONTENT_TYPE_TEXT;
    Content_Subtype = CONTENT_SUBTYPE_PLAIN;
-   Char_Set = Compatable_Charsets[0];
+   Char_Set = Compatible_Charsets[0];
    
    if (NULL == (line = find_header_line (a, "Content-Type:")))
      return 0;
@@ -231,7 +288,7 @@ static int parse_content_type_line (Slrn_Article_Type *a)
 	       b++;
 	     len = b - charset;
 	     
-	     Char_Set = find_compatable_charset (charset, len);
+	     Char_Set = find_compatible_charset (charset, len);
 	     if (Char_Set == NULL) return -1;
 	     return 0;
 	  }
@@ -276,6 +333,7 @@ static int parse_content_transfer_encoding_line (Slrn_Article_Type *a)
      }
    return 0;
 }
+#endif /* NOT SLRNPULL_CODE*/
 
 static int Index_Hex[128] =
 {
@@ -345,6 +403,89 @@ static char *decode_base64 (char *dest, char *src, char *srcmax)
    return dest;
 }
 
+static char *utf_to_unicode (int *out, char *in, char *srcmax)
+{
+   int mask = 0;
+   short len = 0, i;
+   
+   for (i = 1; i < 8; i++)
+     {
+	if ((*in & (mask |(0x01 << (8-i)))) == mask)
+	  {
+	     len = i;
+	     break;
+	  }
+	mask = (mask |(0x01 << (8-i)));
+     }
+   
+   if ((len == 0) || (len == 2) || (in + len - 1 > srcmax))
+     {
+	*out = -1;
+	return in + 1;
+     }
+   if (len > 1) len--;
+   
+   for (i = 1; i < len; i++)
+     if ((in[i] & 0xC0) != 0x80)
+       {
+	  *out = -1;
+	  return in + 1;
+       }
+   
+   *out = ((int) in[0]) & ~mask & 0xFF;
+   for (i = 1; i < len; i++)
+     *out = (*out << 6) | (((int) in[i]) & 0x7F);
+   
+   if (!*out)
+     *out = -1;
+   
+   return in + len;
+}
+
+static char *decode_utf8 (char *dest, char *src, char *srcmax,
+			  char *utf8_error)
+{
+   int ch;
+   if (utf8_error != NULL)
+     *utf8_error = 0;
+   
+   while (src < srcmax)
+     {
+	if (*src & 0x80)
+	  {
+	     src = utf_to_unicode (&ch, src, srcmax);
+	     if ((ch < 0) || (ch > 65535))
+	       {
+		  if (utf8_error != NULL)
+		    *utf8_error = 1;
+		  *dest++ = '?';
+	       }
+	     else if (Slrn_Utf8_Table == NULL) /* convert to iso-8859-1 */
+	       {
+		  if (ch < 256)
+		    *dest++ = (char) ch;
+		  else
+		    *dest++ = '?';
+	       }
+	     else /* use user-defined conversion table */
+	       {
+		  char c;
+		  
+		  if (ch < 128)
+		    *dest++ = (char) ch;
+		  else if (0 != (c = Slrn_Utf8_Table[ch]))
+		    *dest++ = c;
+		  else
+		    *dest++ = '?';
+	       }
+	  }
+	else
+	  *dest++ = *src++;
+     }
+   
+   return dest;
+}
+
 int slrn_rfc1522_decode_string (char *s)
 {
    char *s1, *s2, ch;
@@ -358,6 +499,19 @@ int slrn_rfc1522_decode_string (char *s)
    after_whitespace = NULL;
    after_last_encoded_word = NULL;
 
+/* Even if some user agents still send raw 8bit, it is safe to call
+ * decode_utf8() -- if it finds 8bit chars that are not valid UTF-8, it
+ * will set ch to 1 and we can leave the line untouched. */
+   len = strlen (s);
+   s1 = slrn_safe_malloc(len + 1);
+   
+   s2 = decode_utf8 (s1, s, s + len, &ch);
+   *s2 = 0;
+   
+   if (ch == 0)
+     strcpy (s, s1); /* safe */
+   slrn_free (s1);
+   
    while (1)
      {
 	while ((NULL != (s = slrn_strchr (s, '=')))
@@ -379,7 +533,7 @@ int slrn_rfc1522_decode_string (char *s)
 	charset = s1 + 2;
 	len = s - charset;
 	
-	charset = find_compatable_charset (charset, len);
+	charset = find_compatible_charset (charset, len);
 	s++;			       /* skip ? */
 	method = *s++;		       /* skip B,Q */
 	method = UPPER_CASE(method);
@@ -421,10 +575,16 @@ int slrn_rfc1522_decode_string (char *s)
 	/* Note: these functions return a pointer to the END of the decoded
 	 * text.
 	 */
+	s2 = s1;
+	
 	if (method == 'B')
 	  s1 = decode_base64 (s1, txt, s);
 	else s1 = decode_quoted_printable (s1, txt, s, 1);
 	
+	if (slrn_case_strncmp((unsigned char *)"utf-8",
+			      (unsigned char *)charset, 5) == 0)
+	  s1 = decode_utf8 (s2, s2, s1, NULL);
+	
 	/* Now move everything over */
 	s2 = s + 2;		       /* skip final ?= */
 	s = s1;			       /* start from here next loop */
@@ -441,8 +601,7 @@ int slrn_rfc1522_decode_string (char *s)
 }
 
 
-
-
+#ifndef SLRNPULL_CODE /* rest of the file in this ifdef */
 
 static void rfc1522_decode_headers (Slrn_Article_Type *a)
 {
@@ -464,10 +623,98 @@ static void rfc1522_decode_headers (Slrn_Article_Type *a)
      }
 }
 
-
-static void decode_mime_base64 (void)
+static void decode_mime_base64 (Slrn_Article_Type *a)
 {
-   Slrn_Mime_Needs_Metamail = 1;
+   Slrn_Article_Line_Type *l;
+   Slrn_Article_Line_Type *body_start, *next;
+   char *buf_src, *buf_dest, *buf_pos;
+   char *base;
+   int len;
+   
+   if (a == NULL) return;
+   
+   l = a->lines;
+   
+   /* skip header and separator */
+   while (((l != NULL) && (l->flags & HEADER_LINE)) || l->buf[0] == '\0')
+     l = l->next;
+   
+   if (l == NULL) return;
+   
+   body_start = l;
+   
+   /* let's calculate how much space we need... */
+   len = 0;
+   while ( l )
+     {
+	len += strlen(l->buf);
+	l = l->next;
+     }
+   
+   /* get some memory */
+   buf_src = slrn_safe_malloc (len + 1);
+   buf_dest = slrn_safe_malloc (len + 1);
+   
+   /* collect all base64 encoded lines into buf_src */
+   l = body_start;
+   buf_pos = buf_src;
+   while ( l )
+     {
+	strcat (buf_pos, l->buf); /* safe */
+	buf_pos += strlen(l->buf);
+	l = l->next;
+     }
+   
+   /* put decoded article into buf_dest */
+   buf_pos = decode_base64(buf_dest, buf_src, buf_src+len);
+   *buf_pos = '\0';
+   
+   l = body_start;
+   body_start = body_start->prev;
+   
+   /* free old body */
+   while ( l )
+     {
+	slrn_free(l->buf);
+	next = l->next;
+	slrn_free((char *)l);
+	l = next;
+     }
+   body_start->next = NULL;
+   
+   l = body_start;
+   
+   base = buf_dest;
+   buf_pos = buf_dest;
+   
+   /* put decoded article back into article structure */
+   while ( (buf_pos=strchr(buf_dest, '\n')) != NULL )
+     {
+	len = buf_pos - buf_dest;
+	
+	l->next = (Slrn_Article_Line_Type *)
+	  slrn_malloc(sizeof(Slrn_Article_Line_Type), 1, 1);
+	
+	l->next->prev = l;
+	l->next->next = NULL;
+	l = l->next;
+	l->buf = slrn_malloc(sizeof(char) * len + 1, 0, 1);
+	
+	strncpy(l->buf, buf_dest, len);
+	/* terminate string and strip '\r' if necessary */
+	if ( l->buf[len-1] == '\r' )
+	  l->buf[len-1] = '\0';
+	else
+	  l->buf[len] = '\0';
+	
+	buf_dest = buf_pos + 1;
+     }
+   
+   slrn_free(buf_src);
+   slrn_free(base);
+   
+   a->is_modified = 1;
+   Slrn_Mime_Was_Modified = 1;
 }
 
 /* This function checks if the last character on curr_line is an = and 
@@ -499,7 +746,7 @@ static int merge_if_soft_linebreak (Slrn_Article_Line_Type *curr_line)
 
 	curr_line->buf = b;
 	
-	strcpy (b + len, next_line->buf);
+	strcpy (b + len, next_line->buf); /* safe */
 	
 	/* Unlink next_line from the linked list of lines in the article... */
 	curr_line->next = next_line->next;
@@ -563,7 +810,40 @@ static void decode_mime_quoted_printable (Slrn_Article_Type *a)
      }
 }
 
+static void decode_mime_utf8 (Slrn_Article_Type *a)
+{
+   Slrn_Article_Line_Type *line;
+   
+   if (a == NULL)
+     return;
+   
+   line = a->lines;
 
+   /* skip to body */
+   while ((line != NULL) && (line->flags & HEADER_LINE))
+     line = line->next;
+   if (line == NULL) return;
+   
+   while (line != NULL)
+     {
+	char *b;
+	unsigned int len;
+	
+	b = line->buf;
+	len = strlen (b);
+	
+	b = decode_utf8 (b, b, b + len, NULL);
+	
+	if (b < line->buf + len)
+	  {
+	     *b = 0;
+	     a->is_modified = 1;
+	     Slrn_Mime_Was_Modified = 1;
+	  }
+	
+	line = line->next;
+     }
+}
 
 void slrn_mime_article_init (void)
 {
@@ -597,10 +877,10 @@ void slrn_mime_process_article (Slrn_Article_Type *a)
       case ENCODED_8BIT:
       case ENCODED_BINARY:
 	/* Already done. */
-	return;
+	break;
 	
       case ENCODED_BASE64:
-	decode_mime_base64 ();
+	decode_mime_base64 (a);
 	break;
 	
       case ENCODED_QUOTED:
@@ -611,6 +891,11 @@ void slrn_mime_process_article (Slrn_Article_Type *a)
 	Slrn_Mime_Needs_Metamail = 1;
 	return;
      }
+   
+   if ((Slrn_Mime_Needs_Metamail == 0) &&
+       (slrn_case_strncmp((unsigned char *)"utf-8",
+			  (unsigned char *)Char_Set, 5) == 0))
+     decode_mime_utf8 (a);
 }
 
 #ifndef MAXPATHLEN
@@ -623,11 +908,10 @@ int slrn_mime_call_metamail (void)
    return 0;
 #else
    int init = Slrn_TT_Initialized;
-   char cmd [2 * SLRN_MAX_PATH_LEN];
    char tempfile [SLRN_MAX_PATH_LEN];
    Slrn_Article_Line_Type *ptr;
    FILE *fp;
-   char *tmp, *mm;
+   char *tmp, *mm, *cmd;
    
    if (NULL == Slrn_Current_Article)
      return -1;
@@ -645,7 +929,7 @@ int slrn_mime_call_metamail (void)
    tmp = "/tmp";
 # endif
    
-   fp = slrn_open_tmpfile_in_dir (tmp, tempfile, "w");
+   fp = slrn_open_tmpfile_in_dir (tmp, tempfile, sizeof (tempfile));
 
    if (fp == NULL)
      {
@@ -668,13 +952,14 @@ int slrn_mime_call_metamail (void)
        || (strlen (mm) > SLRN_MAX_PATH_LEN))
      mm = "metamail";
 
-   sprintf (cmd, "%s %s", mm, tempfile);
+   cmd = slrn_strdup_printf ("%s %s", mm, tempfile);
    
    /* Make sure that metamail has a normal environment */
    slrn_set_display_state (0);
    
    slrn_posix_system(cmd, 0);
    slrn_delete_file (tempfile);
+   slrn_free (cmd);
    
    printf("Press return to continue ...");
    getchar();
@@ -746,7 +1031,7 @@ int slrn_mime_scan_file (FILE *fp)
 	else
 	  Mime_Posting_Charset = Slrn_Mime_Display_Charset;
      }
-   else if (NULL != find_compatable_charset (Slrn_Mime_Display_Charset,
+   else if (NULL != find_compatible_charset (Slrn_Mime_Display_Charset,
 					     strlen (Slrn_Mime_Display_Charset)))
      /* 7-bit data.  Check to make sure that this display supports US-ASCII */
      Mime_Posting_Charset = "us-ascii";
@@ -808,16 +1093,16 @@ static int rfc1522_encode (unsigned char *d, unsigned int len,
      {
 	if (s + len < p) return -1;
 	
-	strcpy((char *)d, (char *)s);
+	strcpy ((char *)d, (char *)s); /* safe */
 	return 0;
      }
    
    if (0 == slrn_case_strcmp ((unsigned char *)Slrn_Mime_Display_Charset,
 			      (unsigned char *)"us-ascii"))
      {
-	strcpy (charset, "=?iso-8859-1?Q?");
+	strcpy (charset, "=?iso-8859-1?Q?"); /* safe */
      }
-   else sprintf (charset, "=?%s?Q?", Slrn_Mime_Display_Charset);
+   else slrn_snprintf (charset, sizeof (charset), "=?%s?Q?", Slrn_Mime_Display_Charset);
    charset_len = strlen (charset);
    
    p = s;
@@ -878,7 +1163,7 @@ static int rfc1522_encode (unsigned char *d, unsigned int len,
 	if (d + charset_len >= dmax)
 	  return -1;
 	
-	strcpy ((char *)d, charset);
+	strcpy ((char *)d, charset); /* safe */
 	d += charset_len;
 
 	/* Note: rfc1522 places a limit on the length of the encoded word.  
@@ -889,7 +1174,7 @@ static int rfc1522_encode (unsigned char *d, unsigned int len,
 	       && ((ch = *p) != 0)
 	       && ((ignore_specials != 0) || (0 == IS_RFC850_SPECIAL(ch)))
 	       && ((hibit != 0) 
-		   || ((ch != ' ') && (ch != '\t'))))
+		   || ((ch != ' ') && (ch != '\t') && (ch != '\n'))))
 	  {
 	     if (ch == ' ') *d++ = '_';
 	     else if ((ch & 0x80) 
@@ -905,7 +1190,7 @@ static int rfc1522_encode (unsigned char *d, unsigned int len,
 		       *d = 0;
 		       return -1;
 		    }
-		  sprintf ((char *)d, "=%02X", (int) ch);
+		  sprintf ((char *)d, "=%02X", (int) ch); /* safe */
 		  d += 3;
 		  if (ch & 0x80) hibit--;
 	       }
@@ -948,7 +1233,7 @@ void slrn_mime_header_encode (char *s, unsigned int bytes)
 	ignore_specials = !slrn_case_strncmp ((unsigned char *)s, 
 					      (unsigned char *)"Subject: ", 9);
 	
-	strcpy (buf, s);
+	strcpy (buf, s); /* safe */
 	if (0 == rfc1522_encode ((unsigned char *)s, bytes, 
 				 (unsigned char *)buf,
 				 ignore_specials))
@@ -1017,8 +1302,7 @@ FILE *slrn_mime_encode (FILE *fp)
    return fp;
 }
 
-
-
+#endif /* NOT SLRNPULL_CODE */
 
 #endif  /* SLRN_HAS_MIME */
 
