@@ -1,3 +1,7 @@
+#ifndef _SLRN_MIME_H
+#define _SLRN_MIME_H
+
+extern int slrn_set_compatible_charsets (char *);
 extern int slrn_rfc1522_decode_string (char *);
 
 extern FILE *slrn_mime_encode (FILE *);
@@ -10,9 +14,11 @@ extern int slrn_mime_call_metamail (void);
 extern int slrn_check_rfc1522 (char *);
 
 extern char *Slrn_Mime_Display_Charset;
+extern char *Slrn_Utf8_Table;
 extern int Slrn_Use_Mime;
 extern int Slrn_Mime_Was_Modified;
 extern int Slrn_Mime_Needs_Metamail;
 extern int Slrn_Mime_Was_Parsed;
 extern int Slrn_Use_Meta_Mail;
 extern char *Slrn_MetaMail_Cmd;
+#endif /* _SLRN_MIME_H */
