@@ -3,6 +3,7 @@
  This file is part of SLRN.
 
  Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
+ Copyright (c) 2001 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -51,6 +52,13 @@
 # include <sys/stat.h>
 #endif
 
+#ifdef HAVE_FCNTL_H
+# include <fcntl.h>
+#endif
+#ifdef HAVE_SYS_FCNTL_H
+# include <sys/fcntl.h>
+#endif
+
 #if defined(VMS) && defined(MULTINET)
 # include "multinet_root:[multinet.include]netdb.h"
 #else
@@ -97,6 +105,7 @@ extern int h_errno;
 #include "ttymsg.h"
 #include "art.h"
 #include "chmap.h"
+#include "snprintf.h"
 
 #if SLRN_HAS_MIME
 #include "mime.h"
@@ -124,6 +133,8 @@ char *Slrn_Editor_Post;
 char *Slrn_Editor_Score;
 char *Slrn_Editor_Mail;
 
+char *Slrn_Top_Status_Line;
+
 Slrn_User_Info_Type Slrn_User_Info;
 SLKeyMap_List_Type *Slrn_RLine_Keymap;
 
@@ -211,13 +222,138 @@ void slrn_print_percent (int row, int col, SLscroll_Window_Type *w) /*{{{*/
 
 void slrn_update_top_status_line (void) /*{{{*/
 {
+   char *fmt;
+   char ch;
+   
    if (Slrn_Full_Screen_Update == 0) return;
+   
+   fmt = Slrn_Top_Status_Line;
+   if (fmt == NULL)
+     fmt = "slrn %v ** Press '?' for help, 'q' to quit. ** Server: %s";
+
    SLsmg_gotorc (0, 0);
    slrn_set_color (MENU_COLOR);
-   SLsmg_printf ("\
-slrn %s ** Press '?' for help, 'q' to quit. ** Server: %s",
-		 Slrn_Version,
-		 Slrn_Server_Obj->sv_name);
+   
+   while ((ch = *fmt) != 0)
+     {
+	char *s;
+	int len;
+	int field_len;
+	char buf[32];
+	int right_justify;
+	int spaces = 0;
+	time_t now;
+
+	fmt++;
+	if (ch != '%')
+	  {
+	     SLsmg_write_nchars (&ch, 1);
+	     continue;
+	  }
+
+	ch = *fmt++;
+
+	s = NULL;
+	len = -1;
+
+	field_len = -1;
+	if (ch == '-')
+	  {
+	     right_justify = 1;
+	     ch = *fmt++;
+	  }
+	else
+	  right_justify = 0;
+
+	if (isdigit (ch))
+	  {
+	     field_len = 0;
+	     do
+	       {
+		  field_len = 10 * field_len + (int) (ch - '0');
+		  ch = *fmt++;
+	       }
+	     while (isdigit (ch));
+	  }
+
+	switch (ch)
+	  {
+	   case 0:
+	     slrn_set_color (0);
+	     SLsmg_erase_eol ();
+	     return;
+	     
+	   default:
+	     break;
+	     
+	   case '%':
+	     s = "%";
+	     len = 1;
+	     break;
+	     
+	   case 'd':
+	     time(&now);
+	     len = strftime(buf, sizeof(buf), "%x", localtime(&now));
+	     if (len == 0) s = NULL;
+	     else s = buf;
+	     break;
+	     
+	   case 'n':
+	     if (NULL != Slrn_Group_Current_Group)
+	       s = Slrn_Group_Current_Group->group_name;
+	     break;
+	     
+	   case 's':
+	     s = Slrn_Server_Obj->sv_name;
+	     break;
+	     
+	   case 't':
+	     time(&now);
+	     len = strftime(buf, sizeof(buf), "%X", localtime(&now));
+	     if (len == 0) s = NULL;
+	     else s = buf;
+	     break;
+	     
+	   case 'v':
+	     s = Slrn_Version;
+	     break;
+	     
+           case 'g':
+             SLsmg_erase_eol ();
+             if (right_justify)
+               field_len = SLtt_Screen_Cols - field_len;
+             SLsmg_gotorc (0, field_len);
+             break;
+	  }
+
+	if (s == NULL)
+	  continue;
+
+	if (len == -1)
+	  len = strlen (s);
+	if (field_len != -1)
+	  {
+	     if (field_len > len)
+	       spaces = field_len - len;
+	     else 
+	       len = field_len;
+	  }
+	if (right_justify)
+	  {
+	     while (spaces)
+	       {
+		  SLsmg_write_nchars (" ", 1);
+		  spaces--;
+	       }
+	  }
+	SLsmg_write_nchars (s, len);
+	while (spaces)
+	  {
+	     spaces--;
+	     SLsmg_write_nchars (" ", 1);
+	  }
+     }
+
    SLsmg_erase_eol ();
    slrn_set_color (0);
 }
@@ -275,11 +411,7 @@ static void redraw_message (void)
      
 static void vmessage_1 (int color, char *fmt, va_list ap)
 {
-#ifdef HAVE_VSNPRINTF
-   vsnprintf (Message_Buffer + 1, sizeof(Message_Buffer)-1, fmt, ap);
-#else
-   vsprintf (Message_Buffer + 1, fmt, ap);
-#endif
+   slrn_vsnprintf (Message_Buffer + 1, sizeof(Message_Buffer)-1, fmt, ap);
 
    Message_Buffer[0] = (char) color;
    Slrn_Message_Present = 1;
@@ -425,20 +557,20 @@ static void vms_fix_name(char *name)
 static char Copystr[SLRN_MAX_PATH_LEN];
 static int vms_copyname1 (char *name)
 {
-   strcpy(Copystr, name);
+   slrn_strncpy(Copystr, name, sizeof (Copystr));
    return(1);
 }
 
 static int vms_copyname2 (char *name, int type)
 {
-   strcpy(Copystr, name);
+   slrn_strncpy(Copystr, name, sizeof (Copystr));
    return(1);
 }
 
 /*}}}*/
 #endif
 
-void slrn_make_home_filename (char *name, char *file) /*{{{*/
+void slrn_make_home_filename (char *name, char *file, size_t n) /*{{{*/
 {
    char *home;
 #ifndef VMS
@@ -449,7 +581,7 @@ void slrn_make_home_filename (char *name, char *file) /*{{{*/
 #endif
        )
      {
-	strcpy (file, name);
+	slrn_strncpy (file, name, n);
 #if defined(IBMPC_SYSTEM)
 	slrn_os2_convert_path (file);
 #endif
@@ -460,17 +592,17 @@ void slrn_make_home_filename (char *name, char *file) /*{{{*/
      home = getenv ("HOME");
    
    *file = 0;
-   slrn_dircat (home, name, file);
+   slrn_dircat (home, name, file, n);
 #else
    char *cp, *cp1;
    static char fname[SLRN_MAX_PATH_LEN];
    char fn[SLRN_MAX_PATH_LEN], fn1[SLRN_MAX_PATH_LEN];
    int rc, idx;
    
-   strcpy (fn1, name);
+   slrn_strncpy (fn1, name, sizeof (fn1));
    if (NULL != slrn_strchr (name, ':'))
      {
-	strcpy (file, name);
+	slrn_strncpy (file, name, n);
 	return;
      }
    
@@ -487,20 +619,21 @@ void slrn_make_home_filename (char *name, char *file) /*{{{*/
 	  { /* error translating */ }
 	else
 	  {
-	     strcpy(fname, cp1);
-	     strcat(cp1, "/");
+	     slrn_strncpy(fname, cp1, sizeof (fname));
+	     strcat(cp1, "/"); /* safe ? */
  	  }
-	strcat (cp1, fn1);
+	strcat (cp1, fn1); /* safe ? */
 	
 	vms_fix_name (cp1);
  	
 	rc = decc$to_vms(cp1, vms_copyname2, 0, 2);
  	if (rc > 0)
  	  {
- 	     strcpy(fname, Copystr);
+ 	     slrn_strncpy(fname, Copystr, sizeof (fname));
  	     rc = mkdir(fname, 0755);
  	  }
-	strcat(fname, cp);
+	if (strlen (fname) + strlen (cp) < sizeof (fname))
+	  strcat(fname, cp); /* safe */
 # else
 	*cp = '\0'; cp++;
 	cp1 = shell$translate_vms(home);
@@ -508,38 +641,40 @@ void slrn_make_home_filename (char *name, char *file) /*{{{*/
 	  { /* error translating */ }
 	else
  	  {
-	     strcpy(fname, cp1);
-	     strcat(cp1, "/");
+	     slrn_strncpy(fname, cp1, sizeof (fname));
+	     strcat(cp1, "/"); /* safe ? */
 	  }
-	strcat (cp1, fn1);
+	strcat (cp1, fn1); /* safe ? */
 	
 	vms_fix_name (cp1);
  	
 	rc = shell$to_vms(cp1, vms_copyname2, 0, 2);
 	if (rc > 0)
 	  {
-	     strcpy(fname, Copystr);
+	     slrn_strncpy(fname, Copystr, sizeof (fname));
 	     rc = mkdir(fname, 0755);
 	  }
-	strcat(fname, cp);
+	if (strlen (fname) + strlen (cp) < sizeof (fname))
+	  strcat(fname, cp); /* safe */
 # endif
-	strcpy(file,fname);
+	slrn_strncpy (file, fname, n);
      }
    else
      {
-	if (home != NULL) strcpy(file, home);
-	strcat(file, name);
+	if (home != NULL) slrn_strncpy(file, home, n);
+	if (strlen (file) + strlen (name) < n)
+	  strcat (file, name); /* safe */
      }
 #endif /* VMS */
 }
 
 /*}}}*/
 
-int slrn_make_home_dirname (char *name, char *dir) /*{{{*/
+int slrn_make_home_dirname (char *name, char *dir, size_t n) /*{{{*/
 {
    /* This needs modified to deal with VMS directory syntax */
 #ifndef VMS
-   slrn_make_home_filename (name, dir);
+   slrn_make_home_filename (name, dir, n);
 #else
    char *home, *cp;
    char fn[SLRN_MAX_PATH_LEN];
@@ -548,7 +683,7 @@ int slrn_make_home_dirname (char *name, char *dir) /*{{{*/
    
    if (NULL != slrn_strchr (name, ':'))
      {
-	strcpy (dir, name);
+	slrn_strncpy (dir, name, n);
 	return 0;
      }
    home = getenv ("HOME");
@@ -561,48 +696,54 @@ int slrn_make_home_dirname (char *name, char *dir) /*{{{*/
 	  { /* error translating */ }
 	else
 	  {
-	     strcpy(fname, cp);
-	     strcat(cp, "/");
+	     slrn_strncpy(fname, cp, sizeof (fname));
+	     strcat(cp, "/"); /* safe ? */
 	  }
-	strcat (cp, name);
+	strcat (cp, name); /* safe ? */
 	vms_fix_name (cp);
 	
 	rc = decc$to_vms(cp, vms_copyname2, 0, 2);
 	if (rc > 0)
 	  {
-	     strcpy(fname, Copystr);
+	     slrn_strncpy(fname, Copystr, sizeof (fname));
 	     rc = mkdir(fname, 0755);
 	  }
 #else
 	if (shell$from_vms(home, vms_copyname1, 0))
 	  {
-	     if (Copystr != NULL) strcpy (fn, Copystr);
-	     strcat(fn, "/");
+	     if (Copystr != NULL)
+	       slrn_strncpy (fn, Copystr, sizeof (fn));
+	     if (strlen (fn) + 1 < sizeof (fn))
+	       strcat(fn, "/"); /* safe */
 	  }
-	strcat (fn, name);
+	if (strlen (fn) + strlen (name) < sizeof (fn))
+	  strcat (fn, name); /* safe */
 	vms_fix_name(fn);
 	if (shell$to_vms(fn, vms_copyname1, 0))
-	  strcpy(fname, Copystr);
+	  slrn_strncpy (fname, Copystr, sizeof (fname));
 #endif
-	strcpy(dir,fname);
+	slrn_strncpy (dir, fname, n);
      }
    else
      {
 	if (home != NULL) 
-	 {
-	  strcpy(dir, home);
-	  len = strlen(dir) - 1;
-	  if (dir[len] == ']')
-	   {
-	    dir[len] = '.';
-	    strcat(dir, name);
-	    strcat(dir, "]");
-	   }
-	  else
-	    strcat(dir, name);
+	  {
+	     slrn_strncpy(dir, home, n);
+	     len = strlen(dir) - 1;
+	     if (dir[len] == ']')
+	       {
+		  if (len + 3 + strlen (name) < n)
+		    {
+		       dir[len] = '.';
+		       strcat(dir, name); /* safe */
+		       strcat(dir, "]"); /* safe */
+		    }
+	       }
+	     else if (len + 1 + strlen (name) < n)
+	       strcat(dir, name); /* safe */
 	 }
-	else
-	  strcat(dir, name);
+	else if (strlen (dir) + strlen (name) < n)
+	  strcat(dir, name); /* safe */
      }
 #endif /* VMS */
    
@@ -636,7 +777,7 @@ static unsigned int make_random (void)
  * is closed.
  */
 
-FILE *slrn_open_tmpfile_in_dir (char *dir, char *file, char *mode)
+FILE *slrn_open_tmpfile_in_dir (char *dir, char *file, size_t n)
 {
    FILE *fp;
    unsigned int len;
@@ -649,12 +790,12 @@ FILE *slrn_open_tmpfile_in_dir (char *dir, char *file, char *mode)
 #endif
 
 #if defined(IBMPC_SYSTEM)
-   sprintf (buf, "SLRN%04u", make_random ());
+   slrn_snprintf (buf, sizeof (buf), "SLRN%04u", make_random ());
 #else
-   sprintf (buf, "SLRN%X", make_random ());
+   slrn_snprintf (buf, sizeof (buf), "SLRN%X", make_random ());
 #endif
    
-   if (-1 == slrn_dircat (dir, buf, file))
+   if (-1 == slrn_dircat (dir, buf, file, n))
      return NULL;
    
 #if defined(IBMPC_SYSTEM)
@@ -666,23 +807,26 @@ FILE *slrn_open_tmpfile_in_dir (char *dir, char *file, char *mode)
    len = strlen (file);
    for (i = 0; i < MAX_TMP_FILE_NUMBER; i++)
      {
-	sprintf (file + len, ".%u", i);
+	int fd;
 	
-	/* I wish this were an atomic operation but I do not know of a
-	 * portable way to achieve that using standard C.
-	 */
-	if ((0 == slrn_file_exists (file))	    
-	    && (NULL != (fp = fopen (file, mode))))
+	if (len + 5 < n)
+	  sprintf (file + len, ".%u", i); /* safe */
+	else if (i)
+	  break;
+	
+	fd = open (file, O_WRONLY | O_CREAT | O_EXCL, S_IREAD | S_IWRITE);
+	if (fd != -1)
 	  {
-	     return fp;
+	     if (NULL == (fp = fdopen (fd, "w")))
+	       close (fd);
+	     else
+	       return fp;
 	  }
      }
    return NULL;
 }
 
-	  
-
-FILE *slrn_open_tmpfile (char *file, char *mode) /*{{{*/
+FILE *slrn_open_tmpfile (char *file, size_t n) /*{{{*/
 {
    char *dir;
    
@@ -703,18 +847,23 @@ FILE *slrn_open_tmpfile (char *file, char *mode) /*{{{*/
 #endif
      }
    
-   return slrn_open_tmpfile_in_dir (dir, file, mode);
+   return slrn_open_tmpfile_in_dir (dir, file, n);
 }
 
 /*}}}*/
 
-FILE *slrn_open_home_file (char *name, char *mode, char *file, int create_flag) /*{{{*/
+FILE *slrn_open_home_file (char *name, char *mode, char *file, /*{{{*/
+			   size_t n, int create_flag)
 {
    char filebuf[SLRN_MAX_PATH_LEN];
    
-   if (file == NULL) file = filebuf;
+   if (file == NULL)
+     {
+	file = filebuf;
+	n = sizeof (filebuf);
+     }
 
-   slrn_make_home_filename (name, file);
+   slrn_make_home_filename (name, file, n);
    
 #ifdef VMS
    if (create_flag)
@@ -732,13 +881,17 @@ FILE *slrn_open_home_file (char *name, char *mode, char *file, int create_flag)
 
 /*}}}*/
 
-VFILE *slrn_open_home_vfile (char *name, char *file)
+VFILE *slrn_open_home_vfile (char *name, char *file, size_t n)
 {
    char filebuf[SLRN_MAX_PATH_LEN];
    
-   if (file == NULL) file = filebuf;
+   if (file == NULL)
+     {
+	file = filebuf;
+	n = sizeof (filebuf);
+     }
 
-   slrn_make_home_filename (name, file);
+   slrn_make_home_filename (name, file, n);
    
    return vopen (file, 4096, 0);
 }
@@ -747,7 +900,7 @@ VFILE *slrn_open_home_vfile (char *name, char *file)
 
 int slrn_mail_file (char *file, int edit, unsigned int editline, char *to, char *subject) /*{{{*/
 {
-   char buf [256 + 2 * SLRN_MAX_PATH_LEN];
+   char *buf;
 #if defined(IBMPC_SYSTEM)
    char outfile [SLRN_MAX_PATH_LEN];
 #endif
@@ -769,20 +922,22 @@ int slrn_mail_file (char *file, int edit, unsigned int editline, char *to, char
      }
    slrn_message_now ("Sending ...");
 
+   slrn_chmap_fix_file (file, 0);
+   
    if (-1 == slrn_save_file_to_mail_file (file, Slrn_Save_Replies_File, NULL))
      return -1;
 
-   slrn_chmap_fix_file (file);
-
 #ifdef VMS
-   sprintf(buf, "%s\"%s\"", MAIL_PROTOCOL, to);
-   vms_send_mail( buf, subject, file );
+   buf = slrn_strdup_printf ("%s\"%s\"", MAIL_PROTOCOL, to);
+   vms_send_mail (buf, subject, file);
+   slrn_free (buf);
 #else
    (void) to; (void) subject;
 
    /* What I need to do is to open the file and feed it line by line to the
     * sendmail program.  This way I can strip out blank headers.
     */
+# if SLRN_HAS_MIME
    if (Slrn_Use_Mime)
      {
 	FILE *fp, *pp;
@@ -793,11 +948,11 @@ int slrn_mail_file (char *file, int edit, unsigned int editline, char *to, char
 	if (fp == NULL) return (-1);
 	
 	slrn_mime_scan_file (fp);
-# if defined(IBMPC_SYSTEM)
-	pp = slrn_open_tmpfile (outfile, "w");
-# else
+#  if defined(IBMPC_SYSTEM)
+	pp = slrn_open_tmpfile (outfile, sizeof (outfile));
+#  else
 	pp = slrn_popen (Slrn_SendMail_Command, "w");
-# endif
+#  endif
 	if (pp == NULL)
 	  {
 	     slrn_fclose (fp);
@@ -824,26 +979,35 @@ int slrn_mail_file (char *file, int edit, unsigned int editline, char *to, char
 		    }
 		  slrn_mime_header_encode (line, sizeof(line));
 	       }
-	     fputs (line, pp);
-	     putc('\n', pp);
+	     if ((Slrn_Generate_Email_From) ||
+		 (header == 0) ||
+		 (slrn_case_strncmp ((unsigned char *)"From: ",
+				     (unsigned char *)line, 6)))
+	       {
+		  fputs (line, pp);
+		  putc('\n', pp);
+	       }
 	  }
 	slrn_fclose (fp);
-#if defined(IBMPC_SYSTEM)
+#  if defined(IBMPC_SYSTEM)
 	slrn_fclose (pp);
-	sprintf (buf, "%s %s", Slrn_SendMail_Command, outfile);
+	buf = slrn_strdup_printf ("%s %s", Slrn_SendMail_Command, outfile);
 	slrn_posix_system (buf, 0);
-#else
+	slrn_free (buf);
+#  else
 	slrn_pclose (pp);
-#endif
+#  endif
      }
    else
+# endif /* SLRN_HAS_MIME */
      {
-#if defined(IBMPC_SYSTEM)
-	sprintf (buf, "%s %s", Slrn_SendMail_Command, file);
-#else
-	sprintf (buf, "%s < %s", Slrn_SendMail_Command, file);
-#endif
+# if defined(IBMPC_SYSTEM)
+	buf = slrn_strdup_printf ("%s %s", Slrn_SendMail_Command, file);
+# else
+	buf = slrn_strdup_printf ("%s < %s", Slrn_SendMail_Command, file);
+# endif
 	slrn_posix_system (buf, 0);
+	slrn_free (buf);
      }
 #endif /* NOT VMS */
    slrn_message ("Sending...done");
@@ -1001,7 +1165,7 @@ FILE *slrn_popen (char *cmd, char *mode) /*{{{*/
 
 /*}}}*/
 
-static int create_edit_command (char *edit, char *cmd, char *file, unsigned int line) /*{{{*/
+static char *create_edit_command (char *edit, char *file, unsigned int line) /*{{{*/
 {
    int d, s;
    char ch, *p = edit;
@@ -1025,7 +1189,7 @@ static int create_edit_command (char *edit, char *cmd, char *file, unsigned int
 	else
 	  {
 	     slrn_error ("Invalid Editor definition.");
-	     return 0;
+	     return NULL;
 	  }
 	p++;
      }
@@ -1046,25 +1210,28 @@ static int create_edit_command (char *edit, char *cmd, char *file, unsigned int
    
    if ((d == 0) && (s == 0))
      {
-	sprintf (cmd, "%s %s", edit, file);
+	return slrn_strdup_printf ("%s %s", edit, file);
      }
    else if (d == 0)
      {
-	sprintf (cmd, edit, file);
+	return slrn_strdup_printf (edit, file);
      }
    else if (s == 0)
      {
-	sprintf (cmd, edit, (int) line);
-	strcat (edit, " ");
-	strcat (edit, file);
+	char *retval, *cmd1;
+	cmd1 = slrn_strdup_printf (edit, (int) line);
+	retval = slrn_strdup_printf ("%s %s", cmd1, file);
+	slrn_free (cmd1);
+	return retval;
      }
    else /* d and s */
      {
 	if (d == 1)
-	  sprintf (cmd, edit, line, file);
-	else sprintf (cmd, edit, file, line);
+	  return slrn_strdup_printf (edit, line, file);
+	else return slrn_strdup_printf (edit, file, line);
      }
-   return 1;
+   /* We should never get here */
+   return NULL;
 }
 
 /*}}}*/
@@ -1073,8 +1240,7 @@ static int create_edit_command (char *edit, char *cmd, char *file, unsigned int
 int slrn_edit_file (char *editor, char *file, unsigned int line, 
 		    int check_mtime) /*{{{*/
 {
-   char buf[1024];
-   char editbuf[1024];
+   char *cmd, *editcmd, *msg = NULL;
    int ret;
    unsigned long mtime = 0;
    struct stat st;
@@ -1082,16 +1248,16 @@ int slrn_edit_file (char *editor, char *file, unsigned int line,
    if (Slrn_Abort_Unmodified == 0)
      check_mtime = 0;
      
-   if (editor == NULL)
+   if ((editor == NULL) || (*editor == 0))
      editor = Slrn_Editor;
 
-   if ((editor == NULL)
+   if (((editor == NULL) || (*editor == 0))
        && (NULL == (editor = getenv("SLRN_EDITOR")))
        && (NULL == (editor = getenv("SLANG_EDITOR")))
        && (NULL == (editor = getenv("EDITOR")))
        && (NULL == (editor = getenv("VISUAL"))))
      {
-#if defined(VMS) || defined(__NT__) || defined(__MINGW32__)
+#if defined(VMS) || defined(__NT__) || defined(__WIN32__)
 	editor = "edit";
 #else
 # if defined(__os2__)
@@ -1099,21 +1265,40 @@ int slrn_edit_file (char *editor, char *file, unsigned int line,
 # else
 #  ifdef __unix__
 	editor = "vi";
+#  else
+	slrn_error ("No editor command defined.");
+	return -1;
 #  endif
 # endif
 #endif
      }
-   
-   strcpy (editbuf, editor);
+   editcmd = slrn_safe_strmalloc (editor);
 
-   if (0 == create_edit_command(editbuf, buf, file, line)) return -1;
+   if (NULL == (cmd = create_edit_command (editcmd, file, line))) return -1;
 
    if (check_mtime
        && (0 == stat (file, &st)))
      mtime = (unsigned long) st.st_mtime;
-       
-   ret = slrn_posix_system (buf, 1);
 
+   ret = slrn_posix_system (cmd, 1);
+
+#if defined(WIFEXITED) && defined(WEXITSTATUS)
+   if ((ret != -1) && WIFEXITED(ret))
+     {
+	ret = WEXITSTATUS(ret);
+	if (ret == 127)
+	  {
+	     msg = "The editor could not be found.";
+	     ret = -1;
+	  }
+	else if (ret == 126)
+	  {
+	     msg = "The editor was found, but could not be executed.";
+	     ret = -1;
+	  }
+     }
+#endif
+   
    /* Am I the only one who thinks this is a good idea?? */
    if (Slrn_TT_Initialized) while (SLang_input_pending (5))
      SLang_flush_input ();
@@ -1123,10 +1308,22 @@ int slrn_edit_file (char *editor, char *file, unsigned int line,
        && (0 == stat (file, &st))
        && (mtime == (unsigned long) st.st_mtime))
      {
-	slrn_message ("File was not modified");
+	msg = "File was not modified";
 	ret = -2;
      }
 
+#ifdef IBMPC_SYSTEM
+   if ((strlen (cmd) > 1)
+       && (cmd[1] == ':')
+       && (cmd[2] != '\\'))
+     {
+	msg = "Please use double '\\\\' to separate directories in editor_command";
+     }
+#endif
+
+   if (msg != NULL) slrn_message (msg);
+   slrn_free (cmd);
+   slrn_free (editcmd);
    return ret;
 }
 
@@ -1264,6 +1461,7 @@ void slrn_set_input_chars (char *s)
    Input_Chars_Ptr = s;
 }
 
+/* str needs to have enough space for 256 characters */
 static int generic_read_input (char *prompt, char *dfl, char *str, int trim_flag, 
 			       int no_echo, int point) /*{{{*/
 {
@@ -1275,19 +1473,18 @@ static int generic_read_input (char *prompt, char *dfl, char *str, int trim_flag
    
    Slrn_Full_Screen_Update = 1;
    
-   strncpy (prompt_buf, prompt, sizeof (prompt_buf));
-   prompt_buf[sizeof(prompt_buf) - 1] = 0;
+   slrn_strncpy (prompt_buf, prompt, sizeof (prompt_buf));
    len = strlen (prompt);
    if (len + 3 < sizeof (prompt_buf))
      {
 	if (len && (prompt[len - 1] != '?'))
 	  {
-	     strcpy (prompt_buf + len, ": ");
+	     strcpy (prompt_buf + len, ": "); /* safe */
 	     len += 2;
 	  }
 	else 
 	  {
-	     strcpy (prompt_buf + len, " ");
+	     strcpy (prompt_buf + len, " "); /* safe */
 	     len++;
 	  }
      }
@@ -1296,8 +1493,8 @@ static int generic_read_input (char *prompt, char *dfl, char *str, int trim_flag
      {
 	if (len + 13 + strlen (dfl) < sizeof (prompt_buf))
 	  {
-	     sprintf (prompt_buf + len,
-		      "(default: %s) ", dfl);
+	     sprintf (prompt_buf + len, /* safe */
+		      "(default: %s) ", dfl); 
 	  }
 	else dfl = NULL;
      }
@@ -1313,7 +1510,8 @@ static int generic_read_input (char *prompt, char *dfl, char *str, int trim_flag
 
    if ((str != NULL) && *str)
      {
-	strcpy ((char *) Slrn_Keymap_RLI->buf, str);
+	slrn_strncpy ((char *) Slrn_Keymap_RLI->buf, str,
+		      Slrn_Keymap_RLI->buf_len);
 	if (point == 0)
 	  Slrn_Keymap_RLI->point = 0;
 	else
@@ -1377,7 +1575,7 @@ static int generic_read_input (char *prompt, char *dfl, char *str, int trim_flag
 
 	/* b could be equal to dfl and dfl could be equal to str.  If this is
 	 * the case, there is no need to perform the strcpy */
-	if (b != str) strcpy (str, b);
+	if (b != str) strcpy (str, b); /* safe */
 	i = strlen (str);
      }
    
@@ -1424,7 +1622,7 @@ int slrn_read_integer (char *prompt, int *dflt, int *np) /*{{{*/
    
    if (dflt != NULL)
      {
-	sprintf (sdfl_buf, "%d", *dflt);
+	sprintf (sdfl_buf, "%d", *dflt); /* safe */
 	sdfl = sdfl_buf;
      }
    
@@ -1522,7 +1720,7 @@ int slrn_get_yesno (int dflt, char *str, ...) /*{{{*/
    /* if (SLang_Error) return -1; */
    
    va_start(ap, str);
-   (void) vsprintf(buf, str, ap);
+   (void) slrn_vsnprintf(buf, sizeof (buf) - 15, str, ap);
    va_end(ap);
    
    if (dflt)
@@ -1536,7 +1734,7 @@ int slrn_get_yesno (int dflt, char *str, ...) /*{{{*/
 	fmt = "? \001Yes, [\001N]o";
      }
    
-   strcat (buf, fmt);
+   strcat (buf, fmt); /* safe */
    rsp = slrn_get_response ("yYnN\r", buf);
    if (rsp == '\r') rsp = ch;
    else rsp |= 0x20;
@@ -1553,14 +1751,11 @@ int slrn_get_yesno_cancel (char *str, ...) /*{{{*/
    
    if (SLang_Error) return -1;
    
-   if (strlen (str) >= sizeof (buf) + 25)
-     return -1;
-   
    va_start(ap, str);
-   (void) vsprintf(buf, str, ap);
+   (void) slrn_vsnprintf(buf, sizeof(buf) - 25, str, ap);
    va_end(ap);
    
-   strcat (buf, "? [\001Y]es, \001No, \001Cancel");
+   strcat (buf, "? [\001Y]es, \001No, \001Cancel"); /* safe */
    
    switch (slrn_get_response ("\007yYnNcC\r", buf))
      {
@@ -1739,7 +1934,10 @@ void slrn_get_user_info (void) /*{{{*/
    Slrn_User_Info.followup_string = slrn_safe_strmalloc ("In article %m, %r wrote:");
    Slrn_User_Info.reply_string = slrn_safe_strmalloc ("In %n, you wrote:");
 
-   Slrn_Courtesy_CC_Message = slrn_safe_strmalloc ("[This message has also been posted.]");
+   Slrn_CC_Followup_Message =
+     slrn_safe_strmalloc ("[This message has also been posted to %n.]");
+   Slrn_CC_Post_Message =
+     slrn_safe_strmalloc ("[This message has also been posted.]");
    
    /* Now get default values for rest. */
    host = get_hostname ();
@@ -1747,35 +1945,15 @@ void slrn_get_user_info (void) /*{{{*/
      {
 	if (slrn_is_fqdn (host))
 	  Slrn_User_Info.posting_host = slrn_safe_strmalloc (host);
-#if defined(USE_DOMAIN_NAME) && defined(MY_DOMAIN_NAME)
-	else if (MY_DOMAIN_NAME != NULL)
-	  {
-	     char *my_domain_name;
-	     unsigned int len;
-
-	     my_domain_name = MY_DOMAIN_NAME;
-	     if (*my_domain_name == '.')
-	       my_domain_name++;
-
-	     len = strlen (host);
-	     host1 = slrn_safe_malloc (len + strlen (my_domain_name) + 2);
-	     strcpy (host1, host);
-	     host1[len] = '.';
-	     strcpy (host1 + len + 1, my_domain_name);
-	     slrn_free (host);
-	     host = host1;
-	  }
-#endif				       /* USE_DOMAIN_NAME */
      }
 
-   if (Slrn_User_Info.posting_host == NULL)
-     slrn_message_now ("***Warning: unable to find a fully-qualified host name.");
-
+#if ! SLRN_HAS_STRICT_FROM
    /* Allow user chance to specify another hostname.  However, it will not
     * affect the posting host.
     */
    if ((NULL != (host1 = getenv ("HOSTNAME")))
        && (0 == slrn_is_fqdn (host1)))
+#endif
      host1 = NULL;
 
    /* If a value was compiled in, use it. */
@@ -1815,8 +1993,11 @@ void slrn_get_user_info (void) /*{{{*/
 #endif
    
    if (((name == NULL) || (*name == 0))
+#if ! SLRN_HAS_STRICT_FROM       
        && ((name = getenv("USER")) == NULL)
-       && ((name = getenv("LOGNAME")) == NULL))
+       && ((name = getenv("LOGNAME")) == NULL)
+#endif
+       )
      name = "";
 
    Slrn_User_Info.username = slrn_safe_strmalloc (name);
@@ -1829,7 +2010,12 @@ void slrn_get_user_info (void) /*{{{*/
 #ifdef VMS
    Slrn_User_Info.realname = slrn_vms_fix_fullname(slrn_vms_get_uaf_fullname());
 #else
-   if (((Slrn_User_Info.realname = getenv ("NAME")) == NULL)
+# if ! SLRN_HAS_STRICT_FROM
+   Slrn_User_Info.realname = getenv ("NAME");
+# else
+   Slrn_User_Info.realname = NULL;
+# endif
+   if ((Slrn_User_Info.realname == NULL)
 # ifdef HAS_PASSWORD_CODE
        && ((pw == NULL) || ((Slrn_User_Info.realname = pw->pw_gecos) == NULL))
 # endif
@@ -1874,10 +2060,6 @@ void slrn_get_user_info (void) /*{{{*/
    
    Slrn_User_Info.signature = slrn_safe_strmalloc (SLRN_SIGNATURE_FILE);
    
-#if SLRN_HAS_MIME
-   Slrn_Mime_Display_Charset = slrn_safe_strmalloc ("iso-8859-1");
-#endif
-   
 #ifdef SLRN_SENDMAIL_COMMAND
    Slrn_SendMail_Command = slrn_safe_strmalloc (SLRN_SENDMAIL_COMMAND);
 #endif
@@ -1885,36 +2067,18 @@ void slrn_get_user_info (void) /*{{{*/
 
 /*}}}*/
 
-static char *_slrn_strcat (char *a, char *b, unsigned int n)
-{
-   unsigned int max;
-   unsigned int len = strlen (a);
-   
-   n--;
-   
-   if (n > len)
-     {
-	max = n - len;
-	strncpy (a + len, b, max);
-	a[n] = 0;
-     }
-   return a;
-}
-
 char *slrn_make_from_string (void)
 {
    char *username, *hostname, *realname;
    static char buf[256];
 
-   *buf = 0;
-
-#if SLRN_HAS_SLANG
+#if SLRN_HAS_SLANG && ! SLRN_HAS_STRICT_FROM
    if ((1 == SLang_run_hooks ("make_from_string_hook", 0))
        && (0 == SLang_pop_slstring (&username)))
      {
 	if (*username != 0)
 	  {
-	     _slrn_strcat (buf, username, sizeof (buf));
+	     slrn_strncpy (buf, username, sizeof (buf));
 	     SLang_free_slstring (username);
 	     return buf;
 	  }
@@ -1933,13 +2097,13 @@ char *slrn_make_from_string (void)
      realname = "";
 
    /* Build string of the form "username@hostname (realname)" */
-   _slrn_strcat (buf, username, sizeof(buf));
-   _slrn_strcat (buf, "@", sizeof(buf));
-   _slrn_strcat (buf, hostname, sizeof(buf));
-   _slrn_strcat (buf, " (", sizeof(buf));
-   _slrn_strcat (buf, realname, sizeof(buf));
-   _slrn_strcat (buf, ")", sizeof(buf));
-   
+   slrn_snprintf (buf, sizeof (buf), "%s@%s", username, hostname);
+   if (*realname != 0)
+     {
+	char *p = buf + strlen (buf);
+	if (strlen (realname) + 3 < sizeof(buf) - (size_t) (p - buf))
+	  sprintf (p, " (%s)", realname); /* safe */
+     }
    return buf;
 }
 
