@@ -2,6 +2,7 @@
  This file is part of SLRN.
 
  Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
+ Copyright (c) 2001 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -25,8 +26,8 @@
 #include "ttymsg.h"
 #include "vfile.h"
 
-extern void slrn_make_home_filename (char *, char *);
-extern int slrn_make_home_dirname (char *, char *);
+extern void slrn_make_home_filename (char *, char *, size_t);
+extern int slrn_make_home_dirname (char *, char *, size_t);
 extern void slrn_redraw (void);
 extern void slrn_update_screen (void);
 extern int Slrn_Full_Screen_Update;
@@ -34,8 +35,8 @@ extern int slrn_get_yesno (int, char *, ...);
 extern int slrn_get_yesno_cancel (char *str, ...);
 extern void slrn_clear_message (void);
 extern void slrn_print_percent (int, int, SLscroll_Window_Type *);
-extern FILE *slrn_open_home_file (char *, char *, char *, int);
-extern VFILE *slrn_open_home_vfile (char *, char *);
+extern FILE *slrn_open_home_file (char *, char *, char *, size_t, int);
+extern VFILE *slrn_open_home_vfile (char *, char *, size_t);
 extern void slrn_suspend_cmd (void);
 extern int slrn_read_integer (char *, int *, int *);
 extern int slrn_read_input (char *, char *, char *, int, int);
@@ -96,8 +97,8 @@ extern int slrn_pclose (FILE *);
 
 
 extern int Slrn_Use_Tmpdir;
-extern FILE *slrn_open_tmpfile_in_dir (char *, char *, char *);
-extern FILE *slrn_open_tmpfile (char *, char *);
+extern FILE *slrn_open_tmpfile_in_dir (char *, char *, size_t);
+extern FILE *slrn_open_tmpfile (char *, size_t);
 
 extern int slrn_posix_system (char *, int);
 extern char *Slrn_Editor;
@@ -107,6 +108,8 @@ extern char *Slrn_Editor_Mail;
 
 extern char *Slrn_Failed_Post_Filename;
 
+extern char *Slrn_Top_Status_Line;
+
 extern void slrn_push_keymap (SLKeyMap_List_Type *);
 extern void slrn_pop_keymap (void);
 extern SLKeyMap_List_Type *Slrn_Current_Keymap;
