@@ -2,6 +2,7 @@
  This file is part of SLRN.
 
  Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
+ Copyright (c) 2001 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -36,13 +37,12 @@
 
 #include "jdmacros.h"
 #include "server.h"
+#include "snprintf.h"
 
 #ifdef VMS
 # include "vms.h"
 #endif
 
-int Slrn_Query_Reconnect = 1;
-
 static Slrn_Server_Obj_Type NNTP_Server_Obj;
 static Slrn_Post_Obj_Type NNTP_Post_Obj;
 
@@ -160,6 +160,11 @@ static int _nntp_select_group (char *grp, int *min, int *max)
    return nntp_select_group (NNTP_Server, grp, min, max);
 }
 
+static char *_nntp_current_group (void)
+{
+   return NNTP_Server->group_name;
+}
+
 static int _nntp_put_server_cmd (char *cmd, char *buf, unsigned int len)
 {
    int code;
@@ -177,10 +182,10 @@ static int _nntp_xpat_cmd (char *hdr, int rmin, int rmax, char *pat)
    return nntp_xpat_cmd (NNTP_Server, hdr, rmin, rmax, pat);
 }
 
-static int _nntp_xhdr_cmd (char *hdr, int num, 
-			   char *buf, unsigned int buflen)
+static int _nntp_one_xhdr_cmd (char *hdr, int num, 
+			       char *buf, unsigned int buflen)
 {
-   return nntp_xhdr_cmd (NNTP_Server, hdr, num, buf, buflen);
+   return nntp_one_xhdr_cmd (NNTP_Server, hdr, num, buf, buflen);
 }
 
 static int _nntp_xgtitle_cmd (char *pat)
@@ -210,6 +215,11 @@ static int _nntp_list_active (void)
    return code;
 }
 
+static int _nntp_send_authinfo (void)
+{
+   return nntp_authorization (NNTP_Server);
+}
+
 static int _nntp_start_post (void)
 {
    /* Make sure we're connected to a server -- won't be the case the first
@@ -275,7 +285,7 @@ static int _nntp_po_printf (char *fmt, ...)
    char buf[NNTP_BUFFER_SIZE];
    
    va_start (ap, fmt);
-   vsprintf (buf, fmt, ap);
+   slrn_vsnprintf (buf, sizeof (buf), fmt, ap);
    va_end (ap);
    
    return _nntp_po_puts (buf);
@@ -293,12 +303,34 @@ static int _nntp_xover_cmd (int min, int max)
    return status;
 }
 
+static int _nntp_xhdr_cmd (char *field, int min, int max)
+{
+   int status;
+   
+   if (OK_HEAD == (status = nntp_xhdr_cmd (NNTP_Server, field, min, max)))
+     {
+	_NNTP_Abort_On_Disconnection = -1;
+     }
+   
+   return status;
+}
 
 static int _nntp_next_cmd (int *id)
 {
    return nntp_next_cmd (NNTP_Server, id);
 }
 
+static unsigned int _nntp_get_bytes (int clear)
+{
+   unsigned int temp;
+   
+   temp = NNTP_Server->number_bytes_received;
+   if (clear)
+     NNTP_Server->number_bytes_received = 0;
+   
+   return temp;
+}
+
 static int nntp_init_objects (void)
 {
    NNTP_Post_Obj.po_start = _nntp_start_post;
@@ -308,23 +340,28 @@ static int nntp_init_objects (void)
    NNTP_Post_Obj.po_can_post = 1;
    
    NNTP_Server_Obj.sv_select_group = _nntp_select_group;
+   NNTP_Server_Obj.sv_current_group = _nntp_current_group;
    NNTP_Server_Obj.sv_read_line = _nntp_read_line;
    NNTP_Server_Obj.sv_close = _nntp_close_server;
    NNTP_Server_Obj.sv_initialize = _nntp_initialize_server;
    NNTP_Server_Obj.sv_select_article = _nntp_select_article;
    NNTP_Server_Obj.sv_put_server_cmd = _nntp_put_server_cmd;
    NNTP_Server_Obj.sv_xpat_cmd = _nntp_xpat_cmd;
-   NNTP_Server_Obj.sv_xhdr_command = _nntp_xhdr_cmd;
+   NNTP_Server_Obj.sv_xhdr_command = _nntp_one_xhdr_cmd;
    NNTP_Server_Obj.sv_xgtitle_cmd = _nntp_xgtitle_cmd;
    NNTP_Server_Obj.sv_has_cmd = _nntp_has_cmd;
    NNTP_Server_Obj.sv_list_newsgroups = _nntp_list_newsgroups;
    NNTP_Server_Obj.sv_list_active = _nntp_list_active;
+   NNTP_Server_Obj.sv_send_authinfo = _nntp_send_authinfo;
    
    NNTP_Server_Obj.sv_has_xover = 0;
+   NNTP_Server_Obj.sv_has_xhdr = -1;
    NNTP_Server_Obj.sv_nntp_xover = _nntp_xover_cmd;
+   NNTP_Server_Obj.sv_nntp_xhdr = _nntp_xhdr_cmd;
    NNTP_Server_Obj.sv_nntp_head = _nntp_head_cmd;
    NNTP_Server_Obj.sv_nntp_next = _nntp_next_cmd;
    NNTP_Server_Obj.sv_reset = _nntp_reset;
+   NNTP_Server_Obj.sv_nntp_bytes = _nntp_get_bytes;
    return 0;
 }
 
@@ -412,6 +449,8 @@ static int nntp_parse_args (char **argv, int argc)
 		  NNTP_Debug_Fp = fopen (arg1, "w");
 		  if (NNTP_Debug_Fp == NULL)
 		    slrn_exit_error ("Unable to open %s for debugging.", arg1);
+		  else
+		    setbuf (NNTP_Debug_Fp, (char*) NULL);
 	       }
 	     else break;
 
