@@ -2,6 +2,7 @@
  This file is part of SLRN.
 
  Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
+ Copyright (c) 2001 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -18,6 +19,7 @@
  Mass Ave, Cambridge, MA 02139, USA. 
 */
 #include "config.h"
+#include "slrnfeat.h"
 #include <stdio.h>
 #include <string.h>
 
@@ -33,17 +35,23 @@
 #include <slang.h>
 #include "jdmacros.h"
 
-#include "sltcp.h"
+#if SLRN_USE_SLTCP 
+# include "sltcp.h"
+#else
+# include "clientlib.h"
+#endif
 
 #include "nntpcodes.h"
 #include "nntplib.h"
 #include "util.h"
 #include "ttymsg.h"
+#include "snprintf.h"
 
 void (*NNTP_Connection_Lost_Hook) (NNTP_Type *);
 int (*NNTP_Authorization_Hook) (char *, char **, char **);
 
 FILE *NNTP_Debug_Fp;
+int Slrn_Broken_Xref;
 
 static int _nntp_connect_server (NNTP_Type *);
 
@@ -151,6 +159,10 @@ int nntp_gets_server (NNTP_Type *s, char *buf, unsigned int len)
      return -1;
 
    len = strlen (buf);
+   
+   /* Update bytes received */
+   s->number_bytes_received += len;
+
    if (len && (buf[len - 1] == '\n'))
      {
 	len--;
@@ -318,7 +330,7 @@ int nntp_server_vcmd (NNTP_Type *s, char *fmt, ...)
    va_list ap;
 
    va_start (ap, fmt);
-   vsprintf (buf, fmt, ap);
+   slrn_vsnprintf (buf, sizeof (buf), fmt, ap);
    va_end (ap);
 
    return nntp_server_cmd (s, buf);
@@ -330,7 +342,7 @@ int nntp_start_server_vcmd (NNTP_Type *s, char *fmt, ...)
    va_list ap;
 
    va_start (ap, fmt);
-   vsprintf (buf, fmt, ap);
+   slrn_vsnprintf (buf, sizeof (buf), fmt, ap);
    va_end (ap);
 
    return nntp_start_server_cmd (s, buf);
@@ -361,7 +373,7 @@ int nntp_close_server (NNTP_Type *s)
    return 0;
 }
 
-static int _nntp_authorization (NNTP_Type *s)
+int nntp_authorization (NNTP_Type *s)
 {
    char *name, *pass;
 
@@ -407,7 +419,8 @@ static int _nntp_connect_server (NNTP_Type *s)
      return -1;
 
    s->init_state = 1;
-
+   s->number_bytes_received = 0;
+   
    /* Read logon message. */
    switch (nntp_get_server_response (s))
      {
@@ -425,7 +438,7 @@ static int _nntp_connect_server (NNTP_Type *s)
 
    if ((-1 == nntp_server_cmd (s, "MODE READER"))
        || (ERR_ACCESS == s->code)
-       || (-1 == _nntp_authorization (s)))
+       || (-1 == nntp_authorization (s)))
      goto failed;
 
    slrn_message_now ("Connected to host.  Posting %sOk.",
@@ -454,14 +467,14 @@ static char *_nntp_getserverbyfile (char *file)
 
    if (host != NULL)
      {
-	strcpy (buf, host);
+	slrn_strncpy (buf, host, sizeof (buf));
 	return buf;
      }
 
    if (file == NULL)
      {
 #ifdef NNTPSERVER_NAME
-	strcpy (buf, NNTPSERVER_NAME);
+	slrn_strncpy (buf, NNTPSERVER_NAME, sizeof (buf));
 	return buf;
 #else
 	return NULL;
@@ -698,7 +711,7 @@ int nntp_has_cmd (NNTP_Type *s, char *cmd)
 
 static int _nntp_num_or_msgid_cmd (NNTP_Type *s, char *cmd, int n, char *msgid)
 {
-   if (n != -1)
+   if ((n != -1) && ((Slrn_Broken_Xref == 0) || (msgid == NULL)))
      return nntp_server_vcmd (s, "%s %d", cmd, n);
    else
      {
@@ -722,6 +735,11 @@ int nntp_xover_cmd (NNTP_Type *s, int min, int max)
    return nntp_server_vcmd (s, "XOVER %d-%d", min, max);
 }
 
+int nntp_xhdr_cmd (NNTP_Type *s, char *field, int min, int max)
+{
+   return nntp_server_vcmd (s, "XHDR %s %d-%d", field, min, max);
+}
+
 int nntp_article_cmd (NNTP_Type *s, int n, char *msgid)
 {
    return _nntp_num_or_msgid_cmd (s, "ARTICLE", n, msgid);
@@ -742,7 +760,8 @@ int nntp_select_group (NNTP_Type *s, char *name, int *minp, int *maxp)
 {
    int estim;
    int min, max;
-
+   static int try_again = 1;
+   
    switch (nntp_server_vcmd (s, "GROUP %s", name))
      {
       case -1:
@@ -756,14 +775,21 @@ int nntp_select_group (NNTP_Type *s, char *name, int *minp, int *maxp)
 	if (minp != NULL) *minp = min;
 	if (maxp != NULL) *maxp = max;
 
-	strcpy (s->group_name, name);
+	slrn_strncpy (s->group_name, name, NNTP_MAX_GROUP_NAME_LEN);
 	break;
-
+	
+      case ERR_NOAUTH:
+	if (try_again && (-1 != nntp_authorization (s)))
+	  {
+	     try_again = 0;
+	     return nntp_select_group (s, name, minp, maxp);
+	  }
       case ERR_ACCESS:
       default:
 	break;
      }
 
+   try_again = 1;
    return s->code;
 }
 
@@ -811,7 +837,7 @@ int nntp_xgtitle_cmd (NNTP_Type *s, char *pattern)
 }
 
 /* hdr should not include ':' */
-int nntp_xhdr_cmd (NNTP_Type *s, char *hdr, int num, char *buf, unsigned int buflen)
+int nntp_one_xhdr_cmd (NNTP_Type *s, char *hdr, int num, char *buf, unsigned int buflen)
 {
    char tmpbuf[1024];
    int found;
@@ -940,7 +966,7 @@ char *nntp_read_and_malloc (NNTP_Type *s)
 	     mbuf = new_mbuf;
 	  }
 
-	strcpy (mbuf + buffer_len, line);
+	strcpy (mbuf + buffer_len, line); /* safe */
 	buffer_len += len;
 	mbuf [buffer_len++] = '\n';
 	mbuf [buffer_len] = 0;
