@@ -2,6 +2,7 @@
  This file is part of SLRN.
 
  Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
+ Copyright (c) 2001 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -44,6 +45,8 @@ typedef struct
    
    int (*auth_hook)(char *, char **, char **);
 
+   int number_bytes_received;
+   
    SLTCP_Type *tcp;
 }
 NNTP_Type;
@@ -77,6 +80,7 @@ extern int nntp_discard_output (NNTP_Type *s);
 extern int nntp_has_cmd (NNTP_Type *, char *);
 extern int nntp_list_newsgroups (NNTP_Type *);
 extern int nntp_list_active (NNTP_Type *);
+extern int nntp_authorization (NNTP_Type *);
 extern int nntp_end_post (NNTP_Type *);
 extern int nntp_post_cmd (NNTP_Type *);
 extern int nntp_list_active_cmd (NNTP_Type *);
@@ -84,12 +88,13 @@ extern int nntp_list_active_cmd (NNTP_Type *);
 extern int nntp_select_group (NNTP_Type *, char *, int *, int *);
 extern int nntp_xpat_cmd (NNTP_Type *, char *, int, int, char *);
 extern int nntp_xgtitle_cmd (NNTP_Type *, char *);
-extern int nntp_xhdr_cmd (NNTP_Type *, char *, int, char *, unsigned int);
+extern int nntp_one_xhdr_cmd (NNTP_Type *, char *, int, char *, unsigned int);
 
 extern int nntp_listgroup (NNTP_Type *, char *);
 extern int nntp_head_cmd (NNTP_Type *, int, char *, int *);
 
 extern int nntp_xover_cmd (NNTP_Type *, int, int);
+extern int nntp_xhdr_cmd (NNTP_Type *, char *, int, int);
 extern int nntp_next_cmd (NNTP_Type *s, int *);
 extern int nntp_body_cmd (NNTP_Type *s, int, char *);
 extern int nntp_article_cmd (NNTP_Type *s, int, char *);
