@@ -2,6 +2,7 @@
  This file is part of SLRN.
 
  Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
+ Copyright (c) 2001 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -51,6 +52,7 @@
 #include "art.h"
 #include "uudecode.h"
 #include "util.h"
+#include "snprintf.h"
 #include "chmap.h"
 #include "menu.h"
 #include "version.h"
@@ -61,16 +63,19 @@
 
 #define MAX_LINE_BUFLEN	2048
 
-char *Slrn_Courtesy_CC_Message = NULL;
+char *Slrn_CC_Followup_Message = NULL;
+char *Slrn_CC_Post_Message = NULL;
 char *Slrn_Save_Posts_File;
 char *Slrn_Save_Replies_File;
 char *Slrn_Last_Message_Id;
 char *Slrn_Post_Custom_Headers;
 char *Slrn_Failed_Post_Filename;
+char *Slrn_Signoff_String;
 
 int Slrn_Reject_Long_Lines = 1;
 char *Slrn_Postpone_Dir;
 int Slrn_Generate_Message_Id = 1;
+int Slrn_Smart_Quote = 1;
 
 static int postpone_file (char *);
 
@@ -130,11 +135,14 @@ static char *slrn_create_message_id (void)
    t = (unsigned char *) Slrn_User_Info.username;
    if (t != NULL)
      {
+	unsigned char ch;
+	
 	while ((*t != 0) && (b < baseid + sizeof(baseid) - 1))
 	  {
-	     if (*t == ' ')
-	       continue;
-	     *b++ = *t++;
+	     if (((ch = *t) > ' ') && ((ch & 0x80) == 0)
+		 && (ch != '>') && (ch != '<'))
+	       *b++ = ch;
+	     t++;
 	  }
      }
    *b = 0;
@@ -147,15 +155,25 @@ void slrn_add_signature (FILE *fp)
    FILE *sfp;
    char file[SLRN_MAX_PATH_LEN];
    char buf [256];
+
+   if ((NULL != Slrn_Signoff_String) && (*Slrn_Signoff_String != 0))
+     {
+        fputs ("\n", fp);
+        fputs (Slrn_Signoff_String, fp);
+     }
    
    if ((Slrn_User_Info.signature == NULL)
        || (Slrn_User_Info.signature[0] == 0))
      return;
-
-   if ((sfp = slrn_open_home_file (Slrn_User_Info.signature, "r", file, 0)) != NULL)
+   
+   if ((sfp = slrn_open_home_file (Slrn_User_Info.signature, "r", file,
+				   sizeof (file), 0)) != NULL)
      {
+ 	if (! Slrn_Signoff_String)
+ 	  fputs ("\n", fp);
+	
 	/* Apparantly some RFC suggests the -- \n. */
-        fputs ("\n\n-- \n", fp);
+        fputs ("\n-- \n", fp);
 	
 	/* If signature file already has -- \n, do not add it. */
 	if ((NULL != fgets (buf, sizeof (buf), sfp))
@@ -198,7 +216,8 @@ static int slrn_cc_file (char *file, char *to, char *msgid)
    unsigned int linenum;
    char buf [MAX_LINE_BUFLEN];
    unsigned int nth;
-   char *l;
+   char *l, *ref = NULL;
+   int reflen = 0, ret = -1;
 
    if (NULL == (fp = fopen (file, "r")))
      {
@@ -229,7 +248,7 @@ static int slrn_cc_file (char *file, char *to, char *msgid)
      }
    
 #if defined(IBMPC_SYSTEM)
-   pp = slrn_open_tmpfile (outfile, "w");
+   pp = slrn_open_tmpfile (outfile, sizeof (outfile));
 #else
    pp = slrn_popen (Slrn_SendMail_Command, "w");
 #endif
@@ -270,10 +289,6 @@ static int slrn_cc_file (char *file, char *to, char *msgid)
 
    linenum = 0;
    
-#if SLRN_GEN_FROM_EMAIL_HEADER
-   fprintf (pp, "From: %s\n", slrn_make_from_string ());
-#endif
-   
    if (msgid != NULL)
      fprintf (pp, "Message-Id: <slrn%s@%s>\n", 
 	      msgid, Slrn_User_Info.posting_host);
@@ -287,6 +302,21 @@ static int slrn_cc_file (char *file, char *to, char *msgid)
 	if (0 == slrn_case_strncmp ((unsigned char *)line,
 				    (unsigned char *) "To: ", 4))
 	  continue;
+	if ((Slrn_Generate_Email_From == 0) &&
+	    (0 == slrn_case_strncmp ((unsigned char *)line,
+				     (unsigned char *) "From: ", 6)))
+	  continue;
+	if (0 == slrn_case_strncmp ((unsigned char *)line,
+				    (unsigned char *) "References: ", 12))
+	  {
+	     if (NULL != (ref = strrchr (line, '<')))
+	       {
+		  reflen = strlen (ref);
+		  if ((ref[reflen - 1] == '\n') &&
+		      (ref[--reflen] == '\r'))
+		    reflen--;
+	       }
+	  }
 	
 	/* There is some discussion of this extension to mail headers.  For
 	 * now, assume that this extension will be adopted.
@@ -307,11 +337,15 @@ static int slrn_cc_file (char *file, char *to, char *msgid)
    
    fputs ("\n", pp);
    
-   if ((NULL != Slrn_Courtesy_CC_Message) && (*Slrn_Courtesy_CC_Message))
-     {
-	slrn_insert_followup_format (Slrn_Courtesy_CC_Message, pp);
-	fputs ("\n", pp);
-     }
+   if ((Slrn_Current_Header != NULL) && (ref != NULL) &&
+       (0 == strncmp ((unsigned char*)ref,
+		      (unsigned char*)Slrn_Current_Header->msgid, reflen)))
+     ret = slrn_insert_followup_format (Slrn_CC_Followup_Message, pp);
+   else if ((NULL != Slrn_CC_Post_Message) && (*Slrn_CC_Post_Message))
+     ret = fputs (Slrn_CC_Post_Message, pp);
+   
+   if (ret >= 0)
+     ret = fputs ("\n", pp);
    
 # if SLRN_HAS_MIME
    if (Slrn_Use_Mime) fp = slrn_mime_encode (fp);
@@ -324,7 +358,7 @@ static int slrn_cc_file (char *file, char *to, char *msgid)
    slrn_fclose (fp);
 # if defined(IBMPC_SYSTEM)
    slrn_fclose (pp);
-   sprintf (buf, "%s %s", Slrn_SendMail_Command, outfile);
+   slrn_snprintf (buf, sizeof (buf), "%s %s", Slrn_SendMail_Command, outfile);
    slrn_posix_system (buf, 0);
 # else
    slrn_pclose (pp);
@@ -411,11 +445,13 @@ static int check_file_for_posting (char *file)
 	     break;
 	  }
 	
+#if SLRN_HAS_STRICT_FROM
 	if (!slrn_case_strncmp ((unsigned char *) line, (unsigned char *) "From:", 5))
 	  {
 	     err = "This news reader will not accept user generated From lines.";
 	     break;
 	  }
+#endif
      }
 
    if (err == NULL) 
@@ -534,6 +570,8 @@ int slrn_save_file_to_mail_file (char *file, char *save_file, char *msgid)
    time_t now;
    char save_post_file[SLRN_MAX_PATH_LEN];
    char line [MAX_LINE_BUFLEN];
+   char *system_os_name;
+   int has_from = 0, has_messageid = 0, header = 1;
 
    if ((save_file == NULL) || (*save_file == 0))
      return 0;
@@ -543,9 +581,14 @@ int slrn_save_file_to_mail_file (char *file, char *save_file, char *msgid)
 	slrn_error ("File not found: %s--- message not posted.", file);
 	return -1;
      }
-	
-   if (NULL == (outfp = slrn_open_home_file (save_file, "a",
-					     save_post_file, 1)))
+
+#if SLRN_HAS_MIME
+   if (Slrn_Use_Mime)
+     slrn_mime_scan_file (infp);
+#endif
+   
+   if (NULL == (outfp = slrn_open_home_file (save_file, "a", save_post_file,
+					     sizeof (save_post_file), 1)))
      {
 	slrn_message_now ("***Warning: error saving to %s", save_post_file);
 	SLang_input_pending (30);
@@ -553,20 +596,56 @@ int slrn_save_file_to_mail_file (char *file, char *save_file, char *msgid)
 	slrn_fclose (infp);
 	return 0;
      }
-	
+   
    time (&now);
    fprintf (outfp, "From %s@%s %s", Slrn_User_Info.username, Slrn_User_Info.hostname, ctime(&now));
-   fprintf (outfp, "From: %s\n", slrn_make_from_string ());
-   if (msgid != NULL) fprintf (outfp, "Message-Id: <slrn%s@%s>\n", msgid, Slrn_User_Info.posting_host);
-
+   
    while (NULL != fgets (line, sizeof(line) - 1, infp))
      {
+	if (header)
+	  {
+	     if ((has_from == 0) &&
+		 !slrn_case_strncmp ((unsigned char*) "from:",
+				     (unsigned char*) line, 5))
+	       has_from = 1;
+	     if ((has_messageid == 0) &&
+		 !slrn_case_strncmp ((unsigned char*) "message-id:",
+				     (unsigned char*) line, 11))
+	       has_messageid = 1;
+	     
+	     if ((unsigned char)*line == '\n')
+	       {
+		  if (has_from == 0)
+		    fprintf (outfp, "From: %s\n", slrn_make_from_string ());
+		  if ((has_messageid == 0) && (msgid != NULL))
+		    fprintf (outfp, "Message-Id: <slrn%s@%s>\n", msgid,
+			     Slrn_User_Info.posting_host);
+#if SLRN_HAS_MIME
+		  if (Slrn_Use_Mime)
+		    slrn_mime_add_headers (outfp);
+#endif
+		  system_os_name = slrn_get_os_name ();
+#if SLRN_HAS_USER_AGENT
+		  fprintf (outfp, "User-Agent: slrn/%s (%s)\n", Slrn_Version,
+			   system_os_name);
+#else
+		  fprintf (outfp, "X-Newsreader: slrn (%s %s)\n", Slrn_Version,
+			   system_os_name);
+#endif
+		  header = 0;
+	       }
+	     else if (is_empty_header (line))
+	       continue;
+#if SLRN_HAS_MIME
+	     else if (Slrn_Use_Mime)
+	       slrn_mime_header_encode (line, sizeof(line));
+#endif
+	  }
+	
 	if ((*line == 'F')
 	    && !strncmp ("From", line, 4)
 	    && ((unsigned char)line[4] <= ' '))
-	  {
-	     putc ('>', outfp);
-	  }
+	  putc ('>', outfp);
 	
 	fputs (line, outfp);
      }
@@ -654,8 +733,8 @@ static int saved_failed_post (char *file)
    if (NULL == (fp = fopen (file, "r")))
      return -1;
 
-   if (NULL == (outfp = slrn_open_home_file (save_file, "a",
-					     filebuf, 1)))
+   if (NULL == (outfp = slrn_open_home_file (save_file, "a", filebuf,
+					     sizeof (filebuf), 1)))
      {
 	slrn_message_now ("***Warning: error writing to %s", filebuf);
 	fclose (fp);
@@ -816,11 +895,11 @@ int slrn_post_file (char *file, char *to, int is_postponed)
    msgid = slrn_create_message_id ();
 #endif
 
+   slrn_chmap_fix_file (file, 0);
+   
    if (-1 == slrn_save_file_to_mail_file (file, Slrn_Save_Posts_File, msgid))
      return -1;
 
-   slrn_chmap_fix_file (file);
-
    if ((fp = fopen (file, "r")) == NULL)
      {
 	slrn_error ("File not found: %s--- message not posted.", file);
@@ -866,16 +945,17 @@ int slrn_post_file (char *file, char *to, int is_postponed)
    if (Slrn_User_Info.username != NULL)
      Slrn_Post_Obj->po_printf ("Path: %s\n", Slrn_User_Info.username);
 
+#if SLRN_HAS_STRICT_FROM
    /* line is not used until now, so we can use it as a buffer */
-   strncpy(line, slrn_make_from_string (), sizeof(line));
-   line[sizeof(line) - 1] = 0;
+   slrn_strncpy(line, slrn_make_from_string (), sizeof(line));
 
-#if SLRN_HAS_MIME
+# if SLRN_HAS_MIME
    if (Slrn_Use_Mime)
      slrn_mime_header_encode(line, sizeof(line));
-#endif
+# endif
 
    Slrn_Post_Obj->po_printf ("From: %s\n", line);
+#endif /*SLRN_HAS_STRICT_FROM*/
    /* if (Slrn_User_Info.posting_host != NULL)
      * Slrn_Post_Obj->po_printf ("X-Posting-Host: %s\n", Slrn_User_Info.posting_host); */
 
@@ -1003,7 +1083,8 @@ int slrn_post_file (char *file, char *to, int is_postponed)
 	if ((rsp == 7)
 	    || (rsp == 'c') || (rsp == 'C')
 	    || (((rsp == 'e') || (rsp == 'E'))
-		&& (slrn_edit_file (Slrn_Editor_Post, file, 1, 0) < 0)))
+		&& ((slrn_chmap_fix_file (file, 1) < 0)
+		    || (slrn_edit_file (Slrn_Editor_Post, file, 1, 0) < 0))))
 	  {
 	     (void) saved_failed_post (file);
 	     return -1;
@@ -1029,8 +1110,9 @@ int slrn_post (char *newsgroup, char *subj, char *dist)
    int ret;
    
    if (Slrn_Use_Tmpdir)
-     fp = slrn_open_tmpfile (file, "w");
-   else fp = slrn_open_home_file (SLRN_ARTICLE_FILENAME, "w", file, 0);
+     fp = slrn_open_tmpfile (file, sizeof (file));
+   else fp = slrn_open_home_file (SLRN_ARTICLE_FILENAME, "w", file,
+				  sizeof (file), 0);
    
    if (fp == NULL)
      {
@@ -1038,8 +1120,12 @@ int slrn_post (char *newsgroup, char *subj, char *dist)
 	return -1;
      }
    
-   header_lines = 9;
-   fprintf (fp, "Newsgroups: %s\nSubject: %s\n", newsgroup, subj);
+   header_lines = 8;
+   fprintf (fp, "Newsgroups: %s\n", newsgroup);
+#if ! SLRN_HAS_STRICT_FROM
+   fprintf (fp, "From: %s\n", slrn_make_from_string ()); header_lines++;
+#endif
+   fprintf (fp, "Subject: %s\n", subj);
    
    if (Slrn_User_Info.org != NULL)
      {
@@ -1106,7 +1192,7 @@ int slrn_add_custom_headers (FILE *fp, char *headers, int (*write_fun)(char *, F
    return n;
 }
 
-static int get_postpone_dir (char *dirbuf)
+static int get_postpone_dir (char *dirbuf, size_t n)
 {
    char *dir;
    
@@ -1116,7 +1202,7 @@ static int get_postpone_dir (char *dirbuf)
      dir = Slrn_Save_Directory;
    
    if ((dir == NULL)
-       || (-1 == slrn_make_home_dirname (dir, dirbuf))
+       || (-1 == slrn_make_home_dirname (dir, dirbuf, n))
        || (2 != slrn_file_exists (dirbuf)))
      {
 	slrn_error_now (2, "postpone_directory not specified or does not exist");
@@ -1136,14 +1222,14 @@ static int postpone_file (char *file)
    char dir [SLRN_MAX_PATH_LEN];
    char dirfile [SLRN_MAX_PATH_LEN + 256];
 
-   if (-1 == get_postpone_dir (dir))
+   if (-1 == get_postpone_dir (dir, sizeof (dir)))
      return -1;
    
    while (1)
      {
 	int status;
 
-	if (-1 == slrn_dircat (dir, NULL, dirfile))
+	if (-1 == slrn_dircat (dir, NULL, dirfile, sizeof (dirfile)))
 	  return -1;
 
 	if (-1 == slrn_read_input ("Save to", NULL, dirfile, 1, 1))
@@ -1181,7 +1267,7 @@ void slrn_post_postponed (void)
    char dir [SLRN_MAX_PATH_LEN];
    char *file;
 
-   if (-1 == get_postpone_dir (dir))
+   if (-1 == get_postpone_dir (dir, sizeof (dir)))
      return;
    
    file = slrn_browse_dir (dir);
