@@ -39,6 +39,7 @@
 #include "misc.h"
 #include "util.h"
 #include "ttymsg.h"
+#include "snprintf.h"
 
 char *Slrn_Printer_Name;
 
@@ -193,14 +194,15 @@ struct _Slrn_Print_Type
 
 int np_close_printer (Slrn_Print_Type *p)
 {
-   char cmd [256 + SLRN_MAX_PATH_LEN];
+   char *cmd;
 
    if ((p == NULL) || (p->fp == NULL))
      return 0;
    
    (void) slrn_fclose (p->fp);
-   sprintf (cmd, NOTEPAD_PRINT_CMD, p->file);
+   cmd = slrn_strdup_printf (NOTEPAD_PRINT_CMD, p->file);
    slrn_posix_system (cmd, 1);
+   slrn_free (cmd);
    (void) slrn_delete_file (p->file);
    
    slrn_free ((char *) p);
@@ -216,7 +218,7 @@ Slrn_Print_Type *np_open_printer (void)
    
    memset ((char *) p, 0, sizeof(Slrn_Print_Type));
 
-   if (NULL == (p->fp = slrn_open_tmpfile (p->file, "w")))
+   if (NULL == (p->fp = slrn_open_tmpfile (p->file, sizeof (p->file))))
      {
 	slrn_error ("Failed to open tmp file");
 	slrn_free ((char *) p);
@@ -278,7 +280,8 @@ Slrn_Print_Type *popen_open_printer (void)
      {
 	print_cmd = getenv ("PRINTER");
 	if (print_cmd == NULL) print_cmd = "";
-	sprintf (print_cmd_buf, "lpr -P%s", print_cmd);
+	slrn_snprintf (print_cmd_buf, sizeof (print_cmd_buf), "lpr -P%s",
+		       print_cmd);
 	print_cmd = print_cmd_buf;
      }
 
