@@ -2,6 +2,7 @@
  This file is part of SLRN.
 
  Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
+ Copyright (c) 2001 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -58,6 +59,7 @@
  */
 
 int Slrn_Perform_Scoring = SLRN_XOVER_SCORING | SLRN_EXPENSIVE_SCORING;
+int Slrn_Prefer_Head = 0;
 
 /* These two structures are pseudo-score types containing no compiled
  * regular expressions.
@@ -75,6 +77,9 @@ typedef struct PScore_Regexp_Type
 #define SCORE_LINES 6
 #define SCORE_MESSAGE_ID 7
 #define SCORE_DATE 8
+#define SCORE_AGE 9
+#define SCORE_SUB_AND 10
+#define SCORE_SUB_OR 11
    /* generic requires extra server interaction */
 #define SCORE_GENERIC 16
    unsigned int flags;
@@ -84,6 +89,7 @@ typedef struct PScore_Regexp_Type
      {
 	unsigned char *regexp_str;
 	int ival;
+	struct PScore_Regexp_Type *psrt;
      }
    ireg;
    struct PScore_Regexp_Type *next;
@@ -120,6 +126,7 @@ typedef struct Score_Regexp_Type
 	int ival;		       /* used by certain headers */
 	Our_SLRegexp_Type re;
 	SLsearch_Type se;
+	struct Score_Regexp_Type *srt;
      }
    search;
    struct Score_Regexp_Type *next;
@@ -161,16 +168,210 @@ static Score_Type *Score_Root, *Score_Tail;
 
 int Slrn_Apply_Score = 1;
 
+/* returns 1 on match 0 if failed */
+static int match_srt (Slrn_Header_Type *h, Score_Regexp_Type *srt,
+		      char *newsgroup, int or_type) 
+{
+   char *s= NULL;
+   int ival = 0;
+   unsigned int len;
+
+   while (srt != NULL)
+     {
+        switch (srt->header_type)
+          {
+           case SCORE_LINES:
+	     ival = h->lines;
+	     goto integer_compare;/* ugly, ugly, ugly!! */
+		  
+           case SCORE_SUBJECT:
+             s = h->subject;
+             break;
+
+           case SCORE_FROM:
+             s = h->from;
+             break;
+
+           case SCORE_DATE:
+             s = h->date;
+             break;
+	     
+	   case SCORE_AGE:
+	     ival = slrn_date_to_order_parm(h->date);
+	     goto integer_compare;
+	     
+           case SCORE_MESSAGE_ID:
+             s = h->msgid;
+             break;
+
+           case SCORE_XREF:
+             s = h->xref;
+             break;
+
+           case SCORE_REFERENCES:
+             s = h->refs;
+             break;
+
+           case SCORE_NEWSGROUP:
+             s = newsgroup;
+             break;
+
+           case SCORE_GENERIC:
+#ifdef SLRNPULL_CODE
+             s = slrn_get_extra_xover_header (srt->generic_keyword);
+#else
+	     if (Slrn_Perform_Scoring & SLRN_EXPENSIVE_SCORING)
+	       s = slrn_extract_add_header (h, srt->generic_keyword);
+	     else
+	       s = NULL;
+#endif
+             break;
+
+           case SCORE_SUB_AND:
+             if (match_srt(h, srt->search.srt, newsgroup, 0))
+	       { /* sub matches */
+                  if (srt->not_flag)
+                    {
+                       if (or_type) goto next_srt;
+                       return 0;
+                    }
+               }
+             else /* sub does not match */
+	       {
+		 if (srt->not_flag == 0)
+                   {
+                      if (or_type) goto next_srt;
+                      return 0;
+                   }
+	       }
+	     if (or_type) return 1;
+	     goto next_srt;
+             break;
+
+	   case SCORE_SUB_OR:
+             if (match_srt(h, srt->search.srt, newsgroup, 1))
+               { /* sub matches */
+                  if (srt->not_flag)
+                    {
+                       if (or_type) goto next_srt;
+                       return 0;
+                    }
+               }
+             else /* sub does not match */
+	       {
+		 if (srt->not_flag == 0)
+                   {
+                      if (or_type) goto next_srt;
+                      return 0;
+                   }
+	       }
+             if (or_type) return 1;
+	     goto next_srt;
+             break;
+           default:
+             s = NULL;            /* not supposed to happen */
+          } /*switch (srt->header_type)*/
+
+        if (s == NULL)
+          {
+             if (srt->not_flag)
+               {
+                  /* Match */
+                  if (or_type)
+                    return 1;
+                   goto next_srt;
+               }
+
+             /* Match failed */
+             if (or_type) goto next_srt;
+             return 0;
+          }
+
+        len = strlen (s);
+
+        if (srt->do_osearch)
+          {
+             SLsearch_Type *se = &srt->search.se;
+
+	     if ((len < (unsigned int) se->key_len)
+                      || (NULL == SLsearch ((unsigned char *) s,
+                                            (unsigned char *) s + len,
+                                            se)))
+               { /* expr does not match */
+                  if (srt->not_flag == 0)
+                    {
+                       if (or_type) goto next_srt;
+                       return 0;
+                    }
+               }
+             else if (srt->not_flag)
+               {
+                  if (or_type) goto next_srt;
+                  return 0;
+               }
+          }
+        else
+          {
+             SLRegexp_Type *re;
+
+             re = &srt->search.re.regexp;
+             if ((len < re->min_length)
+                 || (NULL == SLang_regexp_match ((unsigned char *)s, len, re)))
+               {
+                  if (srt->not_flag == 0)
+                    {
+                       if (or_type) goto next_srt;
+                       return 0;
+                    }
+               }
+             else if (srt->not_flag)
+               {
+                  if (or_type) goto next_srt;
+                  return 0;
+               }
+          }
+        /* Get here if above matched */
+        if (or_type) return 1;
+	srt = srt->next;
+	continue;
+	
+	/* This is ugly but I am worried about speed. --- we only get
+	 * here for those headers that have integer values.
+	 */
+	integer_compare:
+	if (ival < srt->search.ival)
+	  {
+	     if (srt->not_flag == 0)
+	       {
+		  if (or_type) goto next_srt;
+		  return 0;
+	       }
+	  }
+	else if (srt->not_flag)
+	  {
+	     if (or_type) goto next_srt;
+	     return 0;
+	  }
+	
+	/* If we get here, the regular expression matched. */
+	if (or_type) return 1;
+	
+	next_srt:
+        srt = srt->next;
+     } /*while (srt != NULL)*/
+   if (or_type) return 0;
+   else return 1;
+}
+
 int slrn_score_header (Slrn_Header_Type *h, char *newsgroup)
 {
    Score_Type *st;
    int score = 0;
-   char *s;
-   int ival = 0;
-   
+   int or_type;
+   Score_Regexp_Type *srt;
 #ifndef SLRNPULL_CODE
 #if SLRN_HAS_MSGID_CACHE
-   s = slrn_is_msgid_cached (h->msgid, newsgroup, 1);
+   char *s = slrn_is_msgid_cached (h->msgid, newsgroup, 1);
    if (s != NULL)
      {
 	/* Kill it if this wasn't the newsgroup where we saw it */
@@ -180,200 +381,104 @@ int slrn_score_header (Slrn_Header_Type *h, char *newsgroup)
 #endif
 #endif				       /* NOT SLRNPULL_CODE */
    
-   s = NULL;
-
    st = Score_Root;
    while (st != NULL)
      {
-	unsigned int len;
-	Score_Regexp_Type *srt = &st->regexp_list;
-#ifndef SLRNPULL_CODE
-	char buf[1024];
-#endif
-	int or_type = st->pscore->flags & SCORE_IS_OR_TYPE;
+	srt = &st->regexp_list;
+	or_type = st->pscore->flags & SCORE_IS_OR_TYPE;
 	
-	while (srt != NULL)
-	  {
-	     switch (srt->header_type)
-	       {
-		case SCORE_LINES:
-		  ival = h->lines;
-		  goto integer_compare;/* ugly, ugly, ugly!! */
-		  
-		case SCORE_SUBJECT:
-		  s = h->subject;
-		  break;
-		  
-		case SCORE_FROM:
-		  s = h->from;
-		  break;
-		  
-		case SCORE_DATE:
-		  s = h->date;
-		  break;
-
-		case SCORE_MESSAGE_ID:
-		  s = h->msgid;
-		  break;
-		  
-		case SCORE_XREF:
-		  s = h->xref;
-		  break;
-		  
-		case SCORE_REFERENCES:
-		  s = h->refs;
-		  break;
-		  
-		case SCORE_NEWSGROUP:
-		  s = newsgroup;
-		  break;
-		  
-		case SCORE_GENERIC:
-#ifdef SLRNPULL_CODE
-		  s = slrn_get_extra_xover_header (srt->generic_keyword);
-#else
-		  
-		  /* Yuk.  This will be slow! */
-		  if (Slrn_Score_After_XOver)
-		    {
-		       if (((Slrn_Perform_Scoring & SLRN_EXPENSIVE_SCORING) == 0)
-			   || (-1 == Slrn_Server_Obj->sv_xhdr_command (srt->generic_keyword,
-							h->number, buf, sizeof(buf))))
-			 s = NULL;
-		       else s = buf;
-		    }
-		  else
-		    {
-		       if (Slrn_Perform_Scoring & SLRN_EXPENSIVE_SCORING)
-			 s = slrn_get_extra_xover_header (srt->generic_keyword);
-		       else s = NULL;
-		    }
-#endif
-		  break;
-		  
-		default:
-		  s = NULL;	       /* not supposed to happen */
-	       }
-	     
-	     if (s == NULL)
-	       {
-		  if (srt->not_flag)
-		    {
-		       /* Match */
-		       if (or_type)
-			 break;
-		       
-		       goto next_srt;
-		    }
-		  
-		  /* Match failed */
-		  if (or_type) goto next_srt;
-		  break;
-	       }
-	     
-	     len = strlen (s);
-	     
-	     if (srt->do_osearch)
-	       {
-		  SLsearch_Type *se = &srt->search.se;
-		  
-		  if ((len < (unsigned int) se->key_len)
-		      || (NULL == SLsearch ((unsigned char *) s,
-					    (unsigned char *) s + len,
-					    se)))
-		    {
-		       if (srt->not_flag == 0)
-			 {
-			    if (or_type) goto next_srt;
-			    break;
-			 }
-		    }
-		  else if (srt->not_flag)
-		    {
-		       if (or_type) goto next_srt;
-		       break;
-		    }
-	       }
-	     else
-	       {
-		  SLRegexp_Type *re;
-		  
-		  re = &srt->search.re.regexp;
-		  if ((len < re->min_length)
-		      || (NULL == SLang_regexp_match ((unsigned char *)s, len, re)))
-		    {
-		       if (srt->not_flag == 0)
-			 {
-			    if (or_type) goto next_srt;
-			    break;
-			 }
-		    }
-		  else if (srt->not_flag)
-		    {
-		       if (or_type) goto next_srt;
-		       break;
-		    }
-	       }
-	     /* Get here if above matched */
-	     if (or_type) break;
-	     srt = srt->next;
-	     continue;
-	     
-	     /* This is ugly but I am worried about speed. --- we only get
-	      * here for those headers that have integer values.
-	      */
-	     integer_compare:
-	     if (ival < srt->search.ival)
-	       {
-		  if (srt->not_flag == 0)
-		    {
-		       if (or_type) goto next_srt;
-		       break;
-		    }
-	       }
-	     else if (srt->not_flag)
-	       {
-		  if (or_type) goto next_srt;
-		  break;
-	       }
-	     
-	     /* If we get here, the regular expression matched. */
-	     if (or_type) break;
-	     
-	     next_srt:
-	     srt = srt->next;
-	  }
-	
-	if (((srt == NULL) && (or_type == 0))
-	    || (or_type && (srt != NULL)))
-	  {
+        if (match_srt(h, srt, newsgroup, or_type))
+          {
 	     int st_score = st->pscore->score;
 	     
 	     if (st->pscore->flags & RETURN_THIS_SCORE)
 	       return st_score;
-	     
+#if 0
 	     if ((st_score == 9999)
 		 || (st_score == -9999))
 	       return st_score;
-	     
+#endif
 	     if (st_score == 0)
 	       return score;
-	     
 	     score += st_score;
 	  }
 	
 	st = st->next;
-     }
+     } /*while (st != NULL)*/
    return score;
 }
 
+static int compile_psrt (PScore_Regexp_Type *psrt, Score_Regexp_Type *srt,
+			 int *generic)
+{
+   SLRegexp_Type *re;
+
+   while (psrt != NULL)
+     {
+        unsigned int flags = psrt->flags;
+
+        if ((SCORE_GENERIC == (srt->header_type = psrt->header_type))
+	    && (Slrn_Perform_Scoring & SLRN_EXPENSIVE_SCORING))
+          {
+             *generic += 1;
+	     slrn_request_additional_header (psrt->keyword);
+          }
+	
+        srt->not_flag = (0 != (flags & NOT_FLAG));
+	  
+	if ((srt->header_type == SCORE_SUB_AND) || 
+		(srt->header_type == SCORE_SUB_OR))
+	  {
+             srt->search.srt = (Score_Regexp_Type *) slrn_malloc (sizeof (Score_Regexp_Type), 1, 0);
+             if (srt->search.srt == NULL) return -1;
+
+	     if (compile_psrt(psrt->ireg.psrt, srt->search.srt, generic) != 0)
+	       return -1;
+          }
+	else
+	  {
+             srt->generic_keyword = psrt->keyword;
+
+	     if (flags & USE_INTEGER)
+               {
+                  srt->search.ival = psrt->ireg.ival;
+               }
+             else
+               {
+                  re = &srt->search.re.regexp;
+                  re->pat = psrt->ireg.regexp_str;
+                  re->buf = srt->search.re.buf;
+                  re->buf_len = sizeof (srt->search.re.buf);
+                  re->case_sensitive = 0;
+
+                  if (0 != SLang_regexp_compile(re))
+                    {
+                       return -1;
+                    }
+
+                  /* If an ordinary search is ok, use it. */
+                  if (re->osearch)
+                    {
+                       srt->do_osearch = 1;
+                       SLsearch_init ((char *) psrt->ireg.regexp_str, 1, 0, &srt->search.se);
+                    }
+               }
+          }
+		
+        psrt = psrt->next;
+        if (psrt == NULL) break;
+
+        srt->next = (Score_Regexp_Type *) slrn_malloc (sizeof (Score_Regexp_Type), 1, 0);
+        if (NULL == (srt = srt->next)) return -1;
+     } /* while (psrt != NULL)*/
+   return 0;
+}
 
 static int chain_group_regexp (PScore_Type *pst, int *generic)
 {
    PScore_Regexp_Type *psrt;
    Score_Regexp_Type *srt;
    Score_Type *st;
-   SLRegexp_Type *re;
    
    while (pst != NULL)
      {
@@ -394,51 +499,11 @@ static int chain_group_regexp (PScore_Type *pst, int *generic)
 	psrt = pst->pregexp_list;
 	srt = &st->regexp_list;
 	
-	while (psrt != NULL)
-	  {
-	     unsigned int flags = psrt->flags;
-	     
-	     if (SCORE_GENERIC == (srt->header_type = psrt->header_type))
-	       *generic += 1;
-	     
-	     srt->generic_keyword = psrt->keyword;
-	     srt->not_flag = (0 != (flags & NOT_FLAG));
-	     
-	     if (flags & USE_INTEGER)
-	       {
-		  srt->search.ival = psrt->ireg.ival;
-	       }
-	     else
-	       {
-		  re = &srt->search.re.regexp;
-		  re->pat = psrt->ireg.regexp_str;
-		  re->buf = srt->search.re.buf;
-		  re->buf_len = sizeof (srt->search.re.buf);
-		  re->case_sensitive = 0;
-		  
-		  if (0 != SLang_regexp_compile (re))
-		    {
-		       return -1;
-		    }
-		  
-		  /* If an ordinary search is ok, use it. */
-		  if (re->osearch)
-		    {
-		       srt->do_osearch = 1;
-		       SLsearch_init ((char *) psrt->ireg.regexp_str, 1, 0, &srt->search.se);
-		    }
-	       }
-	     
-	     psrt = psrt->next;
-	     if (psrt == NULL) break;
-	     
-	     srt->next = (Score_Regexp_Type *) slrn_malloc (sizeof (Score_Regexp_Type),
-							    1, 0);
-	     if (NULL == (srt = srt->next))
-	       return -1;
-	  }
+	if (compile_psrt(psrt, srt, generic) != 0)
+	  return -1;
+	
 	pst = pst->next;
-     }
+     } /* while (pst != NULL) */
    return 0;
 }
 
@@ -470,7 +535,7 @@ int slrn_open_score (char *group_name)
    int generic;
 
    free_group_chain ();
-
+   
    if ((Slrn_Perform_Scoring == 0) || (Group_Score_Root == NULL))
      {
 	Slrn_Apply_Score = 0;
@@ -512,17 +577,10 @@ int slrn_open_score (char *group_name)
    if (Score_Root == NULL) return 0;
    
    Slrn_Apply_Score = 1;
-   
-   if ((generic == 0)
-       || (0 == (Slrn_Perform_Scoring & SLRN_EXPENSIVE_SCORING)))
-     return 1;
 
 #ifndef SLRNPULL_CODE
-   if ((generic >= 1) || !Slrn_Server_Obj->sv_has_xover || !Slrn_Server_Obj->sv_has_cmd ("XHDR"))
-     {
-	Slrn_Score_After_XOver = 0;
-	slrn_open_suspend_xover ();
-     }
+   if (generic && (Slrn_Prefer_Head || !Slrn_Server_Obj->sv_has_cmd("XHDR")))
+     slrn_open_suspend_xover ();
 #endif
 
    return 1;
@@ -538,49 +596,48 @@ void slrn_close_score (void)
 
 
 
-static int add_group_regexp (PScore_Type *pst,
-			     unsigned char *str,
-			     unsigned char *keyword, unsigned int type, int not_flag)
+static int add_group_regexp (PScore_Regexp_Type *psrt, unsigned char *str,
+			     unsigned char *keyword, unsigned int type,
+			     int not_flag)
 {
    unsigned int len;
-   PScore_Regexp_Type *psrt;
-   
    *str++ = 0;			       /* null terminate keyword by zeroing
 					* out the colon.  This is by agreement
 					* with the calling routine.
 					*/
    
-   if (*str == ' ') str++;	       /* space following colon not meaningful */
+   if (*str == ' ') str++;       /* space following colon not meaningful */
 
-   len = (unsigned int) (slrn_trim_string ((char *) str) - (char *) str);
-   if (0 == len) return -1;
-   
-   psrt = (PScore_Regexp_Type *) slrn_safe_malloc (sizeof (PScore_Regexp_Type));
-   
-   if (type != SCORE_LINES)
-     psrt->ireg.regexp_str = (unsigned char *) slrn_safe_strmalloc ((char *)str);
+   psrt->header_type = type;
+   if ((type != SCORE_SUB_AND) && (type != SCORE_SUB_OR))
+     {
+	len = (unsigned int) (slrn_trim_string ((char *) str) - (char *) str);
+	if (0 == len) return -1;
+	
+	if (type == SCORE_LINES)
+	  {
+	     psrt->ireg.ival = atoi((char *)str);
+	     psrt->flags |= USE_INTEGER;
+	  }
+	else if (type == SCORE_AGE)
+	  {
+	     psrt->ireg.ival = time(NULL) - atoi((char *)str) * 86400;
+	     psrt->flags |= USE_INTEGER;
+	  }
+	else
+	  psrt->ireg.regexp_str = (unsigned char *) slrn_safe_strmalloc ((char *)str);
+	
+	strncpy (psrt->keyword, (char *) keyword, MAX_KEYWORD_LEN);
+	psrt->keyword[MAX_KEYWORD_LEN - 1] = 0;
+     }
    else
      {
-	psrt->ireg.ival = atoi((char *)str);
-	psrt->flags |= USE_INTEGER;
+	psrt->ireg.psrt = NULL;
      }
-   
-   psrt->header_type = type;
-   strncpy (psrt->keyword, (char *) keyword, MAX_KEYWORD_LEN);
-   psrt->keyword[MAX_KEYWORD_LEN - 1] = 0;
-   
+
    if (not_flag) psrt->flags |= NOT_FLAG;
    psrt->next = NULL;
-   if (pst->pregexp_list == NULL)
-     {
-	pst->pregexp_list = psrt;
-     }
-   else
-     {
-	PScore_Regexp_Type *last = pst->pregexp_list;
-	while (last->next != NULL) last = last->next;
-	last->next = psrt;
-     }
+   
    return 0;
 }
 
@@ -834,7 +891,7 @@ static int handle_include_line (char *file, char *line, Score_Context_Type *c)
    if (NULL == (dir = slrn_strnmalloc (file, (unsigned int) (f - file), 1)))
      return -1;
 
-   if (-1 == slrn_dircat (dir, line, buf))
+   if (-1 == slrn_dircat (dir, line, buf, sizeof (buf)))
      {
 	slrn_free (dir);
 	return -1;
@@ -844,136 +901,139 @@ static int handle_include_line (char *file, char *line, Score_Context_Type *c)
    slrn_free (dir);
    return status;
 }
-	
-static int read_score_file_internal (char *file, Score_Context_Type *c)
+
+static int phrase_score_file (char *file, FILE *fp, Score_Context_Type *c,
+			      unsigned int *linenum,
+			      PScore_Regexp_Type *sub_psrt)
 {
-   char line[1024];
-   FILE *fp;
-   unsigned int linenum = 0;
-
-   if (NULL == (fp = fopen (file, "r")))
-     {
-	slrn_error ("Unable to open score file %s", file);
-	return -1;
-     }
-
-   while (fgets (line, sizeof (line) - 1, fp))
+    char line[1024];
+    PScore_Regexp_Type *psrt;
+    
+    while (fgets (line, sizeof (line) - 1, fp))
      {
 	unsigned char *lp;
 	int not_flag;
+        
+	
+	(*linenum)++;
 
-	linenum++;
-
+	psrt = NULL;
+	
 	if (0 == SLprep_line_ok (line, &c->pt))
 	  continue;
 
 	lp = (unsigned char *) slrn_skip_whitespace (line);
 	if ((*lp == '#') || (*lp == '%') || (*lp <= ' ')) continue;
 
-	if ((0 == slrn_case_strncmp (lp, (unsigned char *) "include", 7))
-	    && ((lp[7] == ' ') || (lp[7] == '\t')))
+	if (sub_psrt == NULL)
 	  {
-	     if (0 == handle_include_line (file, (char *)lp + 7, c))
-	       continue;
-	     
-	     score_error ("Error handling INCLUDE line", line, linenum, file);
-	     goto error_return;
-	  }
-
-	if (*lp == '[')
-	  {
-	     unsigned char *g, *gmax, ch;
-	     g = c->group;
-	     gmax = g + sizeof (c->group);
-	     
-	     lp++;
-	     lp = (unsigned char *) slrn_skip_whitespace ((char *)lp);
-	     
-	     c->gnt_not_flag = 0;
-	     if (*lp == '~')
+	     if ((0 == slrn_case_strncmp (lp, (unsigned char *) "include", 7))
+		 && ((lp[7] == ' ') || (lp[7] == '\t')))
 	       {
-		  c->gnt_not_flag = 1;
-		  lp = (unsigned char *) slrn_skip_whitespace ((char *)lp + 1);
+		  if (0 == handle_include_line (file, (char *)lp + 7, c))
+		    continue;
+		  
+		  score_error ("Error handling INCLUDE line", line, *linenum,
+			       file);
+		  return -1;
 	       }
 	     
-	     while (((ch = *lp++) != 0)
-		    && (ch != ']') && (g < gmax))
-	       *g++ = ch;
-	     
-	     if ((ch != ']') || (g == gmax))
+	     if (*lp == '[')
 	       {
-		  score_error ("Syntax Error.", line, linenum, file);
-		  goto error_return;
-	       }
-	     *g = 0;
-	     c->start_new_group = 1;
-	     c->score_has_expired = 0;
-	     c->start_new_score = 0;
-	     continue;
-	  }
-
-	if (!slrn_case_strncmp (lp, (unsigned char *)"Score:", 6))
-	  {
-	     unsigned char *lpp = lp + 6;
-	     c->pscore_flags = 0;
-	     if (*lpp == ':')
-	       {
-		  lpp++;
-		  c->pscore_flags |= SCORE_IS_OR_TYPE;
-	       }
-	     lpp = (unsigned char *) slrn_skip_whitespace ((char *) lpp);
-	     if (*lpp == '=')
-	       {
-		  c->pscore_flags |= RETURN_THIS_SCORE;
-		  lpp++;
-	       }
-	     c->score = atoi ((char *)lpp);
-	     c->start_new_score = 1;
-	     continue;
-	  }
-
-	if (c->start_new_score)
-	  {
-	     if (!slrn_case_strncmp (lp, (unsigned char *) "Expires:", 8))
-	       {
-		  int ret;
-		  ret = has_score_expired (lp + 8, c->today);
-		  if (ret == -1)
+		  unsigned char *g, *gmax, ch;
+		  g = c->group;
+		  gmax = g + sizeof (c->group);
+		  
+		  lp++;
+		  lp = (unsigned char *) slrn_skip_whitespace ((char *)lp);
+		  
+		  c->gnt_not_flag = 0;
+		  if (*lp == '~')
 		    {
-		       score_error ("Expecting 'Expires: MM/DD/YYYY' or 'Expires: DD-MM-YYYY'",
-				    line, linenum, file);
-		       goto error_return;
+		       c->gnt_not_flag = 1;
+		       lp = (unsigned char *) slrn_skip_whitespace ((char *)lp + 1);
 		    }
-		  if (ret)
+		  
+		  while (((ch = *lp++) != 0)
+			 && (ch != ']') && (g < gmax))
+		    *g++ = ch;
+		  
+		  if ((ch != ']') || (g == gmax))
 		    {
-		       slrn_message ("%s has expired score on line %d",
-				     file, linenum);
-		       c->start_new_score = 0;
-		       c->score_has_expired = 1;
+		       score_error ("Syntax Error.", line, *linenum, file);
+		       return -1;
 		    }
-		  else c->score_has_expired = 0;
+		  *g = 0;
+		  c->start_new_group = 1;
+		  c->score_has_expired = 0;
+		  c->start_new_score = 0;
 		  continue;
 	       }
-
-	     
-	     if (NULL == (c->pst = create_new_score (c->group, c->start_new_group, c->gnt_not_flag,
-						     c->score, c->pscore_flags)))
+        
+	     if (!slrn_case_strncmp (lp, (unsigned char *)"Score:", 6))
 	       {
-		  score_error ("Bad group regular expression.", line, linenum, file);
-		  goto error_return;
+		  unsigned char *lpp = lp + 6;
+		  c->pscore_flags = 0;
+		  if (*lpp == ':')
+		    {
+		       lpp++;
+		       c->pscore_flags |= SCORE_IS_OR_TYPE;
+		    }
+		  lpp = (unsigned char *) slrn_skip_whitespace ((char *) lpp);
+		  if (*lpp == '=')
+		    {
+		       c->pscore_flags |= RETURN_THIS_SCORE;
+		       lpp++;
+		    }
+		  c->score = atoi ((char *)lpp);
+		  c->start_new_score = 1;
+		  continue;
 	       }
-	     c->start_new_group = 0;
-	     c->start_new_score = 0;
-	     c->score_has_expired = 0;
-	  }
-	
-	if (c->score_has_expired) continue;
-	
-	if (c->pst == NULL)
-	  {
-	     score_error ("Expecting Score keyword.", line, linenum, file);
-	     goto error_return;
-	  }
+	     
+	     if (c->start_new_score)
+	       {
+		  if (!slrn_case_strncmp (lp, (unsigned char *) "Expires:", 8))
+		    {
+		       int ret;
+		       ret = has_score_expired (lp + 8, c->today);
+		       if (ret == -1)
+			 {
+			    score_error ("Expecting 'Expires: MM/DD/YYYY' or 'Expires: DD-MM-YYYY'",
+					 line, *linenum, file);
+			    return -1;
+			 }
+		       if (ret)
+			 {
+			    slrn_message ("%s has expired score on line %d",
+					  file, *linenum);
+			    c->start_new_score = 0;
+			    c->score_has_expired = 1;
+			 }
+		       else c->score_has_expired = 0;
+		       continue;
+		    }
+		  
+		  if (NULL == (c->pst = create_new_score (c->group, c->start_new_group, c->gnt_not_flag,
+							  c->score, c->pscore_flags)))
+		    {
+		       score_error ("Bad group regular expression.", line,
+				    *linenum, file);
+		       return -1;
+		    }
+		  c->start_new_group = 0;
+		  c->start_new_score = 0;
+		  c->score_has_expired = 0;
+	       }
+	     
+	     if (c->score_has_expired) continue;
+	     
+	     if (c->pst == NULL)
+	       {
+		  score_error ("Expecting Score keyword.", line, *linenum, file);
+		  return -1;
+	       }
+	     
+	  } /*if (sub_psrt==null) */
 	
 	if (*lp == '~')
 	  {
@@ -982,45 +1042,112 @@ static int read_score_file_internal (char *file, Score_Context_Type *c)
 	  }
 	else not_flag = 0;
 
+	psrt = (PScore_Regexp_Type *) slrn_safe_malloc (sizeof (PScore_Regexp_Type));
 	/* Otherwise the line is a kill one */
 	if (!slrn_case_strncmp (lp, (unsigned char *)"Subject:", 8))
-	  add_group_regexp (c->pst, lp + 7, lp, SCORE_SUBJECT, not_flag);
+	  add_group_regexp (psrt, lp + 7, lp, SCORE_SUBJECT, not_flag);
 	else if (!slrn_case_strncmp (lp, (unsigned char *)"From:", 5))
-	  add_group_regexp (c->pst, lp + 4, lp, SCORE_FROM, not_flag);
+	  add_group_regexp (psrt, lp + 4, lp, SCORE_FROM, not_flag);
 	else if (!slrn_case_strncmp (lp, (unsigned char *)"Xref:", 5))
-	  add_group_regexp (c->pst, lp + 4, lp, SCORE_XREF, not_flag);
+	  add_group_regexp (psrt, lp + 4, lp, SCORE_XREF, not_flag);
 	else if (!slrn_case_strncmp (lp, (unsigned char *)"Newsgroup:", 10))
-	  add_group_regexp (c->pst, lp + 9, lp, SCORE_NEWSGROUP, not_flag);
+	  add_group_regexp (psrt, lp + 9, lp, SCORE_NEWSGROUP, not_flag);
 	else if (!slrn_case_strncmp (lp, (unsigned char *)"References:", 11))
-	  add_group_regexp (c->pst, lp + 10, lp, SCORE_REFERENCES, not_flag);
+	  add_group_regexp (psrt, lp + 10, lp, SCORE_REFERENCES, not_flag);
 	else if (!slrn_case_strncmp (lp, (unsigned char *)"Lines:", 6))
-	  add_group_regexp (c->pst, lp + 5, lp, SCORE_LINES, not_flag);
+	  add_group_regexp (psrt, lp + 5, lp, SCORE_LINES, not_flag);
 	else if (!slrn_case_strncmp (lp, (unsigned char *)"Date:", 5))
-	  add_group_regexp (c->pst, lp + 5, lp, SCORE_DATE, not_flag);
+	  add_group_regexp (psrt, lp + 4, lp, SCORE_DATE, not_flag);
+	else if (!slrn_case_strncmp (lp, (unsigned char *)"Age:", 4))
+	  add_group_regexp (psrt, lp + 3, lp, SCORE_AGE, not_flag);
 	else if (!slrn_case_strncmp (lp, (unsigned char *)"Message-Id:", 11))
-	  add_group_regexp (c->pst, lp + 10, lp, SCORE_MESSAGE_ID, not_flag);
+	  add_group_regexp (psrt, lp + 10, lp, SCORE_MESSAGE_ID, not_flag);
+        else if (!slrn_case_strncmp (lp, (unsigned char *)"{:", 1))
+	  {
+            if (lp[2] ==':')
+	      {
+		add_group_regexp (psrt, lp + 1, lp, SCORE_SUB_OR, not_flag);
+	      }
+	    else
+	      {
+		add_group_regexp (psrt, lp + 1, lp, SCORE_SUB_AND, not_flag);
+	      }
+	    if (phrase_score_file(file, fp, c, linenum, psrt))  return -1;
+          }
+        else if (!slrn_case_strncmp (lp, (unsigned char *)"}", 1))
+	  {
+	     if (sub_psrt != NULL)
+	       return 0;
+	     else
+	       {
+		  score_error ("Missing COLON.", line, *linenum, file);
+		  return -1;
+	       }
+	  }
 	else
 	  {
 	     unsigned char *lpp = lp;
 	     while (*lpp && (*lpp != ':')) lpp++;
 	     if (*lpp != ':')
 	       {
-		  score_error ("Missing COLON.", line, linenum, file);
-		  goto error_return;
+		  score_error ("Missing COLON.", line, *linenum, file);
+		  return -1;
 	       }
 	     
-	     add_group_regexp (c->pst, lpp, lp, SCORE_GENERIC, not_flag);
+	     add_group_regexp (psrt, lpp, lp, SCORE_GENERIC, not_flag);
 	  }
+   
+	if (sub_psrt == NULL)
+	  {
+	     if (c->pst->pregexp_list == NULL)
+	       {
+		  c->pst->pregexp_list = psrt;
+	       }
+	     else
+	       {
+		  PScore_Regexp_Type *last = c->pst->pregexp_list;
+		  while (last->next != NULL) last = last->next;
+		  last->next = psrt;
+	       }
+	  }
+	else
+	  {
+	     if (sub_psrt->ireg.psrt == NULL)
+	       {
+		  sub_psrt->ireg.psrt = psrt;
+	       }
+	     else
+	       {
+		  PScore_Regexp_Type *last = sub_psrt->ireg.psrt;
+		  while (last->next != NULL) last = last->next;
+		  last->next = psrt;
+	       }
+	  }
+     } /*while (fgets (line, sizeof (line) - 1, fp))*/
+   if (sub_psrt == NULL) return 0;
+   else
+     {
+	score_error ("Missing '}' for  subscore", line, *linenum, file);
+	return -1;
      }
-
-   fclose (fp);
-   return 0;
-
-   error_return:
-   fclose (fp);
-   return -1;
 }
 
+static int read_score_file_internal (char *file, Score_Context_Type *c)
+{
+   FILE *fp;
+   unsigned int linenum = 0;
+   int ret;
+
+   if (NULL == (fp = fopen (file, "r")))
+     {
+	slrn_error ("Unable to open score file %s", file);
+	return -1;
+     }
+
+   ret = phrase_score_file(file, fp, c, &linenum, NULL);
+   fclose (fp);
+   return ret;
+}
 			
 int slrn_read_score_file (char *file)
 {
