@@ -2,6 +2,7 @@
  This file is part of SLRN.
 
  Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
+ Copyright (c) 2001 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -26,6 +27,20 @@
 # include <stdlib.h>
 #endif
 
+#ifdef VMS
+# include <file.h>
+#else
+# include <sys/types.h>
+# include <sys/stat.h>
+#endif
+
+#ifdef HAVE_FCNTL_H
+# include <fcntl.h>
+#endif
+#ifdef HAVE_SYS_FCNTL_H
+# include <sys/fcntl.h>
+#endif
+
 #include <slang.h>
 #include "jdmacros.h"
 
@@ -34,9 +49,9 @@
 #include "misc.h"
 #include "util.h"
 #include "startup.h"
+#include "snprintf.h"
 #if SLRN_USE_SLTCP 
 # if SLRN_HAS_NNTP_SUPPORT || SLRN_HAS_GROUP_LENS
-#  include "sltcp.h"
 #  include "sltcp.c"
 # endif
 #endif
@@ -110,14 +125,16 @@ static int inews_puts (char *s)
 static int inews_printf (char *fmt, ...)
 {
    va_list ap;
-
-   char buf [1024];
+   char *line;
+   int retval;
    
    va_start (ap, fmt);
-   vsprintf (buf, fmt, ap);
+   line = slrn_strdup_vprintf (fmt, ap);
    va_end (ap);
+   retval = inews_puts (line);
+   slrn_free (line);
    
-   return inews_puts(buf);
+   return retval;
 }
 
 
@@ -130,7 +147,7 @@ static int inews_init_objects (void)
    Inews_Post_Obj.po_printf = inews_printf;
    Inews_Post_Obj.po_puts = inews_puts;
    Inews_Post_Obj.po_can_post = 1;
-   Slrn_Inews_Pgm = slrn_safe_strmalloc (SLRN_INEWS_PROGRAM);
+   Slrn_Inews_Pgm = slrn_safe_strmalloc (SLRN_INEWS_COMMAND);
    return 0;
 }
 
@@ -465,41 +482,26 @@ unsigned int Pull_Num_Posted;
 char Pull_Post_Filename [SLRN_MAX_PATH_LEN + 1];
 char Pull_Post_Dir [SLRN_MAX_PATH_LEN + 1];
 
-static int pull_make_tempname (char *file, char *prefix)
+static int pull_make_tempname (char *file, size_t n, char *prefix,
+			       unsigned int num)
 {   
    int pid;
-   unsigned int max_tries;
    time_t now;
    char name[256];
-   static unsigned int num;
    char *login_name;
    
    pid = getpid ();
-   max_tries = 0;
    time (&now);
    
    login_name = Slrn_User_Info.login_name;
    if ((login_name == NULL) || (*login_name == 0))
      login_name = "unknown";
 
-   do
-     {
-	sprintf (name, "%s%lu-%d-%u.%s", prefix,
-		 (unsigned long) now, pid, num, login_name);
-	
-	if (-1 == slrn_dircat (Pull_Post_Dir, name, file))
-	  return ERR_FAULT;
-	
-	num++;
-	
-	max_tries++;
-	if (max_tries == 100)
-	  {
-	     slrn_error ("Unable to create temporary file in %s.", Pull_Post_Dir);
-	     return ERR_FAULT;
-	  }
-     }
-   while (0 != slrn_file_exists (file));
+   slrn_snprintf (name, sizeof (name), "%s%lu-%d-%u.%s", prefix,
+		  (unsigned long) now, pid, num, login_name);
+   
+   if (-1 == slrn_dircat (Pull_Post_Dir, name, file, n))
+     return -1;
    
    return 0;
 }
@@ -507,26 +509,37 @@ static int pull_make_tempname (char *file, char *prefix)
 
 static int pull_start_post (void)
 {
-
-   if (-1 == pull_make_tempname (Pull_Post_Filename, "_"))
-     return ERR_FAULT;
-
-   /* There is a race condition here but the chances of another file with this name
-    * being created is practically nil.
-    */
+   int fd;
+   unsigned int n = 0;
    
-   if (NULL == (Pull_Fp = fopen (Pull_Post_Filename, "w")))
+   do
      {
-	slrn_error ("Unable to open file in %s.", Pull_Post_Dir);
+	if (-1 == pull_make_tempname (Pull_Post_Filename,
+				      sizeof (Pull_Post_Filename), "_", n++))
+	  return ERR_FAULT;
+	fd = open (Pull_Post_Filename, O_WRONLY | O_CREAT | O_EXCL,
+		   S_IREAD | S_IWRITE);
+	if ((-1 == fd) && (n == 100))
+          {
+	     slrn_error ("Unable to open file in %s.", Pull_Post_Dir);
+	     return ERR_FAULT;
+	  }
+     }
+   while (-1 == fd);
+   
+   if (NULL == (Pull_Fp = fdopen (fd, "w")))
+     {
+	close (fd);
 	return ERR_FAULT;
      }
-
+   
    return CONT_POST;
 }
 
 static int pull_end_post (void)
 {
    char file [SLRN_MAX_PATH_LEN + 1];
+   unsigned int n = 0, m = 0;
    
    if (Pull_Fp == NULL)
      return ERR_FAULT;
@@ -554,17 +567,26 @@ static int pull_end_post (void)
       }
 #endif   
  
-   while (1)
+   while (m != 10)
      {
-	if (-1 == pull_make_tempname (file, "X"))
-	  return ERR_FAULT;
+	do
+	  {
+	     if ((100 == n) || (-1 == pull_make_tempname (file, sizeof (file),
+							  "X", n++)))
+	       return ERR_FAULT;
+	  }
+	while (0 != slrn_file_exists (file));
    
 	if (0 == rename (Pull_Post_Filename, file))
 	  break;
 	
-# ifdef EEXIST
-	if (errno == EEXIST) continue;
-# endif
+#ifdef EEXIST
+	if (errno == EEXIST)
+	  {
+	     m++;
+	     continue;
+	  }
+#endif
        
 	slrn_error ("Unable to rename file. errno = %d.", errno);
 	return ERR_FAULT;
@@ -621,7 +643,8 @@ static int pull_init_objects (void)
 
 static int pull_select_post_object (void)
 {
-   if (-1 == slrn_dircat (Slrn_Inn_Root, SLRNPULL_OUTGOING_DIR, Pull_Post_Dir))
+   if (-1 == slrn_dircat (Slrn_Inn_Root, SLRNPULL_OUTGOING_DIR,
+			  Pull_Post_Dir, sizeof (Pull_Post_Dir)))
      return -1;
    
    if (2 != slrn_file_exists (Pull_Post_Dir))
