@@ -3,6 +3,7 @@
  This file is part of SLRN.
 
  Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
+ Copyright (c) 2001 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -78,12 +79,16 @@
 #include "art.h"
 #include "score.h"
 #include "util.h"
+#include "snprintf.h"
 #include "chmap.h"
 
 #if SLRN_USE_SLTCP
 # include "sltcp.h"
 #endif
 
+#if SLRN_HAS_MIME 
+# include "mime.h"
+#endif
 #if SLRN_HAS_GROUPLENS
 # include "grplens.h"
 #endif
@@ -132,8 +137,10 @@ static int Current_Mouse_Mode;
 /*}}}*/
 /*{{{ Static Function Declarations */
 
+#if defined(SIGSTOP) && defined(REAL_UNIX_SYSTEM)
 static int suspend_display_mode (int);
 static int resume_display_mode (int, int, int);
+#endif
 static void init_suspend_signals (int);
 static void slrn_hangup (int);
 static void run_winch_functions (int, int);
@@ -212,18 +219,18 @@ static void lock_file (int how) /*{{{*/
    not_ok_to_unlock = 1;
 
 #ifdef SLRN_USE_OS2_FAT
-   slrn_os2_make_fat (name, Slrn_Newsrc_File, ".lck");
+   slrn_os2_make_fat (name, sizeof (name), Slrn_Newsrc_File, ".lck");
 #else
-   sprintf (name, "%s-lock", Slrn_Newsrc_File);
+   slrn_snprintf (name, sizeof (name), "%s-lock", Slrn_Newsrc_File);
 #endif
 
-   slrn_make_home_filename (name, file);
+   slrn_make_home_filename (name, file, sizeof (file));
    
 #if SLRN_HAS_RNLOCK
    if (0 == strcmp(".newsrc", Slrn_Newsrc_File))
      {
 	rnlock = 1;
-	slrn_make_home_filename (".rnlock", file_rn);
+	slrn_make_home_filename (".rnlock", file_rn, sizeof (file_rn));
      }
 #endif
 
@@ -338,6 +345,8 @@ void slrn_push_suspension (int ok) /*{{{*/
    (void) slrn_handle_interrupts ();
 
    Ok_To_Suspend = ok;
+#else
+   (void) ok;
 #endif
 }
 /*}}}*/
@@ -382,6 +391,8 @@ static void sig_suspend (int sig)
    
    init_suspend_signals (1);
    errno = sig;
+#else
+   (void) sig;
 #endif
 }
 
@@ -409,6 +420,7 @@ static void init_suspend_signals (int state) /*{{{*/
 static void perform_suspend (int smg_suspend_flag) /*{{{*/
 {
 #if !defined(SIGSTOP) || !defined(REAL_UNIX_SYSTEM)
+   (void) smg_suspend_flag;
    slrn_error ("Not implemented.");
    Want_Suspension = 0;
 #else
@@ -699,6 +711,7 @@ static int reset_smg (int smg_suspend_flag) /*{{{*/
 
 /*}}}*/
 
+#if defined(SIGSTOP) && defined(REAL_UNIX_SYSTEM)
 static int suspend_display_mode (int smg_suspend_flag) /*{{{*/
 {
    int mode = Slrn_TT_Initialized;
@@ -730,6 +743,7 @@ static int resume_display_mode (int smg_suspend_flag, int mode, int mouse_mode)
 }
 
 /*}}}*/
+#endif /* SIGSTOP && REAL_UNIX_SYSTEM*/
 
 void slrn_set_display_state (int state) /*{{{*/
 {
@@ -769,37 +783,6 @@ void slrn_enable_mouse (int mode) /*{{{*/
 
 /*}}}*/
 
-int slrn_get_new_news (int no_new_groups, int create_flag) /*{{{*/
-{
-   char *msg1 = "Checking for new groups ...";
-   char *msg2 = "Checking news ...";
-   char *msg3 = "Checking news via active file ...";
-   char *msg;
-   
-   if (Slrn_Server_Obj->sv_initialize () != 0) return (-1);
-   
-   if (create_flag == 0)
-     {
-	if (no_new_groups == 0)
-	  {
-	     slrn_message_now (msg1);
-	     slrn_check_new_groups (create_flag);
-	  }
-	
-	if (Slrn_List_Active_File) msg = msg3;
-	else msg = msg2;
-	
-	slrn_message_now (msg);
-     }
-   slrn_read_newsrc (create_flag);
-   
-   slrn_read_group_descriptions ();
-   
-   return 0;
-}
-
-/*}}}*/
-
 static void perform_cleanup (void)
 {
    if (Slrn_Server_Obj != NULL) 
@@ -879,7 +862,7 @@ Usage: slrn [--inews ...] [--nntp ...] [--spool ...] OPTIONS\n\
                  a faster startup.\n\
 -f newsrc-file  Name of the newsrc file to use.\n\
 -C              Use colors.\n\
--create         Create a newsrc file by getting list of groups from server.\n\
+--create        Create a newsrc file by getting list of groups from server.\n\
 -d              Get new text descriptions of each group from server.\n\
                  Note: This may take a LONG time to retrieve this information.\n\
                  The resulting file can be several hundred Kilobytes!\n\
@@ -887,6 +870,7 @@ Usage: slrn [--inews ...] [--nntp ...] [--spool ...] OPTIONS\n\
 -Dname          Add 'name' to list of predefined preprocessing tokens.\n\
 -k              Do not process score file.\n\
 -k0             Process score file but inhibit expensive scores.\n\
+--kill-log FILE Keep a log of all killed articles in FILE\n\
 -a              Use active file for getting new news\n\
 -m              Force XTerm mouse reporting\n\
 --help          Print this usage.\n\
@@ -944,7 +928,7 @@ static void read_score_file (void)
    if (Slrn_Score_File == NULL)
      return;
    
-   slrn_make_home_filename (Slrn_Score_File, file);
+   slrn_make_home_filename (Slrn_Score_File, file, sizeof (file));
    
    if (1 != slrn_file_exists (file))
      {
@@ -973,11 +957,15 @@ static int main_init_and_parse_args (int argc, char **argv) /*{{{*/
    FILE *fp;
    char file [SLRN_MAX_PATH_LEN];
    char *init_file = SLRN_USER_SLRNRC_FILENAME;
+   char *kill_logfile = NULL;
 
 #ifdef HAVE_SETLOCALE
 # ifdef LC_CTYPE
    (void) setlocale (LC_CTYPE, "");
 # endif
+# ifdef LC_TIME
+   (void) setlocale (LC_TIME, "");
+# endif
 #endif
 
    check_for_suspension ();
@@ -1032,6 +1020,9 @@ static int main_init_and_parse_args (int argc, char **argv) /*{{{*/
 	     else if (!strcmp ("create", argv_i)) create_flag = 1;
 	     else if (!strcmp ("version", argv_i))
 	       slrn_show_version ();
+	     else if ((!strcmp ("kill-log", argv_i)) &&
+		      (i + 1 < (unsigned int) argc))
+	       kill_logfile = argv[++i];
 	     else usage (NULL);
 	  }
 	else if (!strcmp ("-create", argv_i)) create_flag = 1;
@@ -1078,7 +1069,7 @@ static int main_init_and_parse_args (int argc, char **argv) /*{{{*/
 
    if (dsc_flag && create_flag)
      {
-	usage ("The -d and -create flags must not be specified together.");
+	usage ("The -d and --create flags must not be specified together.");
      }
    
    if (Slrn_Batch == 0)
@@ -1121,9 +1112,9 @@ static int main_init_and_parse_args (int argc, char **argv) /*{{{*/
    if (Slrn_Lib_Dir == NULL) Slrn_Lib_Dir = SLRN_LIB_DIR;
    
 #ifdef VMS
-   sprintf (file, "%s%s", Slrn_Lib_Dir, "slrn.rc");
+   slrn_snprintf (file, sizeof (file), "%s%s", Slrn_Lib_Dir, "slrn.rc");
 #else
-   sprintf (file, "%s/%s", Slrn_Lib_Dir, "slrn.rc");
+   slrn_snprintf (file, sizeof (file), "%s/%s", Slrn_Lib_Dir, "slrn.rc");
 #endif
 
    /* Make sure terminal is initialized before setting colors.  The 
@@ -1134,7 +1125,7 @@ static int main_init_and_parse_args (int argc, char **argv) /*{{{*/
 
    if (Slrn_Server_Id == 0) Slrn_Server_Id = Slrn_Default_Server_Obj;
    if (Slrn_Post_Id == 0) Slrn_Post_Id = Slrn_Default_Post_Obj;
-   if (Slrn_Check_New_Groups == 0) no_new_groups = 1;
+   if (no_new_groups) Slrn_Check_New_Groups = 0;
 
 #if SLRN_HAS_CHARACTER_MAP
    if (-1 == slrn_set_charset (Slrn_Charset))
@@ -1143,7 +1134,21 @@ static int main_init_and_parse_args (int argc, char **argv) /*{{{*/
 	slrn_exit_error ("Failed to select character set.");
      }
 #endif
-   
+
+#if SLRN_HAS_MIME
+   if (NULL == Slrn_Mime_Display_Charset)
+     {
+# if SLRN_HAS_CHARACTER_MAP
+	if ((NULL != Slrn_Charset) &&
+	    (0 == slrn_case_strcmp ((unsigned char *)Slrn_Charset,
+				    (unsigned char *)"ibm852")))
+	  Slrn_Mime_Display_Charset = slrn_safe_strmalloc ("iso-8859-2");
+	else
+# endif
+	  Slrn_Mime_Display_Charset = slrn_safe_strmalloc ("iso-8859-1");
+     }
+#endif
+
 #ifdef SIGINT
    if (Slrn_TT_Initialized == 0)
      SLsignal_intr (SIGINT, SIG_DFL);
@@ -1181,6 +1186,11 @@ hostname in your %s file.\r\n",
 Unable to find your user name.  This means that a valid 'From' header line\r\n\
 cannot be constructed.  Try setting the USER environment variable.\r\n");
      }
+
+   if (NULL == Slrn_User_Info.posting_host)
+     slrn_message_now ("\
+***Warning: unable to find a fully-qualified host name\r\n\
+            slrn will not generate any Message-IDs.\r\n");
    
    if (no_score_file == 0) read_score_file ();
    
@@ -1221,19 +1231,28 @@ cannot be constructed.  Try setting the USER environment variable.\r\n");
 	/* Check to see if the .newrc file exists--- I should use the access
 	 * system call but for now, do it this way.
 	 */
-	if (NULL == (fp = slrn_open_home_file (Slrn_Newsrc_File, "r", file, 0)))
+	if (NULL == (fp = slrn_open_home_file (Slrn_Newsrc_File, "r", file,
+					       sizeof (file), 0)))
 	  {
-	     fprintf(stderr, "Unable to open %s.  I will try .newsrc.\n", file);
-	     if (NULL == (fp = slrn_open_home_file (".newsrc", "r", file, 0)))
+	     slrn_error ("Unable to open %s.  I will try .newsrc.", file);
+	     if (NULL == (fp = slrn_open_home_file (".newsrc", "r", file,
+						    sizeof (file), 0)))
 	       {
 		  slrn_exit_error ("\r\nUnable to open %s.\r\n\
 If you want to create %s, add command line options:\r\n\
-   -f %s -create\r\n", file, file, file);
+   -f %s --create\r\n", file, file, file);
 	       }
 	  }
 	slrn_fclose (fp);
      }
    
+   if (kill_logfile != NULL)
+     {
+	Slrn_Kill_Log_FP = fopen(kill_logfile, "a");
+	if (Slrn_Kill_Log_FP == NULL)
+	  slrn_error ("Unable to open %s for logging.", kill_logfile);
+     }
+   
    lock_file (1);
 
 #if SLRN_HAS_SLANG
@@ -1255,12 +1274,20 @@ If you want to create %s, add command line options:\r\n\
 	  }
      }
 #endif
-	
-   if (-1 == slrn_get_new_news (no_new_groups, create_flag))
+
+   if (Slrn_Server_Obj->sv_initialize () != 0)
      {
 	slrn_exit_error ("Failed to initialize server.");
      }
    
+   if (Slrn_Check_New_Groups || create_flag)
+     {
+	slrn_get_new_groups (create_flag);
+     }
+   
+   slrn_read_newsrc (create_flag);
+   slrn_read_group_descriptions ();
+   
    putc ('\n', stdout);
    
    slrn_set_display_state (SLRN_SMG_INIT | SLRN_TTY_INIT);
