@@ -2,6 +2,7 @@
  This file is part of SLRN.
 
  Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
+ Copyright (c) 2001 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -35,7 +36,6 @@ extern void slrn_set_display_state (int);
 extern void slrn_push_suspension (int);
 extern void slrn_pop_suspension (void);
 
-extern int  slrn_get_new_news (int, int);
 extern char *Slrn_Newsrc_File;
 #if 0
 extern void (*Slrn_Hangup_Hook) (int);
@@ -69,6 +69,7 @@ extern void slrn_repeat_last_key (void);
 extern void slrn_call_command (char *);
 
 extern long slrn_date_to_order_parm (char *);
+extern void slrn_strftime (char *, size_t, const char *, char *, int);
 extern int slrn_parse_helpfile (char *);
 extern void slrn_smg_refresh (void);
 extern void slrn_enable_mouse (int);
@@ -85,36 +86,44 @@ extern int Slrn_Batch;
 
 extern int slrn_sys_system (char *);
 
-#define HEADER_COLOR	1
-#define GROUP_COLOR	2
-#define SUBJECT_COLOR	3
-#define AUTHOR_COLOR	4
-#define ERROR_COLOR	5
-#define CURSOR_COLOR	6
-#define ARTICLE_COLOR	7
-#define TREE_COLOR	8
-#define PGP_SIGNATURE_COLOR 9
-#define SIGNATURE_COLOR 10
-#define THREAD_NUM_COLOR 11
-#define HIGH_SCORE_COLOR 12
-#define MENU_PRESS_COLOR 13
-#define HEADER_NUMBER_COLOR 14
-#define GROUP_DESCR_COLOR 15
-#define GROUPLENS_DISPLAY_COLOR 16
-#define SLRN_TILDE_COLOR	17
-#define SLRN_HEADER_KEYWORD_COLOR	18
-#define BOX_COLOR	19
-#define STATUS_COLOR	20
-#define MENU_COLOR	21
-#define RESPONSE_CHAR_COLOR	22
-#define FRAME_COLOR		23
-#define SELECT_COLOR		24
-#define VERBATIM_COLOR		25
+#define HEADER_COLOR		1
+#define GROUP_COLOR		2
+#define AUTHOR_COLOR		3
+#define FROM_MYSELF_COLOR	4
+#define ERROR_COLOR		5
+#define CURSOR_COLOR		6
+#define ARTICLE_COLOR		7
+#define TREE_COLOR		8
+#define PGP_SIGNATURE_COLOR	9
+#define SIGNATURE_COLOR		10
+#define THREAD_NUM_COLOR	11
+#define MENU_PRESS_COLOR	12
+#define HEADER_NUMBER_COLOR	13
+#define GROUP_DESCR_COLOR	14
+#define GROUPLENS_DISPLAY_COLOR	15
+#define SLRN_TILDE_COLOR	16
+#define SLRN_HEADER_KEYWORD_COLOR	17
+#define BOX_COLOR		18
+#define STATUS_COLOR		19
+#define MENU_COLOR		20
+#define RESPONSE_CHAR_COLOR	21
+#define FRAME_COLOR		22
+#define SELECT_COLOR		23
+#define VERBATIM_COLOR		24
 #define DATE_COLOR		36
+/* SUBJECT_COLOR + 5 == UNREAD_SUBJECT_COLOR etc. */
+#define SUBJECT_COLOR		40
+#define NEG_SCORE_COLOR		41
+#define POS_SCORE_COLOR		42
+#define HIGH_SCORE_COLOR	43
+#define UNREAD_SUBJECT_COLOR	45
+#define UNREAD_NEG_SCORE_COLOR	46
+#define UNREAD_POS_SCORE_COLOR	47
+#define UNREAD_HIGH_SCORE_COLOR	48
 
 #define BOLD_COLOR		30
 #define UNDERLINE_COLOR		31
 #define ITALICS_COLOR		32
-#define QUOTE_COLOR	40
+#define QUOTE_COLOR	50
 
 #endif				       /* _SLRN_SLRN_H_ */
