@@ -2,13 +2,78 @@
  * Use sysconf.h for unix!!!  This file is for NON-unix systems.
  * 
  * This file is used to indicate capabilities of the C compiler and
- * operating system.
+ * operating system.  Additionally, you can set optional features
+ * here that are under autoconf control on unix.
  * 
- * See also slrnfeat.h for customization of slrn features.
+ * See also slrnfeat.h for the rest of slrn features.
  */
 
-#define USE_DOMAIN_NAME 0
-#define MY_DOMAIN_NAME "foo.bar"
+/* define if you want slrnpull support */
+#define SLRN_HAS_PULL_SUPPORT		1
+
+/* define if you want NNTP support */
+#define SLRN_HAS_NNTP_SUPPORT		1
+
+/* define if you want support for reading from a news spool */
+#define SLRN_HAS_SPOOL_SUPPORT		1
+
+/* define if you want support for posting via inews */
+#define SLRN_HAS_INEWS_SUPPORT		1
+
+/* define if you want to force the use of inews */
+#define SLRN_FORCE_INEWS		0
+
+/* define if you want to disallow custom From headers
+ * Works only on unix and VMS! */
+#define SLRN_HAS_STRICT_FROM 0
+
+/* define if you want SSL support */
+#define SLRN_HAS_SSL_SUPPORT	0
+
+/* define if you want slang interpreter support */
+#define SLRN_HAS_SLANG			1
+
+/* define if you want MIME support */
+#define SLRN_HAS_MIME			1
+
+/* define if you want slrn to map between ISO-Latin and native charsets */
+#define SLRN_HAS_CHARACTER_MAP		1
+
+/* define if you want spoiler support */
+#define SLRN_HAS_SPOILERS		1
+
+/* define if you want emphasized text support */
+#define SLRN_HAS_EMPHASIZED_TEXT	1
+
+/* define if you want verbatim marks */
+#define SLRN_HAS_VERBATIM_MARKS		1
+
+/* define if you want grouplens (R.I.P.) support */
+#define SLRN_HAS_GROUPLENS		0
+
+/* define if you want slrn to generate Message-IDs */
+#define SLRN_HAS_GEN_MSGID		1
+
+/* define to make slrn cache Message-IDs to eliminate cross-posts */
+#define SLRN_HAS_MSGID_CACHE		0
+
+/* define if slrn should use In-Reply-To if no References are available */
+#define SLRN_HAS_FAKE_REFS		1
+
+/* End of features section. See slrnfeat.h for the rest. */
+
+/* Does your compiler support vsnprintf()? */
+/* #define HAVE_VSNPRINTF 1 */
+
+#define SLTCP_HAS_SSL_SUPPORT SLRN_HAS_SSL_SUPPORT
+
+#if defined (__USE_SVID) || defined (__USE_XOPEN) || (defined (__MSVCRT__) && !defined (_NO_OLDNAMES))
+# define HAVE_TIMEZONE 1
+#endif
+
+#ifdef __USE_BSD
+# define HAVE_TM_GMTOFF 1
+#endif
 
 #ifdef VMS
 # ifndef MAIL_PROTOCOL
@@ -87,11 +152,17 @@
 # define HAVE_NETINET_IN_H 1
 # define HAVE_ARPA_INET_H 1
 # define HAVE_DIRENT_H 1
+# define HAVE_FCNTL_H 1
 #endif
 
 #if defined(__NT__) || defined(__WIN32__)
-# define HAVE_UNISTD_H 1
+# ifndef __CYGWIN__
+#  define HAVE_UNISTD_H 1
+# endif
 # define HAVE_MEMORY_H 1
+# define HAVE_FCNTL_H 1
+# define HAVE_LOCALE_H 1
+# define HAVE_SETLOCALE 1
 # if defined(__CYGWIN32__) || defined(__MINGW32__)
 #  define HAVE_DIRENT_H 1
 # else
@@ -111,6 +182,9 @@
 # if __VMS_VER >= 60200000
 #  define HAVE_UNISTD_H
 # endif
+# if __VMS_VER >= 70000000
+#  define HAVE_GETTIMEOFDAY
+# endif
 #endif
 
 /* Does VMS have these?  Do *all* VMS version have them? */
