@@ -3,6 +3,7 @@
  This file is part of SLRN.
 
  Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
+ Copyright (c) 2001 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -58,13 +59,22 @@
 
 #include "ttymsg.h"
 #include "util.h"
-#include "sltcp.h"
+#include "snprintf.h"
+#if SLRN_USE_SLTCP
+# include "sltcp.c"
+#else
+# include "clientlib.c"
+#endif
 #include "nntplib.h"
 #include "nntpcodes.h"
 #include "slrndir.h"
 #include "version.h"
 
+#include "sortdate.c"
 #include "score.c"
+#if SLRN_HAS_MIME
+# include "mime.c"
+#endif
 #include "xover.c"
 
 #undef SLRN_HAS_MSGID_CACHE
@@ -138,6 +148,7 @@ char *Data_Dir = "data";
 static char *Active_File = "active";
 
 static int Stdout_Is_TTY;
+static int Kill_Score;
 static char *Active_Groups_File;
 static time_t Start_Time;
 
@@ -201,6 +212,7 @@ static Active_Group_Type *Active_Groups_Tail;
 
 static FILE *MLog_Fp;
 static FILE *ELog_Fp;
+static FILE *KLog_Fp;
 
 static void write_timestamp (FILE *fp) /*{{{*/
 {
@@ -231,7 +243,7 @@ static void va_log (FILE *fp, char *pre, char *fmt, va_list ap) /*{{{*/
 {
    char buf[2048];   
 
-   vsprintf (buf, fmt, ap);
+   slrn_vsnprintf (buf, sizeof (buf), fmt, ap);
    
    write_log (fp, pre, buf);
    
@@ -347,11 +359,11 @@ static int do_mkdir (char *dir, int err) /*{{{*/
 /*}}}*/
 
 static FILE *open_group_min_max_file (Active_Group_Type *g, char *mode, /*{{{*/
-				      char *file)
+				      char *file, size_t n)
 {
-   if (-1 == slrn_dircat (SlrnPull_Spool_News_Dir, g->dirname, file))
+   if (-1 == slrn_dircat (SlrnPull_Spool_News_Dir, g->dirname, file, n))
      return NULL;
-   if (-1 == slrn_dircat (file, Group_Min_Max_File, file))
+   if (-1 == slrn_dircat (file, Group_Min_Max_File, file, n))
      return NULL;
    
    return fopen (file, mode);
@@ -372,7 +384,7 @@ static int read_group_min_max_file (Active_Group_Type *g) /*{{{*/
    g->min = 1;
    g->max = 0;
 
-   fp = open_group_min_max_file (g, "r", file);
+   fp = open_group_min_max_file (g, "r", file, sizeof (file));
    
    if (fp == NULL)
      return 0;
@@ -405,7 +417,7 @@ static int write_group_min_max_file (Active_Group_Type *g) /*{{{*/
    char file[SLRN_MAX_PATH_LEN + 1];
    FILE *fp;
    
-   fp = open_group_min_max_file (g, "w", file);
+   fp = open_group_min_max_file (g, "w", file, sizeof (file));
    if (fp == NULL)
      {
 	log_error ("Unable to open %s for writing.", file);
@@ -434,7 +446,7 @@ static int create_group_directory (Active_Group_Type *g) /*{{{*/
    unsigned int len;
    int status;
       
-   strcpy (g->dirname, g->name);
+   strcpy (g->dirname, g->name); /* safe */
    
    d = g->dirname;
    while (*d != 0)
@@ -459,18 +471,22 @@ static int create_group_directory (Active_Group_Type *g) /*{{{*/
 
    len = strlen (SlrnPull_Spool_News_Dir);
    
-   strcpy (dirbuf, SlrnPull_Spool_News_Dir);
+   if (sizeof (dirbuf) < len + strlen (g->dirname) + 2)
+     return -1;
+   
+   strcpy (dirbuf, SlrnPull_Spool_News_Dir); /* safe */
+   
    dirbuf [len] = SLRN_PATH_SLASH_CHAR;
    len++;
    
    dir = dirbuf + len;
-   strcpy (dir, g->dirname);
+   strcpy (dir, g->dirname); /* safe */
    
    d = dir + strlen (dir);
-
+   
    if (0 == do_mkdir (dirbuf, 0))
      return 0;
-     
+   
    /* Go back and create it piece by piece. */
    d = dir;
    do
@@ -929,7 +945,6 @@ static unsigned int Num_Articles_To_Receive;
 
 static void print_time_stats (NNTP_Type *s, int do_log) /*{{{*/
 {
-   char buf[512];
    time_t now;
    unsigned long in;
    unsigned long elapsed_time;
@@ -960,13 +975,10 @@ static void print_time_stats (NNTP_Type *s, int do_log) /*{{{*/
    if (Stdout_Is_TTY)
      {
 	
-	sprintf (buf, "%u/%u (%u killed), Time: %02u:%02u:%02u, BPS: %lu      ",
+	fprintf (stdout, "%u/%u (%u killed), Time: %02u:%02u:%02u, BPS: %lu      \r",
 		 Num_Articles_Received, Num_Articles_To_Receive, Num_Killed,
 		 hour, min, sec,
 		 (unsigned long) (in / elapsed_time));
-	
-	fputs (buf, stdout);
-	fputc ('\r', stdout);
 	fflush (stdout);
      }
    
@@ -1024,10 +1036,11 @@ static int write_head_and_body (Active_Group_Type *g, int n, /*{{{*/
 	return 0;
      }
    
-   sprintf (buf, "%d", n);
+   slrn_snprintf (buf, sizeof (buf), "%d", n);
    
-   if ((-1 == slrn_dircat (SlrnPull_Spool_News_Dir, g->dirname, file))
-       || (-1 == slrn_dircat (file, buf, file)))
+   if ((-1 == slrn_dircat (SlrnPull_Spool_News_Dir, g->dirname,
+			   file, sizeof (file)))
+       || (-1 == slrn_dircat (file, buf, file, sizeof (file))))
      return -1;
    
    fp = fopen (file, "w");
@@ -1105,7 +1118,8 @@ static int get_bodies (NNTP_Type *s, int *numbers, /*{{{*/
 	if (heads[i] == NULL)
 	  continue;
 	
-	sprintf (b, "%sbody %d", crlf, numbers[i]);
+	slrn_snprintf (b, sizeof (buf) - (size_t)(b - buf),
+		       "%sbody %d", crlf, numbers[i]);
 	b += strlen (b);
 	
 	crlf = "\r\n";
@@ -1161,14 +1175,25 @@ static int fetch_head (NNTP_Type *s, int n, char **headers, Slrn_XOver_Type *xov
    
    slrn_map_xover_to_header (xov, &h);
 
+#if SLRN_HAS_MIME
+   if (Slrn_Use_Mime)
+     {
+	slrn_rfc1522_decode_string (h.subject);
+	slrn_rfc1522_decode_string (h.from);
+     }
+#endif
+   
 #if 1
    (void) is_msgid_cached (h.msgid, Current_Newsgroup, (unsigned int) n, 1);
 #endif
 
    score = slrn_score_header (&h, Current_Newsgroup);
-   if (score < 0)
+   if (score < Kill_Score)
      {
 	Num_Killed++;
+	if (KLog_Fp != NULL)
+	  fprintf (KLog_Fp, "Score %d killed article %s\nNewsgroup: %s\nFrom: %s\nSubject: %s\n\n",
+		   score, h.msgid, Current_Newsgroup, h.from, h.subject);
 	slrn_free (*headers);
 	*headers = NULL;
 	return 0;
@@ -1196,7 +1221,8 @@ static int get_heads (NNTP_Type *s, int *numbers, char **heads, /*{{{*/
    /* Final crlf added by nntp_start_server_cmd. */
    for (i = 0; i < num; i++)
      {
-	sprintf (b, "%shead %d", crlf, numbers[i]);
+	slrn_snprintf (b, sizeof (buf) - (size_t) (b - buf),
+		       "%shead %d", crlf, numbers[i]);
 	crlf = "\r\n";
 	b += strlen (b);
 	
@@ -1225,8 +1251,10 @@ static FILE *open_xover_file (Active_Group_Type *g, char *mode) /*{{{*/
    FILE *fp;
    
    fp = NULL;
-   if ((-1 != slrn_dircat (SlrnPull_Spool_News_Dir, g->dirname, ov_file))
-       && (-1 != slrn_dircat (ov_file, Overview_File, ov_file)))
+   if ((-1 != slrn_dircat (SlrnPull_Spool_News_Dir, g->dirname,
+			   ov_file, sizeof (ov_file)))
+       && (-1 != slrn_dircat (ov_file, Overview_File,
+			      ov_file, sizeof (ov_file))))
      {
 	fp = fopen (ov_file, mode);
 	if (fp == NULL)
@@ -1484,7 +1512,8 @@ static int post_file (NNTP_Type *s, char *file) /*{{{*/
 	log_error ("Article %s rejected. status = %d: %s.", file, status, s->rspbuf);
 	
 	name = slrn_basename (file);
-	if (-1 == slrn_dircat (Outgoing_Bad_Dir, name, bad_file))
+	if (-1 == slrn_dircat (Outgoing_Bad_Dir, name,
+			       bad_file, sizeof (bad_file)))
 	  return -1;
 	
 	log_error ("Saving article in %s...", Outgoing_Bad_Dir);
@@ -1550,7 +1579,7 @@ static int post_outgoing (NNTP_Type *s) /*{{{*/
 	if (*name != 'X')
 	  continue;
 	
-	if (-1 == slrn_dircat (Outgoing_Dir, name, file))
+	if (-1 == slrn_dircat (Outgoing_Dir, name, file, sizeof (file)))
 	  break;
 	
 	if (1 != slrn_file_exists (file))
@@ -1577,7 +1606,7 @@ static int write_active (void) /*{{{*/
    FILE *fp;
    char file [SLRN_MAX_PATH_LEN + 5];
    
-   sprintf (file, "%s.tmp", Active_File);
+   slrn_snprintf (file, sizeof (file), "%s.tmp", Active_File);
    
    fp = fopen (file, "w");
    if (fp == NULL)
@@ -1633,17 +1662,20 @@ static void connection_lost_hook (NNTP_Type *s)
    
 static int open_servers (char *host) /*{{{*/
 {
+#if SLRN_USE_SLTCP
    if (-1 == sltcp_open_sltcp ())
      {
 	fprintf (stderr, "Error opening sltcp interface.\n");
 	return -1;
      }
-
+#endif
    time (&Actual_Start_Time);
    Pull_Server = nntp_open_server (host, -1);
    if (Pull_Server == NULL)
      {
+#if SLRN_USE_SLTCP
 	sltcp_close_sltcp ();
+#endif
 	return -1;
      }
 
@@ -1661,7 +1693,9 @@ static int open_servers (char *host) /*{{{*/
    /* Probe for XHDR now because DNEWS does not handle probing later properly. */
    if (-1 == nntp_has_cmd (Pull_Server, "XHDR"))
      {
+#if SLRN_USE_SLTCP
 	sltcp_close_sltcp ();
+#endif
 	return -1;
      }
    return 0;
@@ -1689,7 +1723,9 @@ static void close_servers (void) /*{{{*/
    unsigned long bytes_in = 0;
    unsigned long bytes_out = 0;
 
+#if SLRN_USE_SLTCP
    (void) sltcp_close_sltcp ();
+#endif
 
    if (Pull_Server != NULL)
      {
@@ -1708,14 +1744,14 @@ static void close_servers (void) /*{{{*/
 
 static void init_signals (void);
 
-static void open_log_files (char *logfile) /*{{{*/
+static void open_log_files (char *logfile, char *kill_logfile) /*{{{*/
 {
    char file [SLRN_MAX_PATH_LEN + 1];
    
    ELog_Fp = stderr;
    MLog_Fp = stdout;
    
-   if (-1 == slrn_dircat (SlrnPull_Dir, logfile, file))
+   if (-1 == slrn_dircat (SlrnPull_Dir, logfile, file, sizeof (file)))
      return;
 
    MLog_Fp = fopen (file, "a");
@@ -1727,6 +1763,14 @@ static void open_log_files (char *logfile) /*{{{*/
      }
    
    ELog_Fp = MLog_Fp;
+   
+   if ((kill_logfile == NULL) || 
+       (-1 == slrn_dircat (SlrnPull_Dir, kill_logfile, file, sizeof (file))))
+     return;
+   
+   KLog_Fp = fopen (file, "a");
+   if (KLog_Fp == NULL)
+     log_error ("Unable to open %s for logging.", file);
 }
 
 /*}}}*/
@@ -1739,6 +1783,9 @@ static void close_log_files (void) /*{{{*/
    if ((ELog_Fp != NULL) && (ELog_Fp != stderr) && (ELog_Fp != MLog_Fp))
      fclose (ELog_Fp);
    
+   if (KLog_Fp != NULL)
+     fclose (KLog_Fp);
+   
    ELog_Fp = stderr;
    MLog_Fp = stdout;
 }
@@ -1749,15 +1796,17 @@ static void usage (char *pgm) /*{{{*/
 {
    log_error ("%s usage: %s [options]\n\
  Options:\n\
-  -h HOSTNAME       name of NNTP server\n\
-  -d SPOOLDIR       Spool Directory associated with server\n\
-  --expire          Perform expiration but do not pull news\n\
-  --post-only       Post news but do not pull new news\n\
-  --no-post         Do not post news\n\
-  --new-groups      Get new groups\n\
-  --version         Show version number\n\
-  --logfile FILE    Use FILE as the log file\n\
-  --debug FILE      Write dialogue with server to FILE\n\
+  -h HOSTNAME          name of NNTP server\n\
+  -d SPOOLDIR          Spool Directory associated with server\n\
+  --expire             Perform expiration but do not pull news\n\
+  --post-only          Post news but do not pull new news\n\
+  --no-post            Do not post news\n\
+  --new-groups         Get new groups\n\
+  --kill-score SCORE   Kill articles with a score below SCORE\n\
+  --kill-log FILE      Keep a log of all killed articles in FILE\n\
+  --logfile FILE       Use FILE as the log file\n\
+  --debug FILE         Write dialogue with server to FILE\n\
+  --version            Show version number\n\
 ",
 	      pgm, pgm);
    close_log_files ();
@@ -1778,7 +1827,8 @@ static int read_score_file (void) /*{{{*/
 {
    char file [SLRN_MAX_PATH_LEN + 1];
    
-   if (-1 == slrn_dircat (SlrnPull_Dir, SLRNPULL_SCORE_FILE, file))
+   if (-1 == slrn_dircat (SlrnPull_Dir, SLRNPULL_SCORE_FILE,
+			  file, sizeof (file)))
      return -1;
    
    if (1 != slrn_file_exists (file))
@@ -1802,6 +1852,7 @@ int main (int argc, char **argv) /*{{{*/
    int check_new_groups = 0;
    char *dir;
    char *logfile;
+   char *kill_logfile = NULL;
 
    pgm = argv[0];
    argv++; argc--;
@@ -1850,6 +1901,16 @@ int main (int argc, char **argv) /*{{{*/
 	     logfile = *argv;
 	     argv++; argc--;
 	  }
+	else if (!strcmp (arg, "--kill-log") && (argc > 0))
+	  {
+	     kill_logfile = *argv;
+	     argv++; argc--;
+	  }
+	else if (!strcmp (arg, "--kill-score") && (argc > 0))
+	  {
+	     Kill_Score = atoi (*argv);
+	     argv++; argc--;
+	  }
 	else if (!strcmp (arg, "--debug") && (argc > 0))
 	  {
 	     if (NULL == (NNTP_Debug_Fp = fopen (*argv, "w")))
@@ -1857,6 +1918,8 @@ int main (int argc, char **argv) /*{{{*/
 		  fprintf (stderr, "Unable to create debug file %s\n", *argv);
 		  exit (1);
 	       }
+             else
+	       setbuf (NNTP_Debug_Fp, (char*) NULL);
 	     
 	     argv++;
 	     argc--;
@@ -1874,7 +1937,7 @@ int main (int argc, char **argv) /*{{{*/
 	return 1;
      }
    
-   open_log_files (logfile);
+   open_log_files (logfile, kill_logfile);
 
    if (expire_mode)
      log_message ("slrnpull started in expire mode.");
@@ -2024,25 +2087,38 @@ static int get_new_groups (NNTP_Type *s)
 	*p = 0;
 	
 	parse_error = 0;
+	i = 0;
 	
-	switch (nntp_server_cmd (s, line))
+	do
 	  {
-	   case OK_NEWGROUPS:
-	     break;
-	     
-	   case ERR_FAULT:
-	     return 0;
-	     
-	   case ERR_COMMAND:
-	     log_message ("Server does not implement NEWGROUPS command.");
-	     return 0;
+	     switch (nntp_server_cmd (s, line))
+	       {
+		case OK_NEWGROUPS:
+		  break;
+		  
+		case ERR_FAULT:
+		  return 0;
+		  
+		case ERR_COMMAND:
+		  log_message ("Server does not implement NEWGROUPS command.");
+		  return 0;
 
-	   default:
-	     slrn_message ("Server failed to return proper response to NEWGROUPS:\n%s\n",
-			   s->rspbuf);
-
-	     goto parse_error_label;
+		case ERR_NOAUTH:
+		  if ((i++ != 0) || (-1 == nntp_authorization (s)))
+		    {
+		       slrn_exit_error ("Authorization failed.");
+		       return -1;
+		    }
+		  break;
+		  
+		default:
+		  slrn_message ("Server failed to return proper response to NEWGROUPS:\n%s\n",
+				s->rspbuf);
+		  
+		  goto parse_error_label;
+	       }
 	  }
+	while (i++ == 1); /* loops (at most) once */
       
 	num = 0;
 	while (1 == nntp_read_line (s, line, sizeof (line)))
@@ -2117,7 +2193,7 @@ void slrn_exit_error (char *fmt, ...) /*{{{*/
 
 
 static int Terminate_Slrn_Pull_Requested;
-static void sigint_handler (int sig) /*{{{*/
+static void slrnpull_sigint_handler (int sig) /*{{{*/
 {
    (void) sig;
 
@@ -2130,11 +2206,11 @@ static void sigint_handler (int sig) /*{{{*/
 
 static void init_signals (void) /*{{{*/
 {
-   SLsignal_intr (SIGINT, sigint_handler);
+   SLsignal_intr (SIGINT, slrnpull_sigint_handler);
 #ifdef SIGHUP
-   SLsignal_intr (SIGHUP, sigint_handler);
+   SLsignal_intr (SIGHUP, slrnpull_sigint_handler);
 #endif
-   SLsignal_intr (SIGTERM, sigint_handler);
+   SLsignal_intr (SIGTERM, slrnpull_sigint_handler);
 #ifdef SIGPIPE
    SLsignal_intr (SIGPIPE, SIG_IGN);
 #endif
@@ -2245,7 +2321,7 @@ static char *read_header_from_file (char *file)
 	     mbuf = new_mbuf;
 	  }
    
-	strcpy (mbuf + buffer_len, line);
+	strcpy (mbuf + buffer_len, line); /* safe */
 	buffer_len += len;
      }
    
@@ -2275,7 +2351,8 @@ static int create_overview_for_dir (Active_Group_Type *g, unsigned int *nums, un
    if (xov_fp == NULL)
      return -1;
    
-   if (-1 == slrn_dircat (SlrnPull_Spool_News_Dir, g->dirname, dir))
+   if (-1 == slrn_dircat (SlrnPull_Spool_News_Dir, g->dirname,
+			  dir, sizeof (dir)))
      return -1;
    
    if ((nums != NULL) && (n_nums != 0))
@@ -2306,9 +2383,9 @@ static int create_overview_for_dir (Active_Group_Type *g, unsigned int *nums, un
 	
 	id = (int) nums [i];
 	
-	sprintf (buf, "%d", id);
+	sprintf (buf, "%d", id); /* safe */
 	
-	if (-1 == slrn_dircat (dir, buf, file))
+	if (-1 == slrn_dircat (dir, buf, file, sizeof (file)))
 	  continue;
 	
 	if (-1 == stat (file, &st))
@@ -2361,7 +2438,8 @@ static int expire_group (Active_Group_Type *g) /*{{{*/
    if (g->expire_days == 0)
      perform_expire = 0;
 
-   if (-1 == slrn_dircat (SlrnPull_Spool_News_Dir, g->dirname, dir))
+   if (-1 == slrn_dircat (SlrnPull_Spool_News_Dir, g->dirname,
+			  dir, sizeof (dir)))
      return -1;
 
    dp = slrn_open_dir (dir);
@@ -2433,9 +2511,9 @@ static int expire_group (Active_Group_Type *g) /*{{{*/
 	     continue;
 	  }
 	
-	sprintf (buf, "%d", n);
+	sprintf (buf, "%d", n); /* safe */
 	
-	if (-1 == slrn_dircat (dir, buf, file))
+	if (-1 == slrn_dircat (dir, buf, file, sizeof (file)))
 	  continue;
 	
 	if (-1 == stat (file, &st))
@@ -2524,7 +2602,7 @@ static int read_authinfo (void)
    Auth_Password = NULL;
    Auth_User_Name = NULL;
    
-   if (-1 == slrn_dircat (SlrnPull_Dir, "authinfo", file))
+   if (-1 == slrn_dircat (SlrnPull_Dir, "authinfo", file, sizeof (file)))
      return -1;
    
    if (NULL == (fp = fopen (file, "r")))
