@@ -1,4 +1,5 @@
 /* Local spool support for slrn added by Olly Betts <olly@mantis.co.uk> */
+/* Modified by Thomas Schultz <tststs@gmx.de> */
 #include "config.h"
 #include "slrnfeat.h"
 
@@ -27,6 +28,7 @@
 #include "misc.h"
 #include "slrn.h"
 #include "slrndir.h"
+#include "snprintf.h"
 
 #ifndef SLRN_SPOOL_ROOT
 # define SLRN_SPOOL_ROOT "/var/spool/news" /* a common place for the newsspool */
@@ -55,12 +57,16 @@ static int spool_read_line (char *, unsigned int );
 static int spool_xpat_cmd (char *, int, int, char *);
 static int spool_xgtitle_cmd (char *);
 static int spool_select_article (int, char *);
-static int spool_xhdr_command (char *, int, char *, unsigned int);
+static int spool_one_xhdr_command (char *, int, char *, unsigned int);
+static int spool_xhdr_command (char *, int, int);
 static int spool_list_newsgroups (void);
 static int spool_list_active (void);
+static int spool_send_authinfo (void);
 static int spool_read_xover (char *, unsigned int);
 static int spool_read_xpat (char *, unsigned int);
+static int spool_read_xhdr (char *, unsigned int);
 static int spool_article_num_exists (int);
+static unsigned int spool_get_bytes (int clear);
 
 static Slrn_Server_Obj_Type Spool_Server_Obj;
 
@@ -78,6 +84,7 @@ static int Spool_Min_Artnum = 0;
 
 static int Spool_Doing_XOver;	       /* if non-zero, reading from ,overview */
 static int Spool_Doing_XPat;	       /* reading xpat */
+static char *Spool_XHdr_Field;	       /* when reading xhdr */
 static int Spool_fhead=0; /* if non-0 we're emulating "HEAD" so stop on blank line */
 static int Spool_fFakingActive=0; /* if non-0 we're doing funky stuff with MH folders */
 
@@ -85,11 +92,13 @@ static int spool_fake_active( char *);
 static int spool_fakeactive_read_line(char *, int);
 static int Spool_fakeactive_newsgroups=0;
 
+static unsigned int bytes_read=0;
+
 #if DEBUG_SPOOL
 # include <stdarg.h>
 static void spool_debug (char *fmt, ...)
 {
-   FILE *fp = NULL;
+   static FILE *fp = NULL;
    va_list ap;
 
    if (fp == NULL)
@@ -317,7 +326,7 @@ static int spool_find_artnum_from_msgid (char *msgid)
    
 	for (n = Spool_Min_Artnum; n <= Spool_Max_Artnum; n++)
 	  {
-	     if (-1 == spool_xhdr_command ("Message-Id", n, buf, sizeof (buf)))
+	     if (-1 == spool_one_xhdr_command ("Message-Id", n, buf, sizeof (buf)))
 	       continue;
 	     
 	     p = slrn_skip_whitespace (buf);
@@ -369,7 +378,7 @@ static FILE *spool_open_article_num (int num)
 {
    char buf [SLRN_MAX_PATH_LEN];
    
-   sprintf (buf, "%s/%d", Spool_Group, num);
+   slrn_snprintf (buf, sizeof (buf), "%s/%d", Spool_Group, num);
    
    return fopen (buf,"r");
 }
@@ -378,7 +387,7 @@ static int spool_article_num_exists (int num)
 {
    char buf [SLRN_MAX_PATH_LEN];
    
-   sprintf (buf, "%s/%d", Spool_Group, num);
+   slrn_snprintf (buf, sizeof (buf), "%s/%d", Spool_Group, num);
 
    if (1 == slrn_file_exists (buf))
      return 0;
@@ -462,7 +471,6 @@ static int spool_nntp_newgroups (char *line, char *buf, unsigned int len)
    int fGMT;
    int n;
    time_t threshold;
-   struct tm t;
    long fpos;
    char buf1[512];
    int ch;
@@ -491,32 +499,58 @@ static int spool_nntp_newgroups (char *line, char *buf, unsigned int len)
    spool_debug ("%d read, %04d-%02d-%02d %d:%02d:%02d %s",
 		c,Y,M,D,h,m,s,fGMT?"GMT":"(local)");
 #endif
-   t.tm_year = Y - 1900; /* tm_year is years since 1900 */
-   t.tm_mon = M - 1; /* tm_mon has January as 0 */
-   t.tm_mday = D;
-   t.tm_hour = h;
-   t.tm_min = m;
-   t.tm_sec = s;
    
-   /* This used to be 0, but the man page indicates that -1 should be used
-    * if the information is not available.
-    */
-   t.tm_isdst = -1; /* say it's not DST (? !HACK!) */
-   threshold = mktime( &t );
+   if (Y < 1970
+       || M < 1 || M > 12
+       || D < 1 || D > 31
+       || h < 0 || h > 24
+       || m < 0 || m > 59
+       || s < 0 || s > 59)
+     return ERR_FAULT;
 
-#if DEBUG_SPOOL
-   spool_debug ("threshold (local) = %d",threshold);
-#endif
-
-   if (fGMT)
+   /* Compute the number of days since beginning of year.
+    * Beware: the calculations involving leap years, etc... are naive. */
+   D--; M--;
+   D += 31 * M;
+   if (M > 1)
      {
-	/* !HACK! is this correct? */
-	threshold -= mktime (gmtime (&threshold)) - threshold;
+	D -= (M * 4 + 27)/10;
+	if ((Y % 4) == 0) D++;
+     }
+   
+   /* add that to number of days since beginning of time */
+   Y -= 1970;
+   D += Y * 365 + Y / 4;
+   if ((Y % 4) == 3) D++;
+   
+   /* Now convert to secs */
+   s += 60L * (m + 60L * (h + 24L * D));
+   threshold = (time_t) s;
+
+   if (0 == fGMT)
+     {
+	struct tm *t;
+#ifdef HAVE_TIMEZONE
+	(void) localtime (&threshold); /* This call sets timezone */
+	threshold += timezone;
+	t = localtime (&threshold);
+	if (t->tm_isdst) threshold -= (time_t) 3600;
+#else
+# ifdef HAVE_TM_GMTOFF
+	time_t tt;
+	
+	t = localtime (&threshold);
+	tt = threshold - t->tm_gmtoff;
+	if (t->tm_isdst) tt += (time_t) 3600;
+	t = localtime (&tt);
+	threshold -= t->tm_gmtoff;
+# endif /* else: we're out of luck */
+#endif /* NOT HAVE_TIMEZONE */
+     }
 
 #if DEBUG_SPOOL
-	spool_debug ("threshold (GMT) = %d",threshold);
+   spool_debug ("threshold = %d",threshold);
 #endif
-     }
 
    spool_fclose_local();
    Spool_fh_local = fopen (Slrn_ActiveTimes_File, "r");
@@ -937,6 +971,11 @@ static int spool_select_group (char *name, int *min, int *max)
    return OK_GROUP;
 }
 
+static char *spool_current_group (void)
+{
+   return Spool_Group_Name == NULL ? "" : Spool_Group_Name;
+}
+
 static int Spool_Server_Inited = 0;
 
 static void spool_close_server (void)
@@ -974,7 +1013,7 @@ static int spool_initialize_server (void)
 
    if (2 != slrn_file_exists (Slrn_Spool_Root))
      {
-	slrn_message_now ("Local spool directory '%s' doesn't exist.", Slrn_Spool_Root);
+	slrn_error ("Local spool directory '%s' doesn't exist.", Slrn_Spool_Root);
 	return -1;
      }
 
@@ -994,6 +1033,8 @@ static int spool_read_line (char *line, unsigned int len)
    
    if (Spool_Doing_XOver) return spool_read_xover (line, len);
    
+   if (NULL != Spool_XHdr_Field) return spool_read_xhdr (line, len);
+   
    if (Spool_fFakingActive) return spool_fakeactive_read_line (line, len);
    
    if ((NULL == Spool_fh_local)
@@ -1005,6 +1046,9 @@ static int spool_read_line (char *line, unsigned int len)
      }
 
    len = strlen(line);
+
+   bytes_read += len;
+   
    if (len && (line [len - 1] == '\n'))
      line [len-1] = '\0';
 
@@ -1042,8 +1086,8 @@ static int spool_read_xpat (char *buf, unsigned int len)
 	Spool_Doing_XPat = 0;
 	for (num = Spool_XPat_Struct.rmin; num <= Spool_XPat_Struct.rmax; num++)
 	  {
-	     if (-1 == spool_xhdr_command (Spool_XPat_Struct.header, (int) num,
-					   tmpbuf, sizeof (tmpbuf)))
+	     if (-1 == spool_one_xhdr_command (Spool_XPat_Struct.header, (int) num,
+					       tmpbuf, sizeof (tmpbuf)))
 	       continue;
 	     
 	     if (-1 != spool_xpat_match (tmpbuf, Spool_XPat_Struct.pat))
@@ -1053,7 +1097,7 @@ static int spool_read_xpat (char *buf, unsigned int len)
 		  Spool_Doing_XPat = 1;
 		  Spool_XPat_Struct.rmin = num + 1;
 		  
-		  sprintf (buf, "%d ", num);
+		  slrn_snprintf (buf, len, "%d ", num);
 		  blen = strlen (buf);
 		  
 		  strncpy (buf + blen, tmpbuf, len - blen);
@@ -1163,7 +1207,8 @@ static int spool_select_article (int n, char *msgid)
 
 
 /* The hdr string should NOT include the ':' */
-static int spool_xhdr_command (char *hdr, int num, char *buf, unsigned int buflen)
+static int spool_one_xhdr_command (char *hdr, int num, char *buf,
+				   unsigned int buflen)
 {
    char tmpbuf [1024];
    unsigned int colon;
@@ -1198,6 +1243,51 @@ static int spool_xhdr_command (char *hdr, int num, char *buf, unsigned int bufle
    return -1;
 }
 
+static int spool_xhdr_command (char *field, int min, int max)
+{
+#if DEBUG_SPOOL
+   spool_debug ("spool_xhdr_command(%s,%d,%d)", field, min, max);
+#endif
+   
+   spool_fclose_local ();
+   
+   if (max > Spool_Max_Artnum)
+     max = Spool_Max_Artnum;
+   
+   if (min < Spool_Min_Artnum)
+     min = Spool_Min_Artnum;
+   
+   Spool_XOver_Next = Spool_XOver_Min = min;
+   Spool_XOver_Max = max;
+   Spool_XHdr_Field = field;
+   
+   return OK_HEAD;
+}
+
+static int spool_read_xhdr (char *the_buf, unsigned int len)
+{
+   int retval = -1;
+   
+#if DEBUG_SPOOL
+   spool_debug ("spool_read_xhdr");
+#endif
+
+   if (Spool_XHdr_Field == NULL)
+     return 0;
+
+   if (Spool_XOver_Next > Spool_XOver_Max) 
+     {
+	Spool_XHdr_Field = NULL;
+	return 0;
+     }
+   
+   while ((retval == -1) && (Spool_XOver_Next <= Spool_XOver_Max))
+     retval = spool_one_xhdr_command (Spool_XHdr_Field, Spool_XOver_Next++,
+				      the_buf, len);
+   
+   return retval;
+}
+
 static int spool_list_newsgroups (void)
 {
    spool_fclose_local();
@@ -1235,6 +1325,11 @@ static int spool_list_active (void)
    return OK_GROUPS;
 }
 
+static int spool_send_authinfo (void)
+{
+   return 0;
+}
+
 typedef struct _Spool_DirTree_Type
 {
    struct _Spool_DirTree_Type *parent;
@@ -1258,10 +1353,11 @@ static void spool_fake_active_in (char *dir)
 
    p = Spool_Buf + strlen(Spool_Buf);
 
-   if (dir != NULL)
+   if ((dir != NULL) &&
+       (strlen (dir) < sizeof (Spool_Buf) - (size_t) (Spool_Buf - p + 2)))
      {
 	*p = SLRN_PATH_SLASH_CHAR;
-	strcpy (p + 1, dir);
+	strcpy (p + 1, dir); /* safe */
      }
 
    if ((2 != slrn_file_exists (Spool_Buf))
@@ -1285,9 +1381,12 @@ static void spool_fake_active_in (char *dir)
      {
 	tmp->len = strlen (dir);
 
-	p = Spool_nBuf + strlen(Spool_nBuf);
-	if (p != Spool_nBuf) *p++ = '.';
-	strcpy (p, dir);
+	p = Spool_nBuf + strlen (Spool_nBuf);
+	if (strlen (dir) < sizeof (Spool_nBuf) - (size_t) (Spool_nBuf - p + 2))
+	  {
+	     if (p != Spool_nBuf) *p++ = '.';
+	     strcpy (p, dir); /* safe */
+	  }
      }
 
    Spool_Head = tmp;
@@ -1315,7 +1414,7 @@ static void spool_fake_active_out (void)
 
 static int spool_fake_active (char *path)
 {
-   strcpy(Spool_Buf, path);
+   slrn_strncpy (Spool_Buf, path, sizeof (Spool_Buf));
    *Spool_nBuf='\0';
    Spool_Head=NULL;
    spool_fake_active_in (NULL);
@@ -1374,12 +1473,13 @@ static int spool_fakeactive_read_line(char *line, int len)
    if (Spool_fakeactive_newsgroups)
      {
       /* newsgroups: alt.foo A group about foo */
-	sprintf(line,"%s ?\n",Spool_nBuf); /* !HACK! check len */
+	slrn_snprintf (line, len, "%s ?\n", Spool_nBuf);
      }
    else
      {
       /* active: alt.guitar 0000055382 0000055345 y */
-	sprintf(line,"%s %ld %ld y\n",Spool_nBuf,Spool_Head->hi,Spool_Head->lo); /* !HACK! check len */
+	slrn_snprintf (line, len, "%s %ld %ld y\n", Spool_nBuf, 
+		       Spool_Head->hi, Spool_Head->lo);
      }
    spool_fake_active_out();
    return 1;
@@ -1390,6 +1490,17 @@ static void spool_reset (void)
    spool_fclose_local ();
 }
 
+static unsigned int spool_get_bytes (int clear)
+{
+   unsigned int temp;
+   
+   temp = bytes_read;
+   if (clear)
+     bytes_read = 0;
+   
+   return temp;
+}
+
 char *Slrn_Inn_Root;
 char *Slrn_Spool_Root;
 char *Slrn_Nov_Root;
@@ -1402,6 +1513,7 @@ int Slrn_Spool_Check_Up_On_Nov;
 static int spool_init_objects (void)
 {
    Spool_Server_Obj.sv_select_group = spool_select_group;
+   Spool_Server_Obj.sv_current_group = spool_current_group;
    Spool_Server_Obj.sv_read_line = spool_read_line;
    Spool_Server_Obj.sv_close = spool_close_server;
    Spool_Server_Obj.sv_reset = spool_reset;
@@ -1409,16 +1521,20 @@ static int spool_init_objects (void)
    Spool_Server_Obj.sv_select_article = spool_select_article;
    Spool_Server_Obj.sv_put_server_cmd = spool_put_server_cmd;
    Spool_Server_Obj.sv_xpat_cmd = spool_xpat_cmd;
-   Spool_Server_Obj.sv_xhdr_command = spool_xhdr_command;
+   Spool_Server_Obj.sv_xhdr_command = spool_one_xhdr_command;
    Spool_Server_Obj.sv_xgtitle_cmd = spool_xgtitle_cmd;
    Spool_Server_Obj.sv_has_cmd = spool_has_cmd;
    Spool_Server_Obj.sv_list_newsgroups = spool_list_newsgroups;
    Spool_Server_Obj.sv_list_active = spool_list_active;
+   Spool_Server_Obj.sv_send_authinfo = spool_send_authinfo;
 
+   Spool_Server_Obj.sv_has_xhdr = 1;
    Spool_Server_Obj.sv_has_xover = 0;
    Spool_Server_Obj.sv_nntp_xover = spool_nntp_xover;
+   Spool_Server_Obj.sv_nntp_xhdr = spool_xhdr_command;
    Spool_Server_Obj.sv_nntp_head = spool_nntp_head;
    Spool_Server_Obj.sv_nntp_next = spool_nntp_next;
+   Spool_Server_Obj.sv_nntp_bytes = spool_get_bytes;
 
    Slrn_Inn_Root = slrn_safe_strmalloc (SLRN_SPOOL_INNROOT);
    Slrn_Spool_Root = slrn_safe_strmalloc (SLRN_SPOOL_ROOT);
