@@ -3,6 +3,7 @@
  This file is part of SLRN.
 
  Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
+ Copyright (c) 2001 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -71,6 +72,9 @@ static int color_fun (int, SLcmd_Cmd_Table_Type *);
 static int mono_fun (int, SLcmd_Cmd_Table_Type *);
 static int user_data_fun (int, SLcmd_Cmd_Table_Type *);
 static int ignore_quote_fun (int, SLcmd_Cmd_Table_Type *);
+static int strip_re_fun (int, SLcmd_Cmd_Table_Type *);
+static int strip_sig_fun (int, SLcmd_Cmd_Table_Type *);
+static int strip_was_fun (int, SLcmd_Cmd_Table_Type *);
 static int autobaud_fun (int, SLcmd_Cmd_Table_Type *);
 static int set_variable_fun (int, SLcmd_Cmd_Table_Type *);
 static int nnrp_fun (int, SLcmd_Cmd_Table_Type *);
@@ -79,6 +83,8 @@ static int interpret_fun (int, SLcmd_Cmd_Table_Type *);
 static int include_file_fun (int, SLcmd_Cmd_Table_Type *);
 static int set_header_format_fun (int, SLcmd_Cmd_Table_Type *);
 static int set_visible_headers_fun (int, SLcmd_Cmd_Table_Type *);
+static int set_comp_charsets_fun (int, SLcmd_Cmd_Table_Type *);
+static int set_posting_host (int, SLcmd_Cmd_Table_Type *);
 
 /*}}}*/
 /*{{{ Static Global Variables */
@@ -99,22 +105,28 @@ static SLcmd_Cmd_Type Slrn_Startup_File_Cmds[] = /*{{{*/
      {color_fun, "color", "SSS"},
      {mono_fun, "mono", "SSsss"},
      {set_variable_fun, "set", "SG"},
-     {user_data_fun, "hostname", "S"},
      {nnrp_fun, "nnrpaccess", "SSS" },
-#define SLRN_MAX_QUOTE_REGEXP 5
+#define SLRN_MAX_REGEXP 5
+   /* used for ignore_quotes and strip_*_regexp */
      {ignore_quote_fun, "ignore_quotes", "Sssss"},
+     {strip_re_fun, "strip_re_regexp", "Sssss"},
+     {strip_sig_fun, "strip_sig_regexp", "Sssss"},
+     {strip_was_fun, "strip_was_regexp", "Sssss"},
      {autobaud_fun, "autobaud", ""},
      {grouplens_fun, "grouplens_add", "S"},
      {interpret_fun, "interpret", "S"},
      {include_file_fun, "include", "S"},
      {set_header_format_fun, "header_display_format", "IS"},
-     {user_data_fun, "scorefile", "S"},
      {set_visible_headers_fun, "visible_headers", "S"},
+     {set_comp_charsets_fun, "compatible_charsets", "S"},
+     {set_posting_host, "posting_host", "S"},
 
    /* The following are considered obsolete */
+     {user_data_fun, "hostname", "S"},
      {user_data_fun, "username", "S"},
      {user_data_fun, "replyto", "S"},
      {user_data_fun, "organization", "S"},
+     {user_data_fun, "scorefile", "S"},
      {user_data_fun, "signature", "S"},
      {user_data_fun, "realname", "S"},
      {user_data_fun, "followup", "S"},
@@ -132,7 +144,10 @@ static SLcmd_Cmd_Type Slrn_Startup_File_Cmds[] = /*{{{*/
 /*}}}*/
 /*{{{ Public Global Variables */
 
-SLRegexp_Type *Slrn_Ignore_Quote_Regexp [SLRN_MAX_QUOTE_REGEXP + 1];
+SLRegexp_Type *Slrn_Ignore_Quote_Regexp [SLRN_MAX_REGEXP + 1];
+SLRegexp_Type *Slrn_Strip_Re_Regexp [SLRN_MAX_REGEXP + 1];
+SLRegexp_Type *Slrn_Strip_Sig_Regexp [SLRN_MAX_REGEXP + 1];
+SLRegexp_Type *Slrn_Strip_Was_Regexp [SLRN_MAX_REGEXP + 1];
 int Slrn_Autobaud = 0;
 char *Slrn_Score_File;
 int Slrn_Scroll_By_Page;
@@ -260,38 +275,61 @@ static SLRegexp_Type *compile_quote_regexp (char *str) /*{{{*/
 }
 
 /*}}}*/
-static int ignore_quote_fun (int argc, SLcmd_Cmd_Table_Type *table) /*{{{*/
+static void generic_regexp_fun (int argc, SLcmd_Cmd_Table_Type *cmd_table,
+				SLRegexp_Type **regexp_table) /*{{{*/
 {
    unsigned int i;
    SLRegexp_Type *r;
    
-   if (argc > SLRN_MAX_QUOTE_REGEXP + 1)
+   if (argc > SLRN_MAX_REGEXP + 1)
      {
-	slrn_exit_error ("%s: line %d:\n%sToo many expressions specified.",
+	slrn_exit_error ("%s: line %d:\nToo many expressions specified.",
 			 This_File, This_Line_Num, This_Line);
      }
    
-   for (i = 0; i < SLRN_MAX_QUOTE_REGEXP; i++)
+   for (i = 0; i < SLRN_MAX_REGEXP; i++)
      {
-	r = Slrn_Ignore_Quote_Regexp[i];
+	r = regexp_table[i];
 	if (r != NULL)
 	  {
 	     slrn_free ((char *) r->buf);
 	     SLFREE (r);
-	     Slrn_Ignore_Quote_Regexp [i] = NULL;
+	     regexp_table[i] = NULL;
 	  }
      }
-	
+   
    for (i = 1; i < (unsigned int) argc; i++)
      {
-	Slrn_Ignore_Quote_Regexp[i - 1] 
-	  = compile_quote_regexp (table->string_args[i]);
+	regexp_table[i-1] = compile_quote_regexp (cmd_table->string_args[i]);
      }
-   return 0;
 }
 
 /*}}}*/
 
+static int ignore_quote_fun (int argc, SLcmd_Cmd_Table_Type *table)
+{
+   generic_regexp_fun (argc, table, Slrn_Ignore_Quote_Regexp);
+   return 0;
+}
+
+static int strip_re_fun (int argc, SLcmd_Cmd_Table_Type *table)
+{
+   generic_regexp_fun (argc, table, Slrn_Strip_Re_Regexp);
+   return 0;
+}
+
+static int strip_sig_fun (int argc, SLcmd_Cmd_Table_Type *table)
+{
+   generic_regexp_fun (argc, table, Slrn_Strip_Sig_Regexp);
+   return 0;
+}
+
+static int strip_was_fun (int argc, SLcmd_Cmd_Table_Type *table)
+{
+   generic_regexp_fun (argc, table, Slrn_Strip_Was_Regexp);
+   return 0;
+}
+
 static int grouplens_fun (int argc, SLcmd_Cmd_Table_Type *table) /*{{{*/
 {
    (void) argc;
@@ -318,6 +356,16 @@ static int set_header_format_fun (int argc, SLcmd_Cmd_Table_Type *table)
    return slrn_set_header_format (table->int_args[1], table->string_args[2]);
 }
 
+static int set_comp_charsets_fun (int argc, SLcmd_Cmd_Table_Type *table)
+{
+   (void) argc;
+#if SLRN_HAS_MIME
+   return slrn_set_compatible_charsets (table->string_args[1]);
+#else
+   return -1;
+#endif
+}
+
 /*{{{ Setting/Getting Variable Functions */
 
 typedef struct /*{{{*/
@@ -337,7 +385,7 @@ static Set_Int_Type Int_Things_To_Set [] = /*{{{*/
      {"emphasized_text_mask",	&Slrn_Emphasized_Text_Mask},
      {"emphasized_text_mode",	&Slrn_Emphasized_Text_Mode},
      {"process_verbatim_marks", &Slrn_Process_Verbatim_Marks},
-     {"process_verbatum_marks", &Slrn_Process_Verbatim_Marks},
+     {"process_verbatum_marks", &Slrn_Process_Verbatim_Marks}, /* FIXME */
 
      {"use_flow_control",	&Slrn_Use_Flow_Control},
      {"abort_unmodified_edits", &Slrn_Abort_Unmodified},
@@ -352,22 +400,23 @@ static Set_Int_Type Int_Things_To_Set [] = /*{{{*/
      {"ignore_signature", &Slrn_Sig_Is_End_Of_Article},
      {"reject_long_lines", &Slrn_Reject_Long_Lines},
      {"generate_message_id", &Slrn_Generate_Message_Id},
+#if SLRN_HAS_STRICT_FROM
+     {"generate_email_from", NULL},
+#else
+     {"generate_email_from", &Slrn_Generate_Email_From},
+#endif
      {"display_cursor_bar", &Slrn_Display_Cursor_Bar},
 #if SLRN_HAS_NNTP_SUPPORT
-     {"query_reconnect", &Slrn_Query_Reconnect},
+     {"broken_xref", &Slrn_Broken_Xref},
 #else
-     {"query_reconnect", NULL},
-#endif
-#if SLRN_HAS_SORT_BY_SCORE
-     {"display_score", &Slrn_Display_Score},
-#else
-     {"display_score", NULL},
+     {"broken_xref", NULL},
 #endif
+     {"color_by_score", &Slrn_Color_By_Score},
+     {"highlight_unread_subjects", &Slrn_Highlight_Unread},
      {"use_xgtitle", &Slrn_Use_Xgtitle},
      {"show_article", &Slrn_Startup_With_Article},
-     {"author_display", NULL},
-     {"display_author_realname", NULL},
      {"show_descriptions", &Slrn_Group_Display_Descriptions},
+     {"smart_quote", &Slrn_Smart_Quote},
      {"no_backups", &Slrn_No_Backups},
      {"beep", &SLtt_Ignore_Beep},
      {"unsubscribe_new_groups", &Slrn_Unsubscribe_New_Groups},
@@ -382,7 +431,10 @@ static Set_Int_Type Int_Things_To_Set [] = /*{{{*/
      {"sorting_method", &Slrn_Sorting_Mode},
      {"uncollapse_threads", &Slrn_Threads_Visible},
      {"read_active", &Slrn_List_Active_File},
+     {"prefer_head", &Slrn_Prefer_Head},
+#if SLRN_HAS_MIME
      {"use_metamail", &Slrn_Use_Meta_Mail},
+#endif
      {"group_dsc_start_column", &Slrn_Group_Description_Column},
      {"lines_per_update", &Slrn_Reads_Per_Update},
      {"min_high_score", &Slrn_High_Score_Min},
@@ -392,16 +444,18 @@ static Set_Int_Type Int_Things_To_Set [] = /*{{{*/
 #if !defined(IBMPC_SYSTEM)
      {"use_blink", &SLtt_Blink_Mode},
 #endif
+     {"warn_followup_to", &Slrn_Warn_Followup_To},
      {"wrap_flags", &Slrn_Wrap_Mode},
+     {"wrap_method", &Slrn_Wrap_Method},
      {"write_newsrc_flags", &Slrn_Write_Newsrc_Flags},
      {"query_read_group_cutoff", &Slrn_Query_Group_Cutoff},
-     {"prompt_next_group", &Slrn_Prompt_Next_Group},
      {"use_header_numbers", &Slrn_Use_Header_Numbers},
+     {"use_localtime", &Slrn_Use_Localtime},
 #if SLRN_HAS_SPOILERS
      {"spoiler_char", &Slrn_Spoiler_Char},
      {"spoiler_display_mode", &Slrn_Spoiler_Display_Mode},
 #else
-     {"spoiler_start_new_page", NULL},
+     {"spoiler_display_mode", NULL},
      {"spoiler_char", NULL},
 #endif
 #if SLRN_HAS_MIME
@@ -427,15 +481,18 @@ static Set_Int_Type Int_Things_To_Set [] = /*{{{*/
 #else
      {"use_slrnpull", NULL},
 #endif
-#if SLRN_HAS_TILDE_FEATURE
      {"use_tilde", &Slrn_Use_Tildes},
-#else
-     {"use_tilde", NULL},
-#endif
 #if SLRN_HAS_SPOOL_SUPPORT
      {"spool_check_up_on_nov", &Slrn_Spool_Check_Up_On_Nov},
 #else
      {"spool_check_up_on_nov", NULL},
+#endif
+#if 1 /* FIXME: These will be removed before 1.0 */
+     {"author_display", NULL},
+     {"display_author_realname", NULL},
+     {"prompt_next_group", NULL},
+     {"query_reconnect", NULL},
+     {"display_score", &Slrn_Display_Score},
 #endif
      {NULL, NULL}
 };
@@ -454,19 +511,28 @@ Set_String_Type;
 static Set_String_Type String_Things_To_Set [] = /*{{{*/
 {
      {"failed_posts_file", &Slrn_Failed_Post_Filename},
+#if ! SLRN_HAS_STRICT_FROM
      {"hostname", &Slrn_User_Info.hostname},
+     {"realname", &Slrn_User_Info.realname},
+     {"username", &Slrn_User_Info.username},
+#else
+     {"hostname", NULL},
+     {"realname", NULL},
+     {"username", NULL},
+#endif
      {"art_help_line", &Slrn_Art_Help_Line},
      {"header_help_line", &Slrn_Header_Help_Line},
      {"group_help_line", &Slrn_Group_Help_Line},
      {"quote_string", &Slrn_Quote_String},
-     {"realname", &Slrn_User_Info.realname},
-     {"username", &Slrn_User_Info.username},
      {"replyto", &Slrn_User_Info.replyto},
      {"organization", &Slrn_User_Info.org},
      {"followup", &Slrn_User_Info.followup_string},   /* obsolete */
      {"followup_string", &Slrn_User_Info.followup_string},
      {"reply_string", &Slrn_User_Info.reply_string},
-     {"cc_followup_string", &Slrn_Courtesy_CC_Message},
+     {"cc_followup_string", &Slrn_CC_Followup_Message},
+     {"cc_post_string", &Slrn_CC_Post_Message},
+     {"followup_date_format", &Slrn_Followup_Date_Format},
+     {"overview_date_format", &Slrn_Overview_Date_Format},
      {"editor_command", &Slrn_Editor},
      {"post_editor_command", &Slrn_Editor_Post},
      {"score_editor_command", &Slrn_Editor_Score},
@@ -478,9 +544,11 @@ static Set_String_Type String_Things_To_Set [] = /*{{{*/
      {"save_directory", &Slrn_Save_Directory},
      {"postpone_directory", &Slrn_Postpone_Dir},
      {"signature", &Slrn_User_Info.signature},
+     {"signoff_string", &Slrn_Signoff_String},
      {"custom_headers", &Slrn_Post_Custom_Headers},
      {"followup_custom_headers", &Slrn_Followup_Custom_Headers},
      {"reply_custom_headers", &Slrn_Reply_Custom_Headers},
+     {"supersedes_custom_headers", &Slrn_Supersedes_Custom_Headers},
 #if SLRN_HAS_GROUPLENS
      {"grouplens_pseudoname", &Slrn_GroupLens_Pseudoname},
      {"grouplens_host", &Slrn_GroupLens_Host},
@@ -578,6 +646,8 @@ static Set_String_Type String_Things_To_Set [] = /*{{{*/
      {"server_object", &Server_Object},
      {"post_object", &Post_Object},
      {"printer_name", &Slrn_Printer_Name},
+     {"top_status_line", &Slrn_Top_Status_Line},
+     {"scorefile", &Slrn_Score_File},
      {NULL, NULL}
 };
 
@@ -718,6 +788,7 @@ static Color_Handle_Type Color_Handles[] = /*{{{*/
      {"description",	GROUP_DESCR_COLOR,	"magenta",	DEF_BG, 0},
      {"error",		ERROR_COLOR,		"red",		DEF_BG, SLTT_BLINK_MASK},
      {"frame", 		FRAME_COLOR,		"yellow", 	"blue", SLTT_REV_MASK},
+     {"from_myself",	FROM_MYSELF_COLOR,	"brightmagenta", DEF_BG, SLTT_BOLD_MASK},
      {"group",		GROUP_COLOR,		DEF_FG,		DEF_BG, 0},
      {"grouplens_display",GROUPLENS_DISPLAY_COLOR,DEF_FG,	DEF_BG, 0},
      {"header_name",	SLRN_HEADER_KEYWORD_COLOR,"green", 	DEF_BG, SLTT_BOLD_MASK},
@@ -727,7 +798,9 @@ static Color_Handle_Type Color_Handles[] = /*{{{*/
      {"italicstext", 	ITALICS_COLOR,		"magenta", 	DEF_BG, 0},
      {"menu",		MENU_COLOR,		"yellow",	"blue", SLTT_REV_MASK},
      {"menu_press",	MENU_PRESS_COLOR,	DEF_FG,		"yellow", 0},
+     {"neg_score",	NEG_SCORE_COLOR,	"green",	DEF_BG,	0},
      {"normal",		0,			DEF_FG,		DEF_BG, 0},
+     {"pos_score",	POS_SCORE_COLOR,	"blue",		DEF_BG,	SLTT_REV_MASK},
      {"pgpsignature",	PGP_SIGNATURE_COLOR,	"red",		DEF_BG, 0},
      {"quotes",		QUOTE_COLOR,		"red",		DEF_BG, 0},
      {"response_char",	RESPONSE_CHAR_COLOR,	"green", 	DEF_BG, SLTT_BOLD_MASK},
@@ -739,7 +812,7 @@ static Color_Handle_Type Color_Handles[] = /*{{{*/
      {"tilde",		SLRN_TILDE_COLOR,	"green",	DEF_BG, SLTT_BOLD_MASK},
      {"tree",		TREE_COLOR,		"red",		DEF_BG, 0},
      {"underlinetext", 	UNDERLINE_COLOR,	"magenta", 	DEF_BG, SLTT_ULINE_MASK},
-     {"verbatum", 	VERBATIM_COLOR,		"green", 	DEF_BG, 0},
+     {"verbatum", 	VERBATIM_COLOR,		"green", 	DEF_BG, 0}, /* FIXME */
      {"verbatim", 	VERBATIM_COLOR,		"green", 	DEF_BG, 0},
 
      {NULL, -1, NULL, NULL, 0}
@@ -747,6 +820,25 @@ static Color_Handle_Type Color_Handles[] = /*{{{*/
 
 /*}}}*/
 
+/* These are for internal use and automatically set by slrn */
+static Color_Handle_Type Auto_Color_Handles[] = /*{{{*/
+{
+     {"high_score",	UNREAD_HIGH_SCORE_COLOR,	"brightred",	DEF_BG,	SLTT_BOLD_MASK},
+     {"neg_score",	UNREAD_NEG_SCORE_COLOR,	"brightgreen",	DEF_BG,	SLTT_BOLD_MASK},
+     {"pos_score",	UNREAD_POS_SCORE_COLOR,	"brightblue",	DEF_BG, SLTT_REV_MASK | SLTT_BOLD_MASK},
+     {"subject",	UNREAD_SUBJECT_COLOR,	"white",	DEF_BG,	SLTT_BOLD_MASK},
+     {NULL, -1, NULL, NULL, 0}
+};
+
+static char *Color_Names [16] =
+{
+   "black", "blue", "green", "cyan",
+     "red", "magenta", "brown", "lightgray",
+     "gray", "brightblue", "brightgreen", "brightcyan",
+     "brightred", "brightmagenta", "yellow", "white"
+};
+/*}}}*/
+
 int slrn_set_object_color (char *name, char *fg, char *bg)
 {
    Color_Handle_Type *ct = Color_Handles;
@@ -755,7 +847,31 @@ int slrn_set_object_color (char *name, char *fg, char *bg)
      {
 	if (!strcmp (ct->name, name))
 	  {
+	     Color_Handle_Type *ac = Auto_Color_Handles;
+	     
 	     SLtt_set_color (ct->value, name, fg, bg);
+	     
+	     while (ac->name != NULL)
+	       {
+		  if (!strcmp (ac->name, name))
+		    {
+		       int i;
+		       
+		       for (i = 0; i < 16; ++i)
+			 if (!strcmp (fg, Color_Names [i]))
+			   break;
+		       
+		       if (i < 16)
+			 SLtt_set_color (ac->value, NULL,
+					 Color_Names [i | 0x8], bg);
+		       else
+			 SLtt_set_color (ac->value, NULL, fg, bg);
+		       
+		       break;
+		    }
+		  ac++;
+	       }
+	     
 	     return 0;
 	  }
 	ct++;
@@ -808,7 +924,20 @@ static int mono_fun (int argc, SLcmd_Cmd_Table_Type *table) /*{{{*/
      {
 	if (!strcmp (ct->name, what))
 	  {
+	     Color_Handle_Type *ac = Auto_Color_Handles;
+	     
 	     SLtt_set_mono (ct->value, NULL, mono_attr);
+	     
+	     while (ac->name != NULL)
+	       {
+		  if (!strcmp (ac->name, what))
+		    {
+		       SLtt_set_mono (ac->value, NULL, mono_attr | SLTT_BOLD_MASK);
+		       break;
+		    }
+		  ac++;
+	       }
+	     
 	     return 0;
 	  }
 	ct++;
@@ -830,18 +959,33 @@ static int mono_fun (int argc, SLcmd_Cmd_Table_Type *table) /*{{{*/
 
 /*{{{ User Info Data (Hostname, etc) */
 
+static int set_posting_host (int argc, SLcmd_Cmd_Table_Type *table) /*{{{*/
+{
+   char *hostname = table->string_args[1];
+   
+   (void) argc;
+   
+   if (slrn_is_fqdn (hostname))
+     {
+	slrn_free (Slrn_User_Info.posting_host);
+	if (NULL == (Slrn_User_Info.posting_host = SLmake_string (hostname)))
+	  exit_malloc_error ();
+	return 0;
+     }
+   
+   slrn_error ("%s is not a valid fully-qualified host name.", hostname);
+   return -1;
+}
+
+/*}}}*/
+
 /*----------------------------------------------------------------------*\
 * static int user_data_fun ();
  *
  * convenient mechanism to set Slrn_User_Info fields without adding
  * extra environment variables
- *
- * recognized fields
- *
- *   replyto		- alternative name for replies
- *   organization	- use double quotes if there are spaces!
- *   signature		- an alternate to ~/.signature (for news posting)
- *   hostname		- full name of the current host
+ * 
+ * FIXME: This is obsolete and will be removed before 1.0
 \*----------------------------------------------------------------------*/
 
 typedef struct /*{{{*/
@@ -856,18 +1000,21 @@ User_Info_Variable_Type;
 
 static User_Info_Variable_Type User_Info_Variables[] = /*{{{*/
 {
+#if ! SLRN_HAS_STRICT_FROM
      {"hostname", &Slrn_User_Info.hostname, 0},
-     {"scorefile", &Slrn_Score_File, 0},
-
-#define USER_INFO_OBSOLETE_STARTS_HERE 2
-   /* The following are obsolete and will not be supported in the future. */
      {"username", &Slrn_User_Info.username, 0},
      {"realname", &Slrn_User_Info.realname, 0},
+#else
+     {"hostname", NULL, 0},
+     {"username", NULL, 0},
+     {"realname", NULL, 0},
+#endif
+     {"scorefile", &Slrn_Score_File, 0},
      {"replyto", &Slrn_User_Info.replyto, 0},
      {"organization", &Slrn_User_Info.org, 0},
      {"followup", &Slrn_User_Info.followup_string, 0},
      {"signature", &Slrn_User_Info.signature, 0},
-     {"cc_followup_string", &Slrn_Courtesy_CC_Message, 0},
+     {"cc_followup_string", &Slrn_CC_Followup_Message, 0},
 #if SLRN_HAS_DECODE
      {"decode_directory", &Slrn_Decode_Directory, 0},
 #endif
@@ -903,14 +1050,13 @@ static int user_data_fun (int argc, SLcmd_Cmd_Table_Type *table) /*{{{*/
 	     else if (NULL != (ptr = u->addr))
 	       {
 		  contents = safe_malloc (n + 1);
-		  strcpy (contents, field);
+		  strcpy (contents, field); /* safe */
 		  
 		  slrn_free (*ptr);
 		  *ptr = contents;
 	       }
 
-	     if (u >= User_Info_Variables + USER_INFO_OBSOLETE_STARTS_HERE)
-	       issue_obsolete_message ();
+	     issue_obsolete_message ();
 	     
 	     return 0;
 	  }
@@ -1152,8 +1298,16 @@ void slrn_startup_initialize (void) /*{{{*/
    SLang_init_case_tables ();
    
    Slrn_Ignore_Quote_Regexp[0] = compile_quote_regexp ("^ ?[:>]");
+   Slrn_Strip_Sig_Regexp[0] = compile_quote_regexp ("^-- $");
    
    h = Color_Handles;
+   while (h->name != NULL)
+     {
+	SLtt_set_color (h->value, NULL, h->fg, h->bg);
+	SLtt_set_mono (h->value, NULL, h->mono);
+	h++;
+     }
+   h = Auto_Color_Handles;
    while (h->name != NULL)
      {
 	SLtt_set_color (h->value, NULL, h->fg, h->bg);
@@ -1207,7 +1361,7 @@ int slrn_read_startup_file (char *name) /*{{{*/
      }
    
    
-   fp = slrn_open_home_file (name, "r", file, 0);
+   fp = slrn_open_home_file (name, "r", file, sizeof (file), 0);
    if (fp == NULL) return -1;
    
    slrn_message ("Reading startup file %s.", file);
