@@ -26,7 +26,7 @@
 # define SLRN_MAX_PATH_LEN 1024
 #endif
 
-extern int slrn_dircat (char *, char *, char *);
+extern int slrn_dircat (char *, char *, char *, size_t);
 extern char *slrn_spool_dircat (char *, char *, int);
 extern int slrn_copy_file (char *, char *);
 extern int slrn_move_file (char *, char *);
@@ -37,7 +37,7 @@ extern char *slrn_basename (char *);
 extern int slrn_is_absolute_path (char *);
 #if defined(IBMPC_SYSTEM)
 extern void slrn_os2_convert_path (char *);
-extern void slrn_os2_make_fat (char *, char *, char *);
+extern void slrn_os2_make_fat (char *, size_t, char *, char *);
 #endif
 
 extern char *slrn_simple_strtok (char *, char *);
