@@ -2,6 +2,7 @@
  This file is part of SLRN.
 
  Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
+ Copyright (c) 2001 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -39,8 +40,10 @@
 #include "group.h"
 #include "art.h"
 #include "chmap.h"
+#include "snprintf.h"
 
 char *Slrn_Version = SLRN_VERSION;
+char *Slrn_Date = SLRN_RELEASE_DATE;
 
 typedef struct 
 {
@@ -51,74 +54,92 @@ Compile_Option_Type;
 
 static Compile_Option_Type Compile_Options [] =
 {
-     {"CHARACTER_MAP",		SLRN_HAS_CHARACTER_MAP},
-     {"DECODE",	     		SLRN_HAS_DECODE},
-     {"EMPHASIZED_TEXT",	SLRN_HAS_EMPHASIZED_TEXT},
-     {"END_OF_THREAD",		SLRN_HAS_END_OF_THREAD},
-     {"FAKE_REFS",   		SLRN_HAS_FAKE_REFS},
-     {"GEN_MSGID",   		SLRN_HAS_GEN_MSGID},
-     {"GROUPLENS",   		SLRN_HAS_GROUPLENS},
-     {"INEWS_SUPPORT",		SLRN_HAS_INEWS_SUPPORT},
-     {"MIME",		     	SLRN_HAS_MIME},
-     {"MSGID_CACHE", 		SLRN_HAS_MSGID_CACHE},
-     {"NNTP_SUPPORT",		SLRN_HAS_NNTP_SUPPORT},
-     {"PIPING",	     		SLRN_HAS_PIPING},
-     {"PULL_SUPPORT",		SLRN_HAS_PULL_SUPPORT},
-     {"RNLOCK",	     		SLRN_HAS_RNLOCK},
-     {"SLANG",		     	SLRN_HAS_SLANG},
-     {"SORT_BY_SCORE",		SLRN_HAS_SORT_BY_SCORE},
-     {"SSL",			SLRN_HAS_SSL_SUPPORT},
-     {"SPOILERS",    		SLRN_HAS_SPOILERS},
-     {"SPOOL_SUPPORT",		SLRN_HAS_SPOOL_SUPPORT},
-     {"TILDE_FEATURE",		SLRN_HAS_TILDE_FEATURE},
-     {"USER_AGENT",  		SLRN_HAS_USER_AGENT},
-     {"UUDEVIEW",    		SLRN_HAS_UUDEVIEW},
-     {"VERBATIM_MARKS",		SLRN_HAS_VERBATIM_MARKS},
-   
-   {NULL, 0}
+     {"nntp",			SLRN_HAS_NNTP_SUPPORT},
+     {"slrnpull",		SLRN_HAS_PULL_SUPPORT},
+     {"spool",			SLRN_HAS_SPOOL_SUPPORT},
+#define EXTERNAL_PROG_OFFSET 3
+     {"inews",			SLRN_HAS_INEWS_SUPPORT},
+     {"ssl",			SLRN_HAS_SSL_SUPPORT},
+     {"uudeview",    		SLRN_HAS_UUDEVIEW},
+#define FEATURES_OFFSET 6
+     {"charset_mapping",	SLRN_HAS_CHARACTER_MAP},
+     {"decoding",		SLRN_HAS_DECODE},
+     {"emphasized_text",	SLRN_HAS_EMPHASIZED_TEXT},
+     {"end_of_thread",		SLRN_HAS_END_OF_THREAD},
+     {"fake_refs",   		SLRN_HAS_FAKE_REFS},
+     {"gen_msgid",   		SLRN_HAS_GEN_MSGID},
+     {"grouplens",   		SLRN_HAS_GROUPLENS},
+     {"mime",		     	SLRN_HAS_MIME},
+     {"msgid_cache", 		SLRN_HAS_MSGID_CACHE},
+     {"piping",	     		SLRN_HAS_PIPING},
+     {"rnlock",	     		SLRN_HAS_RNLOCK},
+     {"slang",		     	SLRN_HAS_SLANG},
+     {"spoilers",    		SLRN_HAS_SPOILERS},
+     {"strict_from",		SLRN_HAS_STRICT_FROM},
+     {"user_agent",  		SLRN_HAS_USER_AGENT},
+     {"verbatim_marks",		SLRN_HAS_VERBATIM_MARKS},
+     {NULL, 0}
 };
 
 static void show_compile_time_options (void)
 {
    Compile_Option_Type *opt;
-   unsigned int len;
+   unsigned int len, n = 0;
 
-   fprintf (stdout, "\nslrn compile-time options:\n");
+   fputs ("\n COMPILE TIME OPTIONS:", stdout);
    
    opt = Compile_Options;
-   len = 0;
+   len = 1;
    while (opt->name != NULL)
      {
 	unsigned int dlen = strlen (opt->name) + 3;
+	
+	switch (n)
+	  {
+	   case 0:
+	     fputs ("\n  Backends:", stdout);
+	     len = 11;
+	     break;
+	   case EXTERNAL_PROG_OFFSET:
+	     fputs ("\n  External programs / libs:", stdout);
+	     len = 27;
+	     break;
+	   case FEATURES_OFFSET:
+	     fputs ("\n  Features:", stdout);
+	     len = 11;
+	     break;
+	  }
 
 	len += dlen;
 	if (len >= 80)
 	  {
-	     fputc ('\n', stdout);
-	     len = dlen;
+	     fputs ("\n   ", stdout);
+	     len = dlen + 3;
 	  }
 	fprintf (stdout, " %c%s", (opt->value ? '+' : '-'), opt->name);
-	opt++;
+	opt++; n++;
      }
    fputc ('\n', stdout);
 }
 
 
 
-static char *make_slang_version (unsigned int v)
+static char *make_slang_version (int v)
 {
 #if SLANG_VERSION >= 10307
-   (void) v;
-   return SLang_Version_String;
+   if (v == SLang_Version)
+     return SLang_Version_String;
+   else
+     return SLANG_VERSION_STRING;
 #else
-   unsigned int a, b, c;
+   int a, b, c;
    static char buf[32];
    
    a = v/10000;
    b = (v - a * 10000) / 100;
    c = v - (a * 10000) - (b * 100);
 
-   sprintf (buf, "%u.%u.%u", a, b, c);
+   slrn_snprintf (buf, sizeof (buf), "%d.%d.%d", a, b, c);
    return buf;
 #endif
 }
@@ -126,28 +147,26 @@ static char *make_slang_version (unsigned int v)
 void slrn_show_version (void) /*{{{*/
 {
    char *os;
-
    os = slrn_get_os_name ();
 
-# if defined(__DATE__) && defined(__TIME__)
-   fprintf (stdout, "Slrn Version: %s (%s %s)\n", Slrn_Version, __DATE__, __TIME__);
-#else
-   fprintf (stdout, "Slrn Version: %s\n", Slrn_Version);
+   fprintf (stdout, "Slrn %s [%s]\n", Slrn_Version, Slrn_Date);
+#if defined(PATCH_LEVEL)
+   fprintf (stdout, "\t* Note: This version is a developer preview.\n");
 #endif
    fprintf (stdout, "S-Lang Library Version: %s\n", make_slang_version (SLang_Version));
    if (SLANG_VERSION != SLang_Version)
      {
-	fprintf (stdout, "\t** Note: This program was compiled against version %s.\n",
+	fprintf (stdout, "\t* Note: This program was compiled against version %s.\n",
 		 make_slang_version (SLANG_VERSION));
      }
-   fprintf (stdout, "OS: %s\n", os);
+   fprintf (stdout, "Operating System: %s\n", os);
    
    show_compile_time_options ();
 
-   fprintf (stdout, "\nDefault server object: %s\n",
+   fprintf (stdout, " DEFAULTS:\n  Default server object:     %s\n",
 	    slrn_map_object_id_to_name (0, SLRN_DEFAULT_SERVER_OBJ));
    
-   fprintf (stdout, "Default posting mechanism: %s\n",
+   fprintf (stdout, "  Default posting mechanism: %s\n",
 	    slrn_map_object_id_to_name (1, SLRN_DEFAULT_POST_OBJ));
 
    
