@@ -2,6 +2,7 @@
  This file is part of SLRN.
 
  Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
+ Copyright (c) 2001 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -18,7 +19,15 @@
  Mass Ave, Cambridge, MA 02139, USA. 
 */
 
-#define SLRN_VERSION "0.9.6.4" /* Make sure to update version info in VMSMAKE.COM!!! */
+#define VERSION "0.9.7.0" /* Make sure to update version info in VMSMAKE.COM!!! */
+#undef PATCH_LEVEL
+#define SLRN_RELEASE_DATE "2001-03-28"
+
+#ifdef PATCH_LEVEL
+# define SLRN_VERSION (VERSION "pl" PATCH_LEVEL)
+#else  
+# define SLRN_VERSION (VERSION)
+#endif
 
 #ifndef SLRNPULL_CODE
 extern char *Slrn_Version;
