@@ -3,6 +3,7 @@
  This file is part of SLRN.
 
  Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
+ Copyright (c) 2001 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -41,48 +42,50 @@
 #ifndef SLRNPULL_CODE
 # include "group.h"
 # include "art.h"
-#endif
+# if SLRN_HAS_MIME
+#  include "mime.h"
+# endif
+#endif /* NOT SLRNPULL_CODE */
 
 #include "xover.h"
 
 #ifndef SLRNPULL_CODE
+# include "server.h"
+
 static int extract_id_from_xref (char *);
+static Slrn_Header_Line_Type *copy_add_headers (Slrn_Header_Line_Type *);
+static int Suspend_XOver_For_Kill = 0;
 #endif
 
-typedef struct /*{{{*/
-{
-   char *name;
-   unsigned int name_len;
-   char *value;
-}
+static Slrn_Header_Line_Type *Additional_Headers = NULL;
+static Slrn_Header_Line_Type *Unretrieved_Headers = NULL;
+static Slrn_Header_Line_Type *Current_Add_Header = NULL;
+#if SLRN_HAS_FAKE_REFS
+static char *In_Reply_To;
+#endif
 
-/*}}}*/
-Header_Type;
-
-#define MAX_PARSED_HEADERS 36
-static Header_Type Parsed_Headers [MAX_PARSED_HEADERS] = /*{{{*/
+static Slrn_Header_Line_Type Xover_Headers [] = /*{{{*/
 {
-#define SUBJECT_OFFSET	0
-     {"Subject", 7, NULL},
-#define FROM_OFFSET	1
-     {"From", 4, NULL},
-#define DATE_OFFSET	2
-     {"Date", 4, NULL},
-#define MSGID_OFFSET	3
-     {"Message-ID", 10, NULL},
-#define REFS_OFFSET	4
-     {"References", 10, NULL},
-#define LINES_OFFSET	5
-     {"Lines", 5, NULL},
-#define XREF_OFFSET	6
-     {"Xref", 4, NULL},
-   
+#define SUBJECT_OFFSET  0
+     {"Subject", 7, NULL, NULL},
+#define FROM_OFFSET     1
+     {"From", 4, NULL, NULL},
+#define DATE_OFFSET     2
+     {"Date", 4, NULL, NULL},
+#define MSGID_OFFSET    3
+     {"Message-ID", 10, NULL, NULL},
+#define REFS_OFFSET     4
+     {"References", 10, NULL, NULL},
+#define LINES_OFFSET    5
+     {"Lines", 5, NULL, NULL},
+#define XREF_OFFSET     6
+     {"Xref", 4, NULL, NULL},
    /* Technically, bytes is in the overview file.  However, this value
     * is not stored in slrn's article structure.
     */
-#define BYTES_OFFSET	7
-     {"Bytes", 5, NULL}
-#define NON_XOVER_HEADER_START 7
+#define BYTES_OFFSET    7
+     {"Bytes", 5, NULL, NULL}
+#define XOVER_HEADERS_MAX 8
 };
 
 /*}}}*/
@@ -92,21 +95,23 @@ static char *Malloced_Headers;
 
 static void parse_headers (void) /*{{{*/
 {
+   Slrn_Header_Line_Type *addh;
    unsigned int i;
-   unsigned int next_slot;
    char *h, ch;
    
-   for (i = 0; i < NON_XOVER_HEADER_START; i++)
-     Parsed_Headers[i].value = NULL;
+   /* reset all pointers */
    
-   for (i = NON_XOVER_HEADER_START; i < MAX_PARSED_HEADERS; i++)
-     {
-	Parsed_Headers[i].name = NULL;
-	Parsed_Headers[i].value = NULL;
-     }
+   for (i = 0; i < XOVER_HEADERS_MAX; i++)
+     Xover_Headers[i].value = NULL;
+   
+   for (addh = Additional_Headers; addh != NULL; addh = addh->next)
+     addh->value = NULL;
+   
+#if SLRN_HAS_FAKE_REFS
+   In_Reply_To = NULL;
+#endif
    
    h = Malloced_Headers;
-   next_slot = NON_XOVER_HEADER_START;
    
    while (*h != 0)
      {
@@ -123,28 +128,40 @@ static void parse_headers (void) /*{{{*/
 	len = (unsigned int) (colon - h);
 	
 	colon++;
-	if (*colon == ' ') colon++;   /* space is required to be there */
+	if (*colon == ' ') colon++;  /* space is required to be there */
 	
-	for (i = 0; i < NON_XOVER_HEADER_START; i++)
+	for (i = 0; i < XOVER_HEADERS_MAX; i++)
 	  {
-	     if ((Parsed_Headers[i].value != NULL)
-		 || (len != Parsed_Headers[i].name_len)
+	     if ((Xover_Headers[i].value != NULL)
+		 || (len != Xover_Headers[i].name_len)
 		 || (slrn_case_strcmp ((unsigned char *)h, 
-				       (unsigned char *) Parsed_Headers[i].name)))
+				       (unsigned char *) Xover_Headers[i].name)))
 	       continue;
 	     
-	     Parsed_Headers[i].value = colon;
+	     Xover_Headers[i].value = colon;
 	     break;
 	  }
 	
-	if ((i == NON_XOVER_HEADER_START) && (next_slot < MAX_PARSED_HEADERS))
-	  {
-	     /* Must not be an XOVER header.  Add it to list. */
-	     Parsed_Headers[next_slot].name = h;
-	     Parsed_Headers[next_slot].name_len = len;
-	     Parsed_Headers[next_slot].value = colon;
-	     next_slot++;
-	  }
+	for (addh = Additional_Headers; addh != NULL; addh = addh->next)
+	  if ((addh->value == NULL)
+	      && (len == addh->name_len)
+	      && (0 == slrn_case_strcmp ((unsigned char *)h,
+					 (unsigned char *) addh->name)))
+	    {
+#if SLRN_HAS_MIME
+	       if (Slrn_Use_Mime)
+		 slrn_rfc1522_decode_string (colon);
+#endif
+	       addh->value = colon;
+	       break;
+	    }
+	
+#if SLRN_HAS_FAKE_REFS
+	if ((In_Reply_To == NULL) && (len == 11) &&
+	    (0 == slrn_case_strcmp ((unsigned char *)h,
+				    (unsigned char *) "In-Reply-To")))
+	  In_Reply_To = colon;
+#endif
 	
 	/* Now skip to next header line and take care of continuation if
 	 * present.
@@ -180,27 +197,11 @@ static void parse_headers (void) /*{{{*/
 static char *fake_refs_from_inreply_to (char *buf, unsigned int buflen)
 {
    char *p, *p1;
-   Header_Type *h, *hmax;
    unsigned int len;
-
-   h = Parsed_Headers + NON_XOVER_HEADER_START;
-   hmax = Parsed_Headers + MAX_PARSED_HEADERS;
    
-   while (1)
-     {
-	if ((h == hmax)
-	    || (NULL == (p = h->name)))
-	  return NULL;
-	
-	if ((11 == h->name_len)
-	    && (0 == slrn_case_strcmp ((unsigned char *)p, (unsigned char *)"In-Reply-To")))
-	  break;
-
-	h++;
-     }
+   if (NULL == (p = In_Reply_To))
+     return NULL;
    
-   p = h->value;
-
    /* The In-Reply-To header seems to obey no well-defined format.  I have 
     * seen things like:
     *   In-Reply-To: Your message <msg-id>
@@ -236,7 +237,7 @@ static char *fake_refs_from_inreply_to (char *buf, unsigned int buflen)
    
    if (len >= buflen)
      return NULL;
-	     
+   
    strncpy (buf, p, len);
    buf [len] = '\0';
    return buf;
@@ -252,27 +253,27 @@ static int parsed_headers_to_xover (int id, Slrn_XOver_Type *xov) /*{{{*/
    char fake_ref_buf [512];
 #endif
 
-   if (NULL == (subj = Parsed_Headers[SUBJECT_OFFSET].value))
+   if (NULL == (subj = Xover_Headers[SUBJECT_OFFSET].value))
      subj = "";
-   if (NULL == (from = Parsed_Headers[FROM_OFFSET].value))
+   if (NULL == (from = Xover_Headers[FROM_OFFSET].value))
      from = "";
-   if (NULL == (date = Parsed_Headers[DATE_OFFSET].value))
+   if (NULL == (date = Xover_Headers[DATE_OFFSET].value))
      date = "";
-   if (NULL == (msgid = Parsed_Headers[MSGID_OFFSET].value))
+   if (NULL == (msgid = Xover_Headers[MSGID_OFFSET].value))
      msgid = "";
 
-   if ((NULL == (refs = Parsed_Headers[REFS_OFFSET].value))
+   if ((NULL == (refs = Xover_Headers[REFS_OFFSET].value))
 #if SLRN_HAS_FAKE_REFS
        && (NULL == (refs = fake_refs_from_inreply_to (fake_ref_buf, sizeof (fake_ref_buf))))
 #endif
        )
      refs = "";
 
-   if (NULL == (xref = Parsed_Headers[XREF_OFFSET].value))
+   if (NULL == (xref = Xover_Headers[XREF_OFFSET].value))
      xref = "";
-   if (NULL == (lines = Parsed_Headers[LINES_OFFSET].value))
+   if (NULL == (lines = Xover_Headers[LINES_OFFSET].value))
      lines = "";
-   if (NULL == (bytes = Parsed_Headers[BYTES_OFFSET].value))
+   if (NULL == (bytes = Xover_Headers[BYTES_OFFSET].value))
      bytes = "";
    
    len = strlen (subj) + strlen (from) + strlen (date) + strlen (msgid)
@@ -283,27 +284,27 @@ static int parsed_headers_to_xover (int id, Slrn_XOver_Type *xov) /*{{{*/
      return -1;
 
    xov->subject_malloced = buf;
-   strcpy (buf, subj);
+   strcpy (buf, subj); /* safe */
    buf += strlen (subj) + 1;
    
    xov->from = buf;
-   strcpy (buf, from);
+   strcpy (buf, from); /* safe */
    buf += strlen (from) + 1;
 
    xov->date = buf;
-   strcpy (buf, date);
+   strcpy (buf, date); /* safe */
    buf += strlen (date) + 1;
    
    xov->message_id = buf;
-   strcpy (buf, msgid);
+   strcpy (buf, msgid); /* safe */
    buf += strlen (msgid) + 1;
 
    xov->references = buf;
-   strcpy (buf, refs);
+   strcpy (buf, refs); /* safe */
    buf += strlen (refs) + 1;
    
    xov->xref = buf;
-   strcpy (buf, xref);
+   strcpy (buf, xref); /* safe */
    buf += strlen (xref) + 1;
 
    xov->bytes = atoi (bytes);
@@ -312,7 +313,12 @@ static int parsed_headers_to_xover (int id, Slrn_XOver_Type *xov) /*{{{*/
 #ifndef SLRNPULL_CODE
    if (id == -1)
      id = extract_id_from_xref (xov->xref);
+   
+   if (!Slrn_Server_Obj->sv_has_xover || Suspend_XOver_For_Kill)
+     xov->add_hdrs = copy_add_headers (Additional_Headers);
+   else
 #endif
+     xov->add_hdrs = NULL;
 
    xov->id = id;
 
@@ -323,21 +329,20 @@ static int parsed_headers_to_xover (int id, Slrn_XOver_Type *xov) /*{{{*/
 
 char *slrn_get_extra_xover_header (char *hdr) /*{{{*/
 {
-   Header_Type *h, *hmax;
+   Slrn_Header_Line_Type *h;
    unsigned int len;
    
-   h = Parsed_Headers + NON_XOVER_HEADER_START;
-   hmax = Parsed_Headers + MAX_PARSED_HEADERS;
+   h = Additional_Headers;
    len = strlen (hdr);
    
-   while ((h < hmax) && (h->value != NULL))
+   while (h != NULL)
      {
 	if ((len == h->name_len)
 	    && (0 == slrn_case_strcmp ((unsigned char *)h->name,
 				       (unsigned char *)hdr)))
 	  return h->value;
 	
-	h++;
+	h = h->next;
      }
    return NULL;
 }
@@ -370,6 +375,7 @@ void slrn_map_xover_to_header (Slrn_XOver_Type *xov, Slrn_Header_Type *h)
    h->refs = xov->references;
    h->lines = xov->lines;
    h->xref = xov->xref;
+   h->add_hdrs = xov->add_hdrs;
    
    /* Since the subject has been malloced, the message_id pointer can be changed.
     */
@@ -389,15 +395,78 @@ void slrn_map_xover_to_header (Slrn_XOver_Type *xov, Slrn_Header_Type *h)
 				(unsigned char *) h->msgid + strlen (h->msgid));
 }
 
+void slrn_free_additional_headers (Slrn_Header_Line_Type *h) /*{{{*/
+{
+   Slrn_Header_Line_Type *tmp;
+   while (h != NULL)
+     {
+	tmp = h->next;
+	slrn_free (h->name);
+	slrn_free ((char *)h);
+	h = tmp;
+     }
+}
+
+/*}}}*/
+
+void slrn_clear_requested_headers (void) /*{{{*/
+{
+   slrn_free_additional_headers (Additional_Headers);
+   Additional_Headers = NULL;
+   Unretrieved_Headers = NULL;
+   Current_Add_Header = NULL;
+}
+
+/*}}}*/
+
+void slrn_request_additional_header (char *hdr) /*{{{*/
+{
+   Slrn_Header_Line_Type *h, **hh;
+   unsigned int len;
+   
+   h = Additional_Headers;
+   hh = &Additional_Headers;
+   len = strlen (hdr);
+   
+   while (h != NULL)
+     {
+	if ((h->name_len == len) &&
+	    (0 == slrn_case_strcmp ((unsigned char*) hdr,
+				    (unsigned char*) h->name)))
+	  return; /* This one was already requested. */
+	hh = &h->next;
+	h = h->next;
+     }
+   
+   if (NULL == (h = (Slrn_Header_Line_Type*)
+		slrn_malloc (sizeof (Slrn_Header_Line_Type), 0, 0)))
+     return;
+   
+   if (NULL == (h->name = slrn_strmalloc (hdr, 0)))
+     {
+	slrn_free ((char *)h);
+	return;
+     }
+
+   h->name_len = len;
+   h->value = NULL;
+   h->next = NULL;
+   *hh = h;
+   
+   if (Unretrieved_Headers == NULL)
+     {
+	Unretrieved_Headers = h;
+	Current_Add_Header = h;
+     }
+}
+
+/*}}}*/
+
 #ifndef SLRNPULL_CODE
-
-# include "server.h"
-
 static int XOver_Done;
 static int XOver_Min;
 static int XOver_Max;
 static int XOver_Next;
-static int Suspend_XOver_For_Kill = 0;
 
 static int extract_id_from_xref (char *xref) /*{{{*/
 {
@@ -477,7 +546,7 @@ static char *server_read_and_malloc (void) /*{{{*/
 	     mbuf = new_mbuf;
 	  }
    
-	strcpy (mbuf + buffer_len, line);
+	strcpy (mbuf + buffer_len, line); /* safe */
 	buffer_len += len;
 	mbuf [buffer_len++] = '\n';
 	mbuf [buffer_len] = 0;
@@ -570,35 +639,35 @@ static int parse_xover_line (char *buf, Slrn_XOver_Type *xov) /*{{{*/
    if (*b) *b++ = 0;
    id = atoi (buf);
    
-   Parsed_Headers [SUBJECT_OFFSET].value = b;
+   Xover_Headers [SUBJECT_OFFSET].value = b;
    while (*b && (*b != '\t')) b++;
    if (*b) *b++ = 0;
 
-   Parsed_Headers [FROM_OFFSET].value = b;
+   Xover_Headers [FROM_OFFSET].value = b;
    while (*b && (*b != '\t')) b++;
    if (*b) *b++ = 0;
    
-   Parsed_Headers [DATE_OFFSET].value = b;
+   Xover_Headers [DATE_OFFSET].value = b;
    while (*b && (*b != '\t')) b++;
    if (*b) *b++ = 0;
    
-   Parsed_Headers [MSGID_OFFSET].value = b;
+   Xover_Headers [MSGID_OFFSET].value = b;
    while (*b && (*b != '\t')) b++;
    if (*b) *b++ = 0;
    
-   Parsed_Headers [REFS_OFFSET].value = b;
+   Xover_Headers [REFS_OFFSET].value = b;
    while (*b && (*b != '\t')) b++;
    if (*b) *b++ = 0;
 
-   Parsed_Headers [BYTES_OFFSET].value = b;
+   Xover_Headers [BYTES_OFFSET].value = b;
    while (*b && (*b != '\t')) b++;
    if (*b) *b++ = 0;
 
-   Parsed_Headers [LINES_OFFSET].value = b;
+   Xover_Headers [LINES_OFFSET].value = b;
    while (*b && (*b != '\t')) b++;
    if (*b) *b++ = 0;
 
-   Parsed_Headers [XREF_OFFSET].value = NULL;
+   Xover_Headers [XREF_OFFSET].value = NULL;
 
    /* Optional fields consist of Header: stuff.  Look for Xref: ... */
    while (*b != 0)
@@ -612,7 +681,7 @@ static int parse_xover_line (char *buf, Slrn_XOver_Type *xov) /*{{{*/
 	if (0 == slrn_case_strncmp ((unsigned char *) xb, 
 				    (unsigned char *) "Xref: ", 6))
 	  {
-	     Parsed_Headers[XREF_OFFSET].value = xb + 6;
+	     Xover_Headers[XREF_OFFSET].value = xb + 6;
 	     break;
 	  }
      }
@@ -630,7 +699,14 @@ int slrn_read_xover (Slrn_XOver_Type *xov) /*{{{*/
    int status;
    
    if (XOver_Done)
-     return 0;
+     {
+	if (!Slrn_Server_Obj->sv_has_xover || Suspend_XOver_For_Kill)
+	  {
+	     Unretrieved_Headers = NULL;
+	     Current_Add_Header = NULL;
+	  }
+	return 0;
+     }
 
    if (Slrn_Server_Obj->sv_has_xover && !Suspend_XOver_For_Kill)
      {
@@ -713,21 +789,22 @@ void slrn_close_xover (void) /*{{{*/
 }
 
 /*}}}*/
+
 void slrn_open_suspend_xover (void) /*{{{*/
 {
-   Suspend_XOver_For_Kill = 1;
+   Suspend_XOver_For_Kill |= 1;
 }
 
 /*}}}*/
 void slrn_close_suspend_xover (void) /*{{{*/
 {
-   Suspend_XOver_For_Kill = 0;
+   Suspend_XOver_For_Kill &= ~1;
 }
 
 /*}}}*/
 int slrn_xover_for_msgid (char *msgid, Slrn_XOver_Type *xov) /*{{{*/
 {
-   int id, status;
+   int id, status, retval;
    
    if ((msgid == NULL) || (*msgid == 0)) return -1;
 
@@ -741,8 +818,189 @@ int slrn_xover_for_msgid (char *msgid, Slrn_XOver_Type *xov) /*{{{*/
      }
       
    if (id == 0) id = -1;
+   retval = read_head_into_xover (id, xov);
+   if (NULL == xov->add_hdrs)
+     xov->add_hdrs = copy_add_headers (Additional_Headers);
    
-   return read_head_into_xover (id, xov);
+   return retval;
+}
+
+/*}}}*/
+
+void slrn_open_all_add_xover (void) /*{{{*/
+{
+   if (!Slrn_Server_Obj->sv_has_xover || Suspend_XOver_For_Kill)
+     Current_Add_Header = NULL;
+   else
+     Current_Add_Header = Additional_Headers;
+}
+
+/*}}}*/
+
+/* Returns -1 upon error, 0 if done, 1 upon success */
+int slrn_open_add_xover (int min, int max) /*{{{*/
+{
+   int status = -1;
+   
+   if (NULL == Current_Add_Header)
+     return 0;
+   
+   if (Slrn_Server_Obj->sv_has_xhdr == -1)
+     Slrn_Server_Obj->sv_has_xhdr = Slrn_Server_Obj->sv_has_cmd ("XHDR");
+   
+   if (Slrn_Server_Obj->sv_has_xhdr)
+     {
+	status = Slrn_Server_Obj->sv_nntp_xhdr (Current_Add_Header->name,
+						min, max);
+	if (status == OK_HEAD)
+	  {
+	     XOver_Next = XOver_Min = min;
+	     XOver_Max = max;
+	     return 1;
+	  }
+	return -1;
+     }
+   
+   /* Server does not support xhdr -- we need to get full HEADers (slow!) */
+   Suspend_XOver_For_Kill |= 2; /* quick hack */
+   status = slrn_open_xover (min, max);
+   
+   return (status == OK_XOVER) ? 1 : -1;
+}
+
+/*}}}*/
+/* Returns -1 when done and on errors, otherwise the article number */
+int slrn_read_add_xover (Slrn_Header_Line_Type **l) /*{{{*/
+{
+   static char valbuf [NNTP_BUFFER_SIZE];
+   static Slrn_Header_Line_Type hlbuf;
+   Slrn_XOver_Type xov;
+   char *b;
+   int id, status;
+   
+   if (Slrn_Server_Obj->sv_has_xhdr)
+     {
+	while (1)
+	  {
+	     status = Slrn_Server_Obj->sv_read_line (valbuf, sizeof (valbuf));
+	     if (status == -1)
+	       return -1;
+	     if (status == 0)
+	       {
+		  Current_Add_Header = Current_Add_Header->next;
+		  return -1;
+	       }
+	     
+	     b = valbuf;
+	     while (*b && (*b != ' ')) b++;
+	     if (*b) *b++ = 0;
+	     
+	     id = atoi (valbuf);
+	     if ((id >= XOver_Min) && (id <= XOver_Max))
+	       break;
+	     /* else ignore server response */
+	  }
+	
+	hlbuf.name = Current_Add_Header->name;
+	hlbuf.name_len = Current_Add_Header->name_len;
+	if (strcmp (b, "(none)")) /* inn returns this */
+	  hlbuf.value = b;
+	else
+	  hlbuf.value = NULL;
+	hlbuf.next = NULL;
+	*l = &hlbuf;
+	return id;
+     }
+   
+   /* Server does not support xhdr -- we need to get full HEADers (slow!) */
+   status = slrn_read_xover (&xov); /* This is a quick hack. */
+   *l = Current_Add_Header;
+   if (status == 1)
+     {
+	slrn_free (xov.subject_malloced);
+	slrn_free_additional_headers (xov.add_hdrs);
+	return xov.id;
+     }
+   return -1;
+}
+/*}}}*/
+
+void slrn_close_add_xover (int done) /*{{{*/
+{
+   Suspend_XOver_For_Kill &= ~2;
+   if (done)
+     Current_Add_Header = Unretrieved_Headers = NULL;
+   else
+     Current_Add_Header = Unretrieved_Headers;
+}
+
+/*}}}*/
+
+int slrn_add_xover_missing (void)
+{
+   return (Current_Add_Header != NULL);
+}
+
+static Slrn_Header_Line_Type *copy_add_headers (Slrn_Header_Line_Type *l) /*{{{*/
+{
+   Slrn_Header_Line_Type *retval = NULL, *old = NULL, *copy;
+   unsigned int len;
+   char *name, *value, *buf;
+   
+   while (NULL != l)
+     {
+	if (NULL == (copy = (Slrn_Header_Line_Type*) slrn_malloc
+		     (sizeof (Slrn_Header_Line_Type), 0, 1)))
+	  return retval;
+	
+	name = l->name;
+	if (NULL == (value = l->value))
+	  value = "";
+	
+	len = strlen (name) + strlen (value) + 2;
+	
+	if (NULL == (buf = slrn_malloc (len, 0, 1)))
+	  {
+	     slrn_free ((char *) copy);
+	     return retval;
+	  }
+	
+	copy->name = buf;
+	strcpy (buf, name); /* safe */
+	buf += strlen (name) + 1;
+	
+	copy->value = buf; 
+	strcpy (buf, value); /* safe */
+	
+	copy->name_len = l->name_len;
+	copy->next = NULL;
+
+	if (NULL != old)
+	  old->next = copy;
+	else
+	  retval = copy;
+	
+	old = copy;
+	l = l->next;
+     }
+   
+   return retval;
+}
+
+/*}}}*/
+void slrn_append_add_xover_to_header (Slrn_Header_Type *h, /*{{{*/
+				      Slrn_Header_Line_Type *l)
+{
+   Slrn_Header_Line_Type *pos;
+   
+   if (NULL != (pos = h->add_hdrs))
+     {
+	while (pos->next != NULL)
+	  pos = pos->next;
+	pos->next = copy_add_headers (l);
+     }
+   else
+     h->add_hdrs = copy_add_headers (l);
 }
 
 /*}}}*/
